@@ -1,7 +1,7 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 2012-2013, The OpenClonk Team and contributors
+ * Copyright (c) 2012-2015, The OpenClonk Team and contributors
  *
  * Distributed under the terms of the ISC license; see accompanying file
  * "COPYING" for details.
@@ -20,7 +20,8 @@
 extern "C" {
 #endif
 
-int c4s_runscript(const char * filename);
+int c4s_runfile(const char *filename);
+int c4s_runstring(const char *script);
 
 #ifdef __cplusplus
 }
