@@ -15,6 +15,8 @@ protected func Initialize()
 protected func Construction() {
 	SetProperty("MeshTransformation", Trans_Mul(Trans_Rotate(RandomX(-35,35),0,10), Trans_Scale(30)));
 	objShine = CreateObjectAbove(EnvPack_Candle_Shine, x, y + 10, -1);
+	SetLightRange(80, 60);
+	SetLightColor(FIRE_LIGHT_COLOR);
 }
 
 private func Shining()
