@@ -23,12 +23,31 @@ func SetVictim(object to_victim)
 	return true;
 }
 
+// Scenario saving
+public func SaveScenarioObject(props, ...)
+{
+	if (!inherited(props, ...)) return false;
+	props->Remove("Name");
+	if (victim) props->AddCall("Goal", this, "SetVictim", victim);
+	return true;
+}
+
 public func IsFulfilled()
 {
 	if (!victim) return true; // pushed out of landscape?
 	return !victim->GetAlive();
 }
 
+public func GetDescription(int plr)
+{
+	var message;
+	if (IsFulfilled())
+		message = "$MsgGoalFulfilled$";		
+	else
+		message = "$MsgGoalUnFulfilled$";
+	return message;
+}
+
 public func Activate(int byplr)
 {
 	if (IsFulfilled())
