@@ -97,9 +97,9 @@ func InitializeObjects()
 	g_king.MaxEnergy = 200000;
 	g_king->DoEnergy(150);
 	g_king.StaticSaveVar = "g_king";
-	S2AI->AddAI(g_king);
-	S2AI->SetGuardRange(g_king, 1185, 342, 415, 136);
-	S2AI->SetEncounterCB(g_king, "EncounterKing");
+	AI->AddAI(g_king);
+	AI->SetGuardRange(g_king, 1185, 342, 415, 136);
+	AI->SetEncounterCB(g_king, "EncounterKing");
 
 	var Goal_Assassination001 = CreateObject(Goal_Assassination);
 	Goal_Assassination001->SetVictim(g_king);
@@ -151,8 +151,10 @@ func InitializeObjects()
 	StoneDoor001->SetComDir(COMD_Down);
 	var StoneDoor002 = CreateObject(StoneDoor, 1348, 508);
 	StoneDoor002->SetComDir(COMD_Down);
+	StoneDoor002->DoDamage(170);
 	var StoneDoor003 = CreateObject(StoneDoor, 1347, 412);
 	StoneDoor003->SetComDir(COMD_Down);
+	StoneDoor003->DoDamage(170);
 
 	var SpinWheel001 = CreateObjectAbove(SpinWheel, 961, 672);
 	SpinWheel001->SetStoneDoor(StoneDoor001);
@@ -198,52 +200,54 @@ func InitializeObjects()
 
 	var Clonk001 = CreateObjectAbove(Clonk, 673, 887);
 	Clonk001->SetName("Horst");
-	S2AI->AddAI(Clonk001);
-	S2AI->SetHome(Clonk001, 670, 878, DIR_Left);
-	S2AI->SetGuardRange(Clonk001, 400, 800, 500, 150);
-	S2AI->SetAllyAlertRange(Clonk001, 60);
-	S2AI->SetEncounterCB(Clonk001, "EncounterOutpost");
+	AI->AddAI(Clonk001);
+	AI->SetHome(Clonk001, 670, 878, DIR_Left);
+	AI->SetGuardRange(Clonk001, 400, 800, 500, 150);
+	AI->SetAllyAlertRange(Clonk001, 60);
+	AI->SetEncounterCB(Clonk001, "EncounterOutpost");
 	Clonk001->SetDir(DIR_Left);
 	var Clonk002 = CreateObjectAbove(Clonk, 710, 886);
 	Clonk002->SetName("Hanniball");
-	S2AI->AddAI(Clonk002);
-	S2AI->SetHome(Clonk002, 709, 877, DIR_Left);
-	S2AI->SetGuardRange(Clonk002, 634, 809, 233, 104);
-	S2AI->SetAllyAlertRange(Clonk002, 60);
+	AI->AddAI(Clonk002);
+	AI->SetHome(Clonk002, 709, 877, DIR_Left);
+	AI->SetGuardRange(Clonk002, 634, 809, 233, 104);
+	AI->SetAllyAlertRange(Clonk002, 60);
 	Clonk002->SetDir(DIR_Left);
 	var Clonk003 = CreateObjectAbove(Clonk, 781, 670);
 	Clonk003->SetDir(DIR_Right);
 	Clonk003->SetName("Twonky");
-	S2AI->AddAI(Clonk003);
-	S2AI->SetHome(Clonk003, 781, 663, DIR_Right);
-	S2AI->SetGuardRange(Clonk003, 481, 511, 600, 300);
+	AI->AddAI(Clonk003);
+	AI->SetHome(Clonk003, 781, 663, DIR_Right);
+	AI->SetGuardRange(Clonk003, 481, 511, 600, 300);
 	var Clonk004 = CreateObjectAbove(Clonk, 1010, 670);
 	Clonk004->SetDir(DIR_Right);
 	Clonk004->SetName("Sven");
-	S2AI->AddAI(Clonk004);
-	S2AI->SetHome(Clonk004, 1010, 663, DIR_Right);
-	S2AI->SetGuardRange(Clonk004, 773, 619, 501, 78);
+	AI->AddAI(Clonk004);
+	AI->SetHome(Clonk004, 1010, 663, DIR_Right);
+	AI->SetGuardRange(Clonk004, 773, 619, 501, 78);
 	var Clonk005 = CreateObjectAbove(Clonk, 984, 671);
 	Clonk005->SetDir(DIR_Right);
 	Clonk005->SetName("Luki");
-	S2AI->AddAI(Clonk005);
-	S2AI->SetHome(Clonk005, 985, 663, DIR_Right);
-	S2AI->SetGuardRange(Clonk005, 773, 621, 596, 146);
+	AI->AddAI(Clonk005);
+	AI->SetHome(Clonk005, 985, 663, DIR_Right);
+	AI->SetGuardRange(Clonk005, 773, 621, 596, 146);
 	var Clonk006 = CreateObjectAbove(Clonk, 1372, 1244);
 	Clonk006->SetColor(0xffff0000);
 	Clonk006->SetName("Anna");
-	S2AI->AddAI(Clonk006);
-	S2AI->SetHome(Clonk006, 1370, 1237, DIR_Left);
-	S2AI->SetGuardRange(Clonk006, 1150, 1140, 320, 150);
-	S2AI->SetAllyAlertRange(Clonk006, 170);
+	Clonk006->SetSkin(1);
+	AI->AddAI(Clonk006);
+	AI->SetHome(Clonk006, 1370, 1237, DIR_Left);
+	AI->SetGuardRange(Clonk006, 1150, 1140, 320, 150);
+	AI->SetAllyAlertRange(Clonk006, 170);
 	Clonk006->SetDir(DIR_Left);
 	var Clonk007 = CreateObjectAbove(Clonk, 1448, 1246);
 	Clonk007->SetColor(0xffff0000);
 	Clonk007->SetName("Cindy");
-	S2AI->AddAI(Clonk007);
-	S2AI->SetGuardRange(Clonk007, 1150, 1140, 320, 150);
-	S2AI->SetAllyAlertRange(Clonk007, 170);
-	S2AI->SetEncounterCB(Clonk007, "EncounterCave");
+	Clonk007->SetSkin(1);
+	AI->AddAI(Clonk007);
+	AI->SetGuardRange(Clonk007, 1150, 1140, 320, 150);
+	AI->SetAllyAlertRange(Clonk007, 170);
+	AI->SetEncounterCB(Clonk007, "EncounterCave");
 	Clonk007->SetDir(DIR_Left);
 	g_farmer = CreateObjectAbove(Clonk, 307, 1167);
 	g_farmer->SetColor(0xff0000);
@@ -254,38 +258,40 @@ func InitializeObjects()
 	var Clonk008 = CreateObjectAbove(Clonk, 1197, 550);
 	Clonk008->SetDir(DIR_Right);
 	Clonk008->SetName("Sabrina");
-	S2AI->AddAI(Clonk008);
-	S2AI->SetHome(Clonk008, 1196, 542, DIR_Right);
-	S2AI->SetGuardRange(Clonk008, 955, 472, 415, 299);
+	Clonk008->SetSkin(1);
+	AI->AddAI(Clonk008);
+	AI->SetHome(Clonk008, 1196, 542, DIR_Right);
+	AI->SetGuardRange(Clonk008, 955, 472, 415, 299);
 	var Clonk009 = CreateObjectAbove(Clonk, 1265, 551);
 	Clonk009->SetName("Laura");
-	S2AI->AddAI(Clonk009);
-	S2AI->SetHome(Clonk009, 1266, 541, DIR_Left);
-	S2AI->SetGuardRange(Clonk009, 966, 391, 383, 283);
+	Clonk009->SetSkin(1);
+	AI->AddAI(Clonk009);
+	AI->SetHome(Clonk009, 1266, 541, DIR_Left);
+	AI->SetGuardRange(Clonk009, 966, 391, 383, 283);
 	Clonk009->SetDir(DIR_Left);
 	var Clonk010 = CreateObjectAbove(Clonk, 1286, 470);
 	Clonk010->SetDir(DIR_Right);
-	S2AI->AddAI(Clonk010);
-	S2AI->SetHome(Clonk010, 1287, 464, DIR_Right);
-	S2AI->SetGuardRange(Clonk010, 987, 312, 367, 313);
+	AI->AddAI(Clonk010);
+	AI->SetHome(Clonk010, 1287, 464, DIR_Right);
+	AI->SetGuardRange(Clonk010, 987, 312, 367, 313);
 	var Clonk011 = CreateObjectAbove(Clonk, 1092, 574);
 	Clonk011->SetDir(DIR_Right);
 	Clonk011->SetName("Wolfgang");
-	S2AI->AddAI(Clonk011);
-	S2AI->SetHome(Clonk011, 1092, 567, DIR_Right);
-	S2AI->SetGuardRange(Clonk011, 792, 416, 600, 300);
+	AI->AddAI(Clonk011);
+	AI->SetHome(Clonk011, 1092, 567, DIR_Right);
+	AI->SetGuardRange(Clonk011, 792, 416, 600, 300);
 	var Clonk012 = CreateObjectAbove(Clonk, 1069, 574);
 	Clonk012->SetDir(DIR_Right);
 	Clonk012->SetName("Hans");
-	S2AI->AddAI(Clonk012);
-	S2AI->SetHome(Clonk012, 1069, 566, DIR_Right);
-	S2AI->SetGuardRange(Clonk012, 769, 416, 600, 300);
+	AI->AddAI(Clonk012);
+	AI->SetHome(Clonk012, 1069, 566, DIR_Right);
+	AI->SetGuardRange(Clonk012, 769, 416, 600, 300);
 	var Clonk013 = CreateObjectAbove(Clonk, 1018, 470);
 	Clonk013->SetDir(DIR_Right);
 	Clonk013->SetName("Joki");
-	S2AI->AddAI(Clonk013);
-	S2AI->SetHome(Clonk013, 1019, 462, DIR_Right);
-	S2AI->SetGuardRange(Clonk013, 848, 293, 498, 319);
+	AI->AddAI(Clonk013);
+	AI->SetHome(Clonk013, 1019, 462, DIR_Right);
+	AI->SetGuardRange(Clonk013, 848, 293, 498, 319);
 	var Clonk014 = CreateObjectAbove(Clonk, 285, 1182);
 	Clonk014->Kill(Clonk014, true);
 	Clonk014->SetDir(DIR_Right);
@@ -297,9 +303,12 @@ func InitializeObjects()
 	g_cannoneer = CreateObjectAbove(Clonk, 1000, 471);
 	g_cannoneer.StaticSaveVar = "g_cannoneer";
 	g_cannoneer->SetDir(DIR_Left);
+	g_cannoneer->SetName("Brutus");
+	g_cannoneer->SetAlternativeSkin("Guard");
 	var Clonk016 = CreateObjectAbove(Clonk, 1317, 334);
-	S2AI->AddAI(Clonk016);
-	S2AI->SetGuardRange(Clonk016, 978, 231, 391, 244);
+	Clonk016->SetName("Archibald");
+	AI->AddAI(Clonk016);
+	AI->SetGuardRange(Clonk016, 978, 231, 391, 244);
 	Clonk016->SetDir(DIR_Left);
 	CreateObject(Rock, 879, 1002);
 	CreateObjectAbove(Rock, 262, 1182);
@@ -544,5 +553,10 @@ func InitializeObjects()
 	Clonk011->CreateContents(Firestone, 4);
 	Chest003->CreateContents(Firestone, 3);
 	g_king->CreateContents(Firestone, 3);
+
+	CreateObject(Rule_Gravestones);
+
+	CreateObject(Rule_Restart);
+
 	return true;
 }
