@@ -10,6 +10,7 @@
 #include Library_HUDAdapter
 // standard controls
 #include Library_ClonkControl
+#include Library_CarryHeavyControl
 // manager for aiming
 #include Library_AimManager
 
@@ -81,7 +82,7 @@ protected func ControlCommand(szCommand, pTarget, iTx, iTy, pTarget2, Data)
 		}
 	}
 	// No overloaded command
-	return 0;
+	return _inherited(szCommand, pTarget, iTx, iTy, pTarget2, Data, ...);
 }
 
 
@@ -106,30 +107,25 @@ public func Redefine(idTo)
 protected func CatchBlow()
 {
 	if (GetAction() == "Dead") return;
-	if (!Random(5)) Hurt();
-}
-	
-protected func Hurt()
-{
-	if(gender == 0)
-		Sound("Hurt?");
-	else
-		Sound("FHurt?");
+	if (!Random(5)) PlaySoundHurt();
 }
 	
 protected func Grab(object pTarget, bool fGrab)
 {
-	Sound("Grab");
+	if (fGrab)
+		Sound("Clonk::Action::Grab");
+	else
+		Sound("Clonk::Action::UnGrab");
 }
 
 protected func Get()
 {
-	Sound("Grab");
+	Sound("Clonk::Action::Grab");
 }
 
 protected func Put()
 {
-	Sound("Grab");
+	Sound("Clonk::Action::Grab");
 }
 
 protected func Death(int killed_by)
@@ -147,12 +143,16 @@ protected func Death(int killed_by)
 	// Some effects on dying.
 	if (!this.silent_death)
 	{
-		if(gender == 0)
-			Sound("Die");
-		else
-			Sound("FDie");
-			
+		PlaySkinSound("Die*");
 		DeathAnnounce();
+		
+		// When killed by a team member, the other Clonk randomly plays a sound.
+		if (!Random(5) && killed_by != NO_OWNER && killed_by != GetOwner() && !Hostile(killed_by, GetOwner()))
+		{
+			var other_cursor = GetCursor(killed_by);
+			if (other_cursor)
+				other_cursor->~PlaySoundTaunt();
+		}
 	}
 	CloseEyes(1);
 	
@@ -175,6 +175,12 @@ protected func DeepBreath()
 	Sound("Breath");
 }
 
+public func Incineration()
+{
+	PlaySoundShock();
+	return _inherited(...);
+}
+
 protected func CheckStuck()
 {
 	// Prevents getting stuck on middle vertex
@@ -187,15 +193,37 @@ public func Eat(object food)
 {
 	if(GetProcedure() == "WALK")
 	{
-		DoEnergy(food->NutritionalValue());
+		Heal(food->NutritionalValue());
 		food->RemoveObject();
-		Sound("Munch?");
+		Sound("Clonk::Action::Munch?");
 		SetAction("Eat");
 	}
 }
 
+// Called when an object was dug free.
 func DigOutObject(object obj)
 {
+	// Some materials can only be transported/collected with a bucket.
+	if (obj->~IsBucketMaterial())
+	{
+		// Assume we might already be carrying a filled bucket and the object is stackable, try it!
+		var handled = obj->~TryPutInto(this);
+		if (!handled)
+		{
+			// Otherwise, force into empty buckets!
+			var empty_bucket = FindObject(Find_Container(this), Find_Func("IsBucket"), Find_Func("IsBucketEmpty"));
+			if (empty_bucket)
+			{
+				obj->Enter(empty_bucket);
+				handled = true;
+			}
+		}
+		// Those objects can only be carried with a bucket, sadly...
+		if (!handled)
+			obj->RemoveObject();
+		// Object might have been removed now.
+		return;
+	}
 	// Collect fragile objects when dug out
 	if (obj->GetDefFragile())
 		return Collect(obj,nil,nil,true);
@@ -228,8 +256,26 @@ public func IsPrey() { return true; }
 
 public func IsJumping(){return WildcardMatch(GetAction(), "*Jump*");}
 public func IsWalking(){return GetProcedure() == "WALK";}
+public func IsSwimming(){return GetProcedure() == "SWIM";}
 public func IsBridging(){return WildcardMatch(GetAction(), "Bridge*");}
 
+// Clonks act as containers for the interaction menu as long as they are alive.
+public func IsContainer() { return GetAlive(); }
+
+// You can not interact with dead Clonks.
+// This would be the place to show a death message etc.
+public func RejectInteractionMenu(object to)
+{
+	if (!GetAlive())
+		return Format("$MsgDeadClonk$", GetName());
+	return _inherited(to, ...);
+}
+
+public func GetSurroundingEntryMessage(object for_clonk)
+{
+	if (!GetAlive()) return Format("{{Clonk_Grave}} %s", Clonk_Grave->GetInscriptionForClonk(this));
+}
+
 /* Carry items on the clonk */
 
 local iHandMesh;
@@ -326,7 +372,7 @@ func DoUpdateAttach(bool sec)
 		if(HasHandAction(sec, 1))
 		{
 			iHandMesh[sec] = AttachMesh(obj, pos_hand, bone, trans);
-			PlayAnimation(closehand, 6, Anim_Const(GetAnimationLength(closehand)), Anim_Const(1000));
+			PlayAnimation(closehand, CLONK_ANIM_SLOT_Hands, Anim_Const(GetAnimationLength(closehand)), Anim_Const(1000));
 		}
 	}
 	else if(iAttachMode == CARRY_HandBack)
@@ -334,7 +380,7 @@ func DoUpdateAttach(bool sec)
 		if(HasHandAction(sec, 1))
 		{
 			iHandMesh[sec] = AttachMesh(obj, pos_hand, bone, trans);
-			PlayAnimation(closehand, 6, Anim_Const(GetAnimationLength(closehand)), Anim_Const(1000));
+			PlayAnimation(closehand, CLONK_ANIM_SLOT_Hands, Anim_Const(GetAnimationLength(closehand)), Anim_Const(1000));
 		}
 		else
 			iHandMesh[sec] = AttachMesh(obj, pos_back, bone2, trans);
@@ -342,7 +388,7 @@ func DoUpdateAttach(bool sec)
 	else if(iAttachMode == CARRY_HandAlways)
 	{
 		iHandMesh[sec] = AttachMesh(obj, pos_hand, bone, trans);
-		PlayAnimation(closehand, 6, Anim_Const(GetAnimationLength(closehand)), Anim_Const(1000));
+		PlayAnimation(closehand, CLONK_ANIM_SLOT_Hands, Anim_Const(GetAnimationLength(closehand)), Anim_Const(1000));
 	}
 	else if(iAttachMode == CARRY_Back)
 	{
@@ -354,7 +400,7 @@ func DoUpdateAttach(bool sec)
 		if(HasHandAction(sec, 1) && !sec && !special_other)
 		{
 			iHandMesh[sec] = AttachMesh(obj, "pos_tool1", bone, trans);
-			PlayAnimation("CarryArms", 6, Anim_Const(obj->~GetCarryPhase(this)), Anim_Const(1000));
+			PlayAnimation("CarryArms", CLONK_ANIM_SLOT_Hands, Anim_Const(obj->~GetCarryPhase(this)), Anim_Const(1000));
 			fBothHanded = 1;
 		}
 	}
@@ -362,7 +408,7 @@ func DoUpdateAttach(bool sec)
 	{
 		if(HasHandAction(sec, 1) && !sec)
 		{
-			PlayAnimation("CarrySpear", 6, Anim_Const(0), Anim_Const(1000));
+			PlayAnimation("CarrySpear", CLONK_ANIM_SLOT_Hands, Anim_Const(0), Anim_Const(1000));
 		}
 		else
 			iHandMesh[sec] = AttachMesh(obj, pos_back, bone2, trans);
@@ -372,7 +418,7 @@ func DoUpdateAttach(bool sec)
 		if(HasHandAction(sec, 1) && !sec)
 		{
 			iHandMesh[sec] = AttachMesh(obj, "pos_hand2", bone, trans);
-			PlayAnimation("CarryMusket", 6, Anim_Const(0), Anim_Const(1000));
+			PlayAnimation("CarryMusket", CLONK_ANIM_SLOT_Hands, Anim_Const(0), Anim_Const(1000));
 			fBothHanded = 1;
 		}
 		else
@@ -383,7 +429,7 @@ func DoUpdateAttach(bool sec)
 		if(HasHandAction(sec, 1) && !sec)
 		{
 			iHandMesh[sec] = AttachMesh(obj, "pos_hand2", bone, trans);
-			PlayAnimation("CarryCrossbow", 6, Anim_Const(0), Anim_Const(1000));
+			PlayAnimation("CarryCrossbow", CLONK_ANIM_SLOT_Hands, Anim_Const(0), Anim_Const(1000));
 			fBothHanded = 1;
 		}
 		else
@@ -459,6 +505,19 @@ public func GetHandAction()
 	return false;
 }
 
+/* Enable the Clonk to pick up stuff from its surrounding in the interaction menu */
+public func OnInteractionMenuOpen(object menu)
+{
+	_inherited(menu, ...);
+	
+	// Allow picking up stuff from the surrounding only if not in a container itself.
+	if (!Contained())
+	{
+		var surrounding = CreateObject(Helper_Surrounding);
+		surrounding->InitFor(this, menu);
+	}
+}
+
 /* Mesh transformations */
 
 local mesh_transformation_list;
@@ -489,7 +548,7 @@ func AttachBackpack()
 {
 	//Places a backpack onto the clonk
 	backpack = AttachMesh(BackpackGraphic, "skeleton_body", "main",       
-	                      Trans_Mul(Trans_Rotate(180,0,1,0), Trans_Scale(700,700,400), Trans_Translate(0,4000,1000)));
+	                      Trans_Mul(Trans_Rotate(180,1,0,0), Trans_Scale(700,400,700), Trans_Translate(4000,-1000,0)));
 }
 
 func RemoveBackpack()
@@ -603,6 +662,61 @@ func GetSkinCount() { return 4; }
 func GetSkin() { return skin; }
 func GetSkinName() { return skin_name; }
 
+
+// Returns the skin name as used to select the right sound subfolder.
+public func GetSoundSkinName()
+{
+	if (skin_name == nil) return "Adventurer";
+	return skin_name;
+}
+
+public func PlaySkinSound(string sound, ...)
+{
+	Sound(Format("Clonk::Skin::%s::%s", GetSoundSkinName(), sound), ...);
+}
+
+/*
+Helper functions to play some sounds. This are encapsulated here in case sound names change.
+*/
+public func PlaySoundConfirm(...)
+{
+	if (GetSoundSkinName() != "Farmer")
+		PlaySkinSound("Confirm*", ...);
+}
+public func PlaySoundDecline(...)
+{
+	if (GetSoundSkinName() != "Farmer")
+		PlaySkinSound("Decline*", ...);
+}
+// Doubtful sound, e.g. when trying a clearly impossible action.
+public func PlaySoundDoubt(...)
+{
+	if (GetSoundSkinName() != "Farmer")
+		PlaySkinSound("Doubt*", ...);
+}
+
+public func PlaySoundHurt(...) { PlaySkinSound("Hurt*", ...); }
+// Sound that is supposed to be funny in situations where the Clonk maybe did something "evil" like killing a teammate.
+public func PlaySoundTaunt(...)
+{
+	if (GetSoundSkinName() == "Alchemist")
+		PlaySkinSound("EvilConfirm*", ...);
+	else if (GetSoundSkinName() == "Steampunk")
+		PlaySkinSound("Laughter*", ...);
+}
+// Surprised sounds, e.g. when catching fire.
+public func PlaySoundShock(...)
+{
+	if (GetSoundSkinName() == "Steampunk" || GetSoundSkinName() == "Adventurer")
+		PlaySkinSound("Shock*", ...);
+}
+public func PlaySoundScream() { PlaySkinSound("Scream*"); }
+// General idle sounds, played when also playing an idle animation.
+public func PlaySoundIdle(...)
+{
+	if (GetSoundSkinName() == "Steampunk")
+		PlaySkinSound("Singing*", ...);
+}
 //Portrait definition of this Clonk for messages
 func GetPortrait()
 {
@@ -615,6 +729,58 @@ func SetPortrait(proplist custom_portrait)
 	return true;
 }
 
+public func CommandFailure() { return PlaySoundDoubt(); } // Callback from the engine when a command failed
+
+/* Magic */
+
+local magic_energy;
+
+public func GetMagicEnergy(int precision)
+{
+	if (precision == nil) precision = 1000;
+
+	if (precision)
+		return magic_energy / precision;
+	else
+		return magic_energy;
+}
+
+public func GetMaxMagicEnergy(int precision)
+{
+	if (precision == nil) precision = 1000;
+
+	if (precision)
+		return this.MaxMagic / precision;
+	else
+		return this.MaxMagic;
+}
+
+public func SetMagicEnergy(int val, int precision)
+{
+	if (precision == nil) precision = 1000;
+
+	magic_energy = BoundBy(val * precision, 0, this.MaxMagic);
+	this->~OnMagicEnergyChange(val);
+
+	return true;
+}
+
+// Adjusts the magic energy but only if change can be applied completely. Returns true if successful, false otherwise.
+// Use partial to bypass the completeness check
+public func DoMagicEnergy(int change, bool partial, int precision)
+{
+	if (precision == nil) precision = 1000;
+	change = change * precision;
+
+	// Can't apply fully?
+	if (!Inside(magic_energy + change, 0, this.MaxMagic) && !partial)
+		return false;
+
+	magic_energy = BoundBy(magic_energy + change, 0, this.MaxMagic);
+	this->~OnMagicEnergyChange(change);
+	return true;
+}
+
 /* Scenario saving */
 
 func SaveScenarioObject(props)
@@ -635,27 +801,26 @@ func SaveScenarioObject(props)
 	return true;
 }
 
-
 /* AI editor helper */
 
 func EditCursorSelection(...)
 {
-	var ai = S2AI->GetAI(this);
-	if (ai) Call(S2AI.EditCursorSelection, ai, ...);
+	var ai = AI->GetAI(this);
+	if (ai) Call(AI.EditCursorSelection, ai, ...);
 	return _inherited(...);
 }
 
 func EditCursorDeselection(...)
 {
-	var ai = S2AI->GetAI(this);
-	if (ai) Call(S2AI.EditCursorDeselection, ai, ...);
+	var ai = AI->GetAI(this);
+	if (ai) Call(AI.EditCursorDeselection, ai, ...);
 	return _inherited(...);
 }
 
 func AI_Add()
 {
 	// Create AI and re-select
-	S2AI->AddAI(this);
+	AI->AddAI(this);
 	EditCursorDeselection();
 	EditCursorSelection();
 	return true;
@@ -679,7 +844,7 @@ Walk = {
 	Name = "Walk",
 	Procedure = DFA_WALK,
 	Accel = 16,
-	Decel = 22,
+	Decel = 48,
 	Speed = 200,
 	Directions = 2,
 	FlipDir = 0,
@@ -735,7 +900,8 @@ Roll = {
 	Y = 0,
 	Wdt = 8,
 	Hgt = 20,
-	StartCall = "StartRoll",
+	StartCall = "OnStartRoll",
+	AbortCall = "OnAbortRoll",
 	NextAction = "Walk",
 	InLiquidAction = "Swim",
 },
@@ -844,6 +1010,7 @@ Swim = {
 //	SwimOffset = -5,
 	StartCall = "StartSwim",
 	AbortCall = "StopSwim",
+	Sound = "Clonk::Movement::DivingLoop*",
 },
 Hangle = {
 	Prototype = Action,
@@ -1009,13 +1176,17 @@ Eat = {
 	Attach=CNAT_Bottom,
 },
 };
+
 local Name = "Clonk";
+local Description = "$Description$";
 local MaxEnergy = 50000;
 local MaxBreath = 720; // Clonk can breathe for 20 seconds under water.
+local MaxMagic = 50000;
 local JumpSpeed = 400;
 local ThrowSpeed = 294;
 local NoBurnDecay = 1;
 local ContactIncinerate = 10;
+local BorderBound = C4D_Border_Sides;
 
 func Definition(def) {
 	// Set perspective
