@@ -8,18 +8,19 @@
 
 protected func Initialize()
 {
-	Incinerate();
+	Incinerate(100, GetController());
 	AddTimer("Burning");
 	return;
 }
 
 public func Burning()
 {
+	// TODO: Consume inflammable material
 	// Split the flame if it is large enough.
 	if (GetCon() > 50 && !Random(3))
 	{
 		var x = Random(15);
-		var new_flame = CreateObjectAbove(GetID(), 0, 0, GetOwner());
+		var new_flame = CreateObjectAbove(GetID());
 		new_flame->SetSpeed(x, -7);
 		new_flame->SetCon(GetCon() / 2);
 		SetSpeed(-x, -7);
@@ -40,4 +41,5 @@ func SaveScenarioObject(props)
 /*-- Properties --*/
 
 local Name = "$Name$";
-local Description = "$Description$";
\ No newline at end of file
+local Description = "$Description$";
+local Plane = 500;
\ No newline at end of file
