@@ -2,7 +2,7 @@
 
 protected func Hit()
 {
-	Sound("GeneralHit?");
+	Sound("Hits::GeneralHit?");
 }
 
 /* Eating */
@@ -10,6 +10,7 @@ protected func Hit()
 protected func ControlUse(object clonk, int iX, int iY)
 {
 	clonk->Eat(this);
+	return true;
 }
 
 public func NutritionalValue() { return 50; }
@@ -21,5 +22,4 @@ public func GetFuelNeed() { return 50; }
 local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
-local Collectible = 1;
-local Rebuy = true;
\ No newline at end of file
+local Collectible = 1;
\ No newline at end of file
