@@ -1,15 +1,22 @@
 /*-- Cooked Mushroom --*/
 
+
+protected func Construction()
+{
+	this.MeshTransformation = Trans_Rotate(RandomX(0, 359), 0, 1, 0);
+}
+
 func Hit()
 {
-	Sound("GeneralHit?");
+	Sound("Hits::GeneralHit?");
 }
 
 /* Eating */
 
-protected func ControlUse(object clonk, int iX, int iY)
+protected func ControlUse(object clonk)
 {
 	clonk->Eat(this);
+	return true;
 }
 
 public func NutritionalValue() { return 25; }
@@ -20,5 +27,4 @@ public func GetFuelNeed() { return 50; }
 local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
-local Collectible = 1;
-local Rebuy = 1;
\ No newline at end of file
+local Collectible = true;
\ No newline at end of file
