@@ -2,12 +2,11 @@
 
 protected func Hit()
 {
-	Sound("GeneralHit?");
+	Sound("Hits::GeneralHit?");
 }
 
 public func IsWindmillProduct() { return true; }
 
 local Name = "$Name$";
 local Description = "$Description$";
-local Collectible = 1;
-local Rebuy = true;
\ No newline at end of file
+local Collectible = 1;
\ No newline at end of file
