@@ -2,7 +2,7 @@
 
 protected func Hit()
 {
-	Sound("GeneralHit?");
+	Sound("Hits::GeneralHit?");
 	return 1;
 }
 
@@ -13,5 +13,4 @@ public func IsValuable(){ return true; }
 local Name = "$Name$";
 local Description = "$Description$";
 local Collectible = 1;
-local Rebuy = true;
 local Plane = 480;
\ No newline at end of file
