@@ -2,7 +2,7 @@
 
 protected func Hit()
 {
-	Sound("CrystalHit?");
+	Sound("Hits::IceHit?");
 }
 
 protected func Construction()
@@ -34,6 +34,9 @@ private func Freeze()
 	DoCon(1);
 }
 
+func IsLiquid() { return "Water"; }
+func GetLiquidAmount() { return GetCon()*2; }
+
 local Collectible = 1;
 local Name = "$Name$";
 local Description = "$Description$";
