@@ -2,7 +2,7 @@
 
 protected func Hit()
 {
-	Sound("WoodHit?");
+	Sound("Hits::Materials::Wood::WoodHit?");
 	return 1;
 }
 
@@ -23,7 +23,6 @@ public func IsSawmillProduct() { return true; }
 local Collectible = 1;
 local Name = "$Name$";
 local Description = "$Description$";
-local Rebuy = true;
 local BlastIncinerate = 5;
 local ContactIncinerate = 1;
 local Plane = 470;
\ No newline at end of file
