@@ -1,46 +1,151 @@
-/*
+/**
 	Grapple Bow
-	Author: Randrian
-
 	A crossbow which is enabled to fire grappling hooks, also has a winching system.
+	
+	@author Randrian
 */
 
-func Hit()
-{
-	Sound("GeneralHit?");
-}
-
-local fAiming;
 
+local is_aiming;
+local animation_set;
 local hook;
 local hook_attach;
 
-public func GetCarrySpecial(clonk) { if(fAiming) return "pos_hand2"; }
-public func GetCarryBone2(clonk) { return "main2"; }
-public func GetCarryMode(clonk) { if(hook && hook->Contained() == nil) return CARRY_Back; if(fAiming >= 0) return CARRY_Grappler; }
 
-/* +++++++++++ Controls ++++++++++++++ */
-
-// holding callbacks are made
-public func HoldingEnabled() { return true; }
-
-local animation_set;
-
-func Initialize()
+private func Initialize()
 {
+	// The aiming animation is done by adjusting the animation position to fit the angle.
 	animation_set = {
-		AimMode        = AIM_Position, // The aiming animation is done by adjusting the animation position to fit the angle
+		AimMode        = AIM_Position, 
 		AnimationAim   = "CrossbowAimArms",
 		AnimationShoot = nil,
 		ShootTime      = 20,
 		TurnType       = 1,
 		WalkSpeed      = 84,
 		WalkBack       = 56,
-		AimSpeed       = 20,            // the speed of aiming
+		AimSpeed       = 20,
 	};
 	OnRopeBreak();
+	return;
 }
 
+
+/*-- Animations --*/
+
+public func GetCarrySpecial(object clonk) 
+{
+	if (is_aiming)
+		return "pos_hand2";
+}
+
+public func GetCarryBone2(object clonk) { return "main2"; }
+
+public func GetCarryMode(object clonk) 
+{
+	if (hook && !hook->Contained())
+		return CARRY_Back;
+	if (is_aiming)
+		return CARRY_Grappler;
+}
+
+public func GetAnimationSet() { return animation_set; }
+
+
+/*-- Controls --*/
+
+public func HoldingEnabled() { return true; }
+
+
+public func RejectUse(object clonk)
+{
+	// Burned?
+	if (GetCon() < 100)
+		return true;
+	// Able to cut the hook? Then never reject the use.
+	if (hook->Contained() != this)
+		return false;
+	return !clonk->HasHandAction();
+}
+
+public func ControlUseStart(object clonk, int x, int y)
+{
+	// Cut rope, or otherwise remove helper object.
+	EnsureHook();
+	if (hook->Contained() != this)
+	{
+		var rope = hook->GetRope();
+		if (rope)
+		{
+			rope->DrawIn();
+			return true;
+		}
+		else
+		{
+			hook->Enter(this);
+		}
+	}
+
+	// Start aiming.
+	is_aiming = true;
+	ControlUseHolding(clonk, x, y);
+	FinishedLoading(clonk);
+	return true;
+}
+
+// Callback from the clonk when loading is finished
+public func FinishedLoading(object clonk)
+{
+	clonk->~StartAim(this);
+	return true;
+}
+
+public func ControlUseHolding(object clonk, int x, int y)
+{
+	// Update the aiming angle on mouse movement.
+	var angle = Angle(0, 0, x, y);
+	angle = Normalize(angle, -180);
+	angle = BoundBy(angle, -160, 160);
+	clonk->SetAimPosition(angle);
+	return true;
+}
+
+// Stopping says the clonk to stop with aiming (he will go on untill he has finished loading and aiming at the given angle).
+public func ControlUseStop(object clonk, int x, int y)
+{
+	clonk->StopAim();
+	return true;
+}
+
+// Callback from the clonk, when he actually has stopped aiming.
+public func FinishedAiming(object clonk, int angle)
+{
+	// Only shoot if the bow did not burn in the meantime.
+	if (GetCon() < 100)
+		return false;
+	
+	// Shoot the hook and detach the mesh from the bow.
+	EnsureHook();
+	hook->Exit();
+	hook->Launch(angle, 100, clonk, this);
+	hook_attach = nil;
+	DetachMesh(hook_attach);
+	Sound("Objects::Weapons::Bow::Shoot?");
+
+	// Open the hand to let the string go and play the fire animation.
+	PlayAnimation("Fire", 6, Anim_Linear(0, 0, GetAnimationLength("Fire"), animation_set["ShootTime"], ANIM_Hold), Anim_Const(1000));
+	clonk->StartShoot(this);
+	return true;
+}
+
+public func ControlUseCancel(object clonk, int x, int y)
+{
+	clonk->CancelAiming();
+	return true;
+}
+
+
+/*-- Bow Mechanics --*/
+
 public func SetHook(object new_hook)
 {
 	hook = new_hook;
@@ -48,14 +153,15 @@ public func SetHook(object new_hook)
 
 private func EnsureHook()
 {
-	// Create hook if it went missing
-	if(!hook) hook = CreateObjectAbove(GrappleHook, 0, 0, NO_OWNER);
+	// Create hook if it went missing.
+	if (!hook) 
+		hook = CreateObject(GrappleHook);
 	return hook;
 }
 
 public func OnRopeBreak()
 {
-	if(hook_attach)
+	if (hook_attach)
 		DetachMesh(hook_attach);
 
 	EnsureHook();
@@ -74,7 +180,7 @@ public func DrawRopeIn()
 	}
 }
 
-protected func Destruction()
+private func Destruction()
 {
 	if (hook)
 	{
@@ -84,7 +190,18 @@ protected func Destruction()
 	}
 }
 
-protected func Departure()
+private func Departure()
+{
+	if (hook)
+	{
+		var rope = hook->GetRope();
+		if (rope)
+			rope->BreakRope();
+	}
+}
+
+// If shot (e.g. by a cannon) the rope is drawn in.
+public func LaunchProjectile()
 {
 	if (hook)
 	{
@@ -92,118 +209,15 @@ protected func Departure()
 		if (rope)
 			rope->DrawIn();
 	}
+	_inherited(...);
 }
 
-public func GetAnimationSet() { return animation_set; }
 
-public func ControlUseStart(object clonk, int x, int y)
+/*-- Fire Effects --*/
+
+private func Incineration()
 {
-	// Burned?
-	if (GetCon()<100) return false;
-	// Cut rope, or otherwise remove helper object.
-	EnsureHook();
-	if (hook->Contained() != this)
-	{
-		var rope = hook->GetRope();
-		if (rope)
-		{
-			rope->DrawIn();
-		//	rope->BreakRope();
-			return true;
-		}
-		else
-		{
-			hook->Enter(this);
-		}
-	}
-
-	// if the clonk doesn't have an action where he can use it's hands do nothing
-	if(!clonk->HasHandAction())
-	{
-		return true;
-	}
-
-	// Start aiming
-	fAiming = 1;
-
-	ControlUseHolding(clonk, x, y);
-
-	FinishedLoading(clonk);
-
-	return true;
-}
-
-// Callback from the clonk when loading is finished
-public func FinishedLoading(object clonk)
-{
-	clonk->~StartAim(this);
-	return true;
-}
-
-// Update the angle on mouse movement
-public func ControlUseHolding(object clonk, int x, int y)
-{
-	// Save new angle
-	var angle = Angle(0,0,x,y);
-	angle = Normalize(angle,-180);
-
-	if(angle >  160) angle =  160;
-	if(angle < -160) angle = -160;
-
-	clonk->SetAimPosition(angle);
-
-	return true;
-}
-
-// Stopping says the clonk to stop with aiming (he will go on untill he has finished loading and aiming at the given angle)
-public func ControlUseStop(object clonk, int x, int y)
-{
-	clonk->StopAim();
-	return true;
-}
-
-// Callback from the clonk, when he actually has stopped aiming
-public func FinishedAiming(object clonk, int angle)
-{
-	if (GetCon()<100) return false;
-	EnsureHook();
-	DetachMesh(hook_attach);
-	hook_attach = nil;
-
-	hook->Exit();
-	hook->Launch(angle, 100, clonk, this);
-	DetachMesh(hook_attach);
-	Sound("BowShoot?");
-
-	// Open the hand to let the string go and play the fire animation
-	PlayAnimation("Fire", 6, Anim_Linear(0, 0, GetAnimationLength("Fire"), animation_set["ShootTime"], ANIM_Hold), Anim_Const(1000));
-	clonk->StartShoot(this);
-	return true;
-}
-
-public func ControlUseCancel(object clonk, int x, int y)
-{
-	clonk->CancelAiming();
-	return true;
-}
-
-public func OnPauseAim(object clonk)
-{
-	Reset(clonk);
-}
-
-public func OnRestartAim(object clonk)
-{
-	ControlUseStart(clonk);
-	if(fAiming) return true;
-	return false;
-}
-
-/* Destroyed by fire? Make it visible. */
-
-func Incineration()
-{
-	// GrappleBow becomes unusable on incineration.
+	// Grapple bow becomes unusable on incineration.
 	if (hook)
 	{
 		var rope = hook->GetRope();
@@ -214,10 +228,10 @@ func Incineration()
 	return _inherited(...);
 }
 
-func Extinguishing()
+private func Extinguishing()
 {
 	// If extinguished on the same frame it got incinerated, make it usable again
-	if (GetCon()>=100)
+	if (GetCon() >= 100)
 	{
 		EnsureHook();
 		SetClrModulation();
@@ -225,25 +239,34 @@ func Extinguishing()
 	return _inherited(...);
 }
 
-/* ++++++++ Animation functions ++++++++ */
 
-public func Reset(clonk)
+/*-- Animation functions --*/
+
+public func Reset(object clonk)
 {
-	fAiming = 0;
-
+	is_aiming = 0;
 	clonk->StopAnimation(clonk->GetRootAnimation(11));
 	StopAnimation(GetRootAnimation(6));
 }
 
-func IsInventorProduct() { return true; }
-
-func Definition(def) {
-	SetProperty("PictureTransformation",Trans_Mul(Trans_Translate(-700,400),Trans_Scale(1150),Trans_Rotate(180,0,1,0),Trans_Rotate(-30,-1,0,-1)),def);
+public func Hit()
+{
+	Sound("Hits::GeneralHit?");
 }
+
+public func IsInventorProduct() { return true; }
+
+
+/*-- Properties --*/
+
+public func Definition(proplist def)
+{
+	def.PictureTransformation = Trans_Mul(Trans_Translate(-2500, 1000),Trans_Scale(1800),Trans_Rotate(-60,1,-1,1), Trans_Rotate(180, 0, 1, 0));
+}
+
 local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
 local Collectible = 1;
-local Rebuy = true;
 local BlastIncinerate = 30;
-local ContactIncinerate = 0;
+local ContactIncinerate = 0;
\ No newline at end of file
