@@ -2,7 +2,7 @@
 
 #include Library_Constructor
 
-private func Hit(x, y)
+private func Hit(int x, int y)
 {
 	StonyObjectHit(x, y);
 	return 1;
@@ -10,16 +10,20 @@ private func Hit(x, y)
 
 public func GetCarryMode()	{	return CARRY_HandBack;	}
 public func GetCarryBone()	{	return "main";	}
-public func GetCarryTransform()	{	return Trans_Rotate(90,0,1,0);	}
+public func GetCarryTransform()	{	return Trans_Rotate(-90,1,0,0);	}
 
 public func IsTool() { return true; }
 public func IsToolProduct() { return true; }
 
-func Definition(def) {
-	SetProperty("PictureTransformation",Trans_Rotate(20,1,0,1),def);
+
+/*-- Properties --*/
+
+func Definition(def) 
+{
+	SetProperty("PictureTransformation", Trans_Rotate(20, 1, 0, 1), def);
 }
+
 local Collectible = 1;
 local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
-local Rebuy = true;
