@@ -8,7 +8,7 @@ func MaxStackCount() { return 4; }
 
 func ControlUseStart(object clonk, int x, int y)
 {
-	clonk->Sound("WallKitClick");
+	clonk->Sound("Objects::WallKit::Click");
 	SetPreview(clonk,x,y);
 	return true;
 }
@@ -45,8 +45,8 @@ private func CreateBridge(object clonk, int x, int y)
 	var c = Offset2BridgeCoords(clonk, x, y);
 	x = clonk->GetX(); 
 	y = clonk->GetY();
-	DrawMaterialQuad("Granite-granite", x + c.x1 - c.dxm, y + c.y1 - c.dym, x + c.x1 + c.dxp, y + c.y1 + c.dyp, x + c.x2 + c.dxp, y + c.y2 + c.dyp, x + c.x2 - c.dxm, y + c.y2 - c.dym, DMQ_Bridge);
-	clonk->Sound("WallKitLock");
+	DrawMaterialQuad(BridgeMaterial, x + c.x1 - c.dxm, y + c.y1 - c.dym, x + c.x1 + c.dxp, y + c.y1 + c.dyp, x + c.x2 + c.dxp, y + c.y2 + c.dyp, x + c.x2 - c.dxm, y + c.y2 - c.dym, DMQ_Bridge);
+	clonk->Sound("Objects::WallKit::Lock");
 	return true;
 }
 
@@ -105,7 +105,7 @@ func StopPreview(object clonk)
 
 func Hit()
 {
-	Sound("GeneralHit?");
+	Sound("Hits::GeneralHit?");
 	return true;
 }
 
@@ -119,6 +119,6 @@ local Collectible = 1;
 local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
-local Rebuy = true;
 local BridgeLength = 20;
 local BridgeThickness = 5;
+local BridgeMaterial = "Granite-granite";
