@@ -9,9 +9,12 @@
 //Uses the extra slot library
 #include Library_HasExtraSlot
 
+// Initial velocity of the bomb
+local shooting_strength = 75;
+
 func Hit()
 {
-	Sound("GeneralHit?");
+	Sound("Hits::GeneralHit?");
 }
 
 local fAiming;
@@ -21,7 +24,7 @@ public func GetCarrySpecial(clonk) { if(fAiming > 0) return "pos_hand2"; }
 public func GetCarryBone()	{	return	"main";	}
 public func GetCarryTransform()
 {
-	return Trans_Mul(Trans_Rotate(-90,0,1,0), Trans_Rotate(10,1,0,0));
+	return Trans_Mul(Trans_Rotate(90,1,0,0), Trans_Rotate(-10,0,0,1));
 }
 
 local animation_set;
@@ -60,15 +63,13 @@ local MuzzleUp; local MuzzleFront; local MuzzleDown; local MuzzleOffset;
 
 protected func HoldingEnabled() { return true; }
 
+public func RejectUse(object clonk)
+{
+	return !clonk->HasHandAction();
+}
+
 func ControlUseStart(object clonk, int x, int y)
 {
-	// if the clonk doesn't have an action where he can use it's hands do nothing
-	if(!clonk->HasHandAction())
-	{
-		holding = true;
-		return true;
-	}
-
 	// nothing in extraslot?
 	if(!Contents(0))
 	{
@@ -105,7 +106,8 @@ func ControlUseStart(object clonk, int x, int y)
 // Callback from the clonk when loading is finished
 public func FinishedLoading(object clonk)
 {
-	SetProperty("PictureTransformation",Trans_Mul(Trans_Translate(500,1000,-000),Trans_Rotate(130,0,1,0),Trans_Rotate(20,0,0,1)));
+	// Change picture to indicate being loaded.
+	this.PictureTransformation = Trans_Mul(Trans_Translate(-3000, 3000, 4000),Trans_Rotate(-45,0,0,1),Trans_Rotate(130,0,1,0));
 	loaded = true;
 	if(holding) clonk->StartAim(this);
 	return holding; // false means stop here and reset the clonk
@@ -158,14 +160,15 @@ private func FireWeapon(object clonk, int angle)
 	var IX=Sin(180-angle,MuzzleFront);
 	var IY=Cos(180-angle,MuzzleUp)+MuzzleOffset;
 
-	shot->LaunchProjectile(angle, 0, 75, IX, IY);
+	shot->LaunchProjectile(angle, 0, shooting_strength, IX, IY);
 	shot->~Fuse(true);
 	shot->SetController(clonk->GetController());
 
 	loaded = false;
-	SetProperty("PictureTransformation",Trans_Mul(Trans_Translate(1500,0,-1500),Trans_Rotate(170,0,1,0),Trans_Rotate(30,0,0,1)));
+	// Reset transformation to indicate empty grenade launcher.
+	this.PictureTransformation = this.Prototype.PictureTransformation;
 
-	Sound("GunShoot?");
+	Sound("Objects::Weapons::Musket::GunShoot?");
 
 	// Muzzle Flash & gun smoke
 	if(Abs(Normalize(angle,-180)) > 90)
@@ -190,12 +193,12 @@ public func IsArmoryProduct() { return true; }
 
 
 
-func Definition(def) {
-	SetProperty("PictureTransformation",Trans_Mul(Trans_Translate(1500,0,-1500),Trans_Rotate(170,0,1,0),Trans_Rotate(30,0,0,1)),def);
+func Definition(def)
+{
+	def.PictureTransformation = Trans_Mul(Trans_Translate(-3000, 1000, 1500),Trans_Rotate(170,0,1,0),Trans_Rotate(30,0,0,1));
 }
 
 local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
-local Collectible = 1;
-local Rebuy = true;
+local Collectible = 1;
\ No newline at end of file
