@@ -3,6 +3,8 @@ protected func Initialize()
 	SetAction("Floor");
 }
 
+local Plane = 300;
+local BorderBound = C4D_Border_Sides | C4D_Border_Top | C4D_Border_Bottom;
 local ActMap = {
 		Floor = {
 			Prototype = Action,
