@@ -3,9 +3,12 @@
 #include Library_Structure
 #include Library_Ownable
 #include Library_Producer
+#include Library_LampPost
 
 local hold_production;
 
+public func LanternPosition(id def) { return [GetCalcDir()*19,-1]; }
+
 func Construction(object creator)
 {
 	SetAction("Default");
@@ -22,13 +25,6 @@ func IsProduct(id product_id)
 private func ProductionTime(id toProduce) { return 500; }
 public func PowerNeed() { return 0; }
 
-public func NeedRawMaterial(id rawmat_id)
-{
-	if (rawmat_id->~IsFuel() || rawmat_id == Flour)
-		return true;
-	return false;
-}
-
 public func OnProductionStart(id product)
 {
 	AddEffect("Working", this, 100, 1, this);
