@@ -3,10 +3,13 @@
 #include Library_Structure
 #include Library_Ownable
 #include Library_Producer
+#include Library_LampPost
 
 local animWork;
 local meshAttach;
 
+public func LampPosition(id def) { return [GetCalcDir()*11,2]; }
+
 func Initialize()
 {
 	animWork = PlayAnimation("Working", 1, Anim_Const(0), Anim_Const(1000));
@@ -29,11 +32,6 @@ public func IsProduct(id product_id)
 private func ProductionTime(id toProduce) { return 140; }
 public func PowerNeed() { return 40; }
 
-public func NeedRawMaterial(id rawmat_id)
-{
-	return true;
-}
-
 private func FxIntWorkAnimTimer(object target, proplist effect, int timer)
 {
 	if(effect.paused == true) return 1;
@@ -79,7 +77,7 @@ public func OnProductionFinish(id product)
 }
 
 func Definition(def){
-	SetProperty("MeshTransformation", Trans_Rotate(70, 0,1,0), def);
+	SetProperty("MeshTransformation", Trans_Rotate(25, 0,1,0), def);
 	SetProperty("PictureTransformation", Trans_Rotate(65,0,1,0), def);
 }
 
