@@ -3,10 +3,13 @@
 #include Library_Structure
 #include Library_Ownable
 #include Library_Producer
+#include Library_LampPost
 
 local animWork;
 local meshAttach;
 
+public func LampPosition(id def) { return [GetCalcDir()*-6,30]; }
+
 func Initialize()
 {
 	animWork = PlayAnimation("Work", 1, Anim_Const(0), Anim_Const(1000));
@@ -29,11 +32,6 @@ public func IsProduct(id product_id)
 private func ProductionTime(id toProduce) { return 400; }
 public func PowerNeed() { return 80; }
 
-public func NeedRawMaterial(id rawmat_id)
-{
-	return true;
-}
-
 private func FxIntWorkAnimTimer(object target, proplist effect, int timer)
 {
 	if(effect.paused == true) return 1;
