@@ -8,9 +8,18 @@ local grow_stage;
 private func SeedChance() { return 1000; }
 private func SeedOffset() { return 10; }
 
-private func IsCrop() { return true; }
 private func SickleHarvesting() { return false; }
 
+protected func Construction()
+{
+	var graphic = Random(6);
+	if(graphic)
+		SetGraphics(Format("%d",graphic));
+	_inherited();
+
+	if(GetCon() < 100) SetCon(100);
+}
+
 protected func Initialize()
 {
 	SetAction("Grow");
