@@ -6,20 +6,24 @@
 */
 
 #include Library_Plant
-#include Library_Crop
 
 private func SeedChance() { return 600; }
 private func SeedArea() { return 150; }
 private func SeedAmount() { return 4; }
 private func SeedOffset() { return 10; }
 
+private func Incineration()
+{
+	SetClrModulation(RGB(48, 32, 32));
+}
+
 /*-- Initialization --*/
 
 protected func Construction()
 {
 	StartGrowth(3);
 	RootSurface();
-	this.MeshTransformation = Trans_Mul(Trans_Translate(0, 10000, 0), Trans_Rotate(RandomX(0, 359), 0, 1, 0));
+	this.MeshTransformation = Trans_Rotate(RandomX(0, 359), 0, 1, 0);
 	return _inherited(...);
 }
 
@@ -36,46 +40,21 @@ public func RootSurface()
 	return;
 }
 
-/*-- Harvesting --*/
+/*-- Eating --*/
 
-private func IsCrop() { return true; }
-private func SickleHarvesting() { return false; }
-
-public func IsHarvestable()
+protected func ControlUse(object clonk)
 {
-	// The mushroom is harvestable if it has grown a little.
-	return GetCon() >= 50;
-}
-
-public func Harvest(object clonk)
-{
-	this.Collectible = true;
-	clonk->Collect(this);
+	clonk->Eat(this);
 	return true;
 }
 
-public func IsInteractable(object clonk)
-{
-	return !this.Collectible && inherited(clonk);
-}
-
-public func GetInteractionMetaInfo(object clonk)
-{
-	return { Description = "$PickMushroom$" };
-}
-
-/*-- Eating --*/
-
-protected func ControlUse(object clonk, int iX, int iY)
-{
-	clonk->Eat(this);
-}
-
 // Nutritional value depends on the completion of the mushroom.
-public func NutritionalValue() { return 3 * GetCon() / 20; }
+public func NutritionalValue() { return GetCon() / 10; }
 
 local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
-local Collectible = false;
+local Collectible = true;
+local BlastIncinerate = 5;
+local ContactIncinerate = 1;
 local Placement = 4;
\ No newline at end of file
