@@ -3,11 +3,11 @@
 #include Library_Plant
 
 
-func Place(int amount, proplist rectangle, proplist settings)
+func Place(int amount, proplist area, proplist settings)
 {
 	var max_tries = 2 * amount;
 	var loc_area = nil;
-	if (rectangle) loc_area = Loc_InRect(rectangle);
+	if (area) loc_area = Loc_InArea(area);
 	
 	while ((amount > 0) && (--max_tries > 0))
 	{
