@@ -14,8 +14,6 @@ local propanim, turnanim;
 local throttle;
 local enginesound;
 
-local health = 30;
-
 //Rectangle defining where to look for objents contained in the gondola
 local gondola = [-20,-2,40,30];
 
@@ -42,10 +40,11 @@ protected func Initialize()
 	AddEffect("IntAirshipMovement", this, 1, 1, this);
 }
 
-public func Damage()
+public func Damage(int change, int cause, int by_player)
 {
-	if(GetDamage() >= health)
+	if (GetDamage() >= this.HitPoints)
 	{
+		SetController(by_player);
 		AirshipDeath();
 	}
 }
@@ -114,16 +113,21 @@ public func FxIntAirshipMovementTimer(object target, proplist effect, int time)
 			CreateParticle("Smoke", i, 18, 0, 0, PV_Random(36, 2 * 36), particles, 2);
 		}
 		// Fan-blade sound
-		if (!enginesound)
+		if (enginesound < 50)
 		{
-			enginesound = true;
-			Sound("FanLoop",nil,nil,nil,1);
+			// Fade pitch from -45 to 0
+			enginesound += 5;
+			Sound("Structures::FanLoop",nil,nil,nil, 1, 0, enginesound - 50);
 		}
 	}
-	else if(enginesound == true)
+	else if(enginesound)
 	{
-		Sound("FanLoop", nil, nil, nil, -1);
-		enginesound = false;
+		// Fade pitch from 0 to minimum -45, then turn off
+		enginesound = Max(enginesound - 10);
+		if (enginesound)
+			Sound("Structures::FanLoop", nil, nil, nil, 1, 0, enginesound - 50);
+		else
+			Sound("Structures::FanLoop", nil, nil, nil, -1);
 	}
 
 	// Wind movement if in the air
@@ -287,13 +291,8 @@ func ControlStop(object clonk, int control)
 
 private func AirshipPilot()
 {
-	//Looks for a clonk within the Gondola
-	var g = gondola;
-	var clonk = FindObject(Find_ID(Clonk), Find_OCF(OCF_Alive),Find_InRect(g[0],g[1],g[2],g[3]));
-	if(clonk)
-		return clonk;
-	else
-		return false;
+	// Looks for a clonk within the gondola.
+	return FindObject(Find_ID(Clonk), Find_OCF(OCF_Alive), Find_InRect(gondola[0], gondola[1], gondola[2], gondola[3]));
 }
 
 /* -- Airship Destruction --*/
@@ -310,14 +309,14 @@ func AirshipDeath()
 	else
 		burntairship->PlayAnimation("TurnRight", 10, Anim_Const(animspot), Anim_Const(1000));
 
-	//Set ruin on fire
-	burntairship->Incinerate();
+	// Set ruin on fire: set controller of the fire to the cause of the death (which is the current controller of the airship).
+	burntairship->Incinerate(100, GetController());
 
 	//Make sure engine sound is gone
-	Sound("FanLoop",nil,nil,nil,-1);
+	Sound("Structures::FanLoop",nil,nil,nil,-1);
 
 	//This object has served its purpose.
-	Explode(27);
+	Explode(20);
 }
 
 public func IsShipyardProduct() { return true; }
@@ -348,6 +347,7 @@ func Definition(def)
 local Name = "$Name$";
 local Description = "$Description$";
 local Touchable = 2;
-local Rebuy = true;
 local Plane = 500;
 local SolidMaskPlane = 275;
+local BorderBound = C4D_Border_Sides | C4D_Border_Top | C4D_Border_Bottom;
+local HitPoints = 30;
