@@ -37,22 +37,21 @@ protected func ContactRight()
 	if(Stuck() && !Random(5)) SetRDir(RandomX(-7, +7));
 }
 
-//Only one object fits in barrel
-private func MaxContentsCount() { return 1; }
+protected func RejectCollect(id def, object obj)
+{
+	// Only accept powder kegs.
+	if (def != PowderKeg)
+		return true;
+	// Only one powder keg at a time.
+	if (ContentsCount() >= 1)
+		return true;
+	return false;
+}
+
 
 /*-- Control --*/
 
 public func ControlUseStart(object clonk, int ix, int iy)
-{
-	return UseAnyStart(clonk,ix,iy,0);
-}
-
-public func ControlUseAltStart(object clonk, int ix, int iy)
-{
-	return UseAnyStart(clonk,ix,iy,1);
-}
-
-private func UseAnyStart(object clonk, int ix, int iy, int item)
 {
 	var result = CheckForKeg(clonk);
 	if (!result)
@@ -61,7 +60,7 @@ private func UseAnyStart(object clonk, int ix, int iy, int item)
 		return true;
 	}
 		
-	if (!clonk->GetHandItem(item))
+	if (!clonk->GetHandItem(0))
 	{
 		PlayerMessage(clonk->GetOwner(),"$TxtNeedsAmmo$");
 		clonk->CancelUse();
@@ -87,7 +86,7 @@ private func CheckForKeg(object clonk)
 		{
 			keg->Exit();
 			keg->Enter(this);
-			Sound("WoodHit?");
+			Sound("Hits::Materials::Wood::WoodHit?");
 		}
 		else // No keg, stop using cannon.
 		{
@@ -100,11 +99,6 @@ private func CheckForKeg(object clonk)
 
 public func HoldingEnabled() { return true; }
 
-public func ControlUseAltHolding(object clonk, int ix, int iy)
-{
-	return ControlUseHolding(clonk, ix, iy);
-}
-
 local angPrec = 1000;
 
 public func ControlUseHolding(object clonk, int ix, int iy)
@@ -116,7 +110,7 @@ public func ControlUseHolding(object clonk, int ix, int iy)
 	var iColor = RGB(255,255,255);
 	if (!Contents(0) || GetEffect("IntCooldown",this))
 		iColor = RGB(255,0,0);
-	AddTrajectory(this, GetX() + 5, GetY() + 2, Cos(r - 90 * angPrec, Fire_Velocity,angPrec), Sin(r - 90 * angPrec, Fire_Velocity,angPrec), iColor, 20);
+	Trajectory->Create(this, GetX() + 5, GetY() + 2, Cos(r - 90 * angPrec, Fire_Velocity,angPrec), Sin(r - 90 * angPrec, Fire_Velocity,angPrec));
 
 	SetCannonAngle(r);
 	
@@ -156,23 +150,12 @@ private func ConvertAngle(int angle)
 
 public func ControlUseStop(object clonk, int ix, int iy)
 {
-	return UseAnyStop(clonk,ix,iy,0);
-}
-
-public func ControlUseAltStop(object clonk, int ix, int iy)
-{
-	return UseAnyStop(clonk,ix,iy,1);
-}
-
-private func UseAnyStop(object clonk, int ix, int iy, int item)
-{
-
-	RemoveTrajectory(this);
+	Trajectory->Remove(this);
 
 	if (!CheckForKeg(clonk))
 		return true;
 
-	var projectile = clonk->GetHandItem(item);
+	var projectile = clonk->GetHandItem(0);
 	if (!projectile) // Needs a projectile
 	{
 		PlayerMessage(clonk->GetOwner(),"$TxtNeedsAmmo$");
@@ -206,15 +189,10 @@ private func UseAnyStop(object clonk, int ix, int iy, int item)
 
 public func ControlUseCancel()
 {
-	RemoveTrajectory(this);
+	Trajectory->Remove(this);
 	return true;
 }
 
-public func ControlUseAltCancel()
-{
-	return ControlUseCancel();
-}
-
 //Stops the player from shooting for the defined amount of frames
 public func FxIntCooldownTimer(object target, effect, int timer)
 {
@@ -262,8 +240,6 @@ func FxIntTurnCannonTimer(object cannon, proplist effect, int timer)
 
 protected func DoFire(object iammo, object clonk, int angle)
 {
-	iammo->~Fuse();
-
 	//Don't excede possible trajectory
 	var r = Normalize(angle,-180 * angPrec, angPrec);
 	if(r > 90 * angPrec + GetR() * angPrec) r = 90 * angPrec + GetR() * angPrec;
@@ -273,6 +249,8 @@ protected func DoFire(object iammo, object clonk, int angle)
 	iammo->SetR(r / angPrec);
 	iammo->SetRDir(-4 + Random(9));
 	iammo->LaunchProjectile(r, 17, Fire_Velocity, 0,0, angPrec);
+	iammo->SetController(clonk->GetController());
+	iammo->~Fuse();
 
 	//Particles
 	var dist = 25;
@@ -285,7 +263,7 @@ protected func DoFire(object iammo, object clonk, int angle)
 	CreateParticle("Smoke", px, py, PV_Random(x - 20, x + 20), PV_Random(y - 20, y + 20), PV_Random(40, 60), Particles_Smoke(), 20);
 	CreateMuzzleFlash(px, py, r / angPrec, 60);
 	//sound
-	Sound("Blast3");
+	Sound("Fire::Blast3");
 }
 
 local ActMap = {
@@ -308,4 +286,4 @@ Roll = {
 local Name = "$Name$";
 local Description = "$Description$";
 local Touchable = 1;
-local Rebuy = true;
+local BorderBound = C4D_Border_Sides;
\ No newline at end of file
