@@ -32,7 +32,7 @@ public:
 	size_t GetDataSize() const { return Data.getLength(); }
 	bool Load(C4Group &hGroup, const char *szFilename, const char *szLanguage=NULL);
 	bool Load(C4GroupSet &hGroupSet, const char *szFilename, const char *szLanguage=NULL);
-	bool GetLanguageString(const char *szLanguage, class StdStrBuf &rTarget);
+	bool GetLanguageString(const char *szLanguage, StdStrBuf &rTarget);
 protected:
 	// The component host's Data has changed. This callback can be used by
 	// derived classes to reload internal structures.
