@@ -23,15 +23,13 @@
 class C4Extra
 {
 public:
-	C4Extra() { Default(); };     // ctor
-	~C4Extra() { Clear(); };      // dtor
-	void Default(); // zero fields
-	void Clear();   // free class members
+	C4Extra();
+	~C4Extra();
 
 	bool Init();      // init extra group, using scneario presets
 	bool InitGroup(); // open extra group
 
-	std::vector<C4Group*> ExtraGroups; // extra.ocg root folders
+	std::vector<std::unique_ptr<C4Group>> ExtraGroups; // extra.ocg root folders
 
 protected:
 	bool LoadDef(C4Group &hGroup, const char *szName); // load preset for definition
