@@ -74,40 +74,31 @@ bool C4Group_PackDirectoryTo(const char *szFilename, const char *szFilenameTo);
 bool C4Group_PackDirectory(const char *szFilename);
 bool C4Group_UnpackDirectory(const char *szFilename);
 bool C4Group_ExplodeDirectory(const char *szFilename);
-bool C4Group_SetOriginal(const char *szFilename, bool fOriginal);
 bool C4Group_ReadFile(const char *szFilename, char **pData, size_t *iSize);
 
 extern const char *C4CFN_FLS[];
 
-extern time_t C4Group_AssumeTimeOffset;
-
 #pragma pack (push, 1)
 
-class C4GroupHeader
+struct C4GroupHeader
 {
-public:
-	C4GroupHeader();
-public:
-	char id[24+4];
-	int Ver1,Ver2;
-	int Entries;
-	char reserved[164];
-public:
-	void Init();
+	char id[24+4] = C4GroupFileID;
+	int Ver1 = C4GroupFileVer1;
+	int Ver2 = C4GroupFileVer2;
+	int Entries = 0;
+	char reserved[164] = { 0 };
 };
 
-class C4GroupEntryCore
+struct C4GroupEntryCore
 {
-public:
-	C4GroupEntryCore();
-public:
-	char FileName[260];
-	int32_t Packed,ChildGroup;
-	int32_t Size, reserved1, Offset;
-	int32_t reserved2;
-	char reserved3; unsigned int reserved4;
-	char Executable;
-	BYTE fbuf[26];
+	char FileName[260] = { 0 };
+	int32_t Packed = 0, ChildGroup = 0;
+	int32_t Size = 0, reserved1 = 0, Offset = 0;
+	int32_t reserved2 = 0;
+	char reserved3 = '\0';
+	unsigned int reserved4 = 0;
+	char Executable = '\0';
+	BYTE fbuf[26] = { 0 };
 };
 
 #pragma pack (pop)
@@ -115,7 +106,6 @@ public:
 class C4GroupEntry: public C4GroupEntryCore
 {
 public:
-	C4GroupEntry();
 	~C4GroupEntry();
 
 	enum EntryStatus
@@ -127,14 +117,14 @@ public:
 	};
 
 public:
-	char DiskPath[_MAX_PATH + 1];
-	EntryStatus Status;
-	bool DeleteOnDisk;
-	bool HoldBuffer;
-	bool BufferIsStdbuf;
-	bool NoSort;
-	BYTE *bpMemBuf;
-	C4GroupEntry *Next;
+	char DiskPath[_MAX_PATH + 1] = { 0 };
+	EntryStatus Status = C4GRES_InGroup;
+	bool DeleteOnDisk = false;
+	bool HoldBuffer = false;
+	bool BufferIsStdbuf = false;
+	bool NoSort = false;
+	BYTE *bpMemBuf = 0;
+	C4GroupEntry *Next = 0;
 public:
 	void Set(const DirectoryIterator & iter, const char * szPath);
 };
@@ -174,8 +164,6 @@ protected:
 	StdStrBuf sPrevAccessedEntry;
 #endif
 	// Folder only
-	//struct _finddata_t Fdt;
-	//long hFdt;
 	DirectoryIterator FolderSearch;
 	C4GroupEntry FolderSearchEntry;
 	C4GroupEntry LastFolderSearchEntry;
@@ -259,7 +247,7 @@ public:
 	inline bool IsPacked() { return Status == GRPF_File; }
 	inline bool HasPackedMother() { if (!Mother) return false; return Mother->IsPacked(); }
 	inline bool SetNoSort(bool fNoSort) { NoSort = fNoSort; return true; }
-	int PreCacheEntries(const char *szSearchPattern); // pre-load entries to memory. return number of loaded entries.
+	int PreCacheEntries(const char *szSearchPattern, bool cache_previous=false); // pre-load entries to memory. return number of loaded entries.
 
 	const C4GroupHeader &GetHeader() const { return Head; }
 	const C4GroupEntry *GetFirstEntry() const { return FirstEntry; }
@@ -294,6 +282,7 @@ protected:
 	C4GroupEntry *SearchNextEntry(const char *szName);
 	C4GroupEntry *GetNextFolderEntry();
 	uint32_t CalcCRC32(C4GroupEntry *pEntry);
+	void PreCacheEntry(C4GroupEntry * p);
 };
 
 #endif
