@@ -227,12 +227,6 @@ public:
 
 // *** C4UpdatePackageCore
 
-C4UpdatePackageCore::C4UpdatePackageCore()
-{
-	// zero fields
-	ZeroMem(this, sizeof(C4UpdatePackageCore));
-}
-
 void C4UpdatePackageCore::CompileFunc(StdCompiler *pComp)
 {
 	pComp->Value(mkNamingAdapt(toC4CArr(RequireVersion), "RequireVersion"));
@@ -347,9 +341,6 @@ bool C4UpdatePackage::Execute(C4Group *pGroup)
 	// check if the update is allowed
 	if (GrpUpdate)
 	{
-		// maker must match
-		/*if(!SEqual(TargetGrp.GetMaker(), pGroup->GetMaker())) - now allowing updates from different makers
-		  return false;*/
 		// check checksum
 		uint32_t iCRC32;
 		if (!GetFileCRC(TargetGrp.GetFullName().getData(), &iCRC32))
