@@ -39,6 +39,7 @@
 #include <C4PlayerList.h>
 #include <C4GameObjects.h>
 #include <C4GameControl.h>
+#include <C4ScriptGuiWindow.h>
 #include "gui/C4MessageInput.h"
 #include "object/C4DefList.h"
 
@@ -190,7 +191,7 @@ void C4ControlSet::Execute() const
 		if (Game.Parameters.isLeague())
 		{
 			Log("/set maxplayer disabled in league!");
-			C4GUI::GUISound("Error");
+			C4GUI::GUISound("UI::Error");
 			break;
 		}
 		// set it
@@ -310,8 +311,6 @@ void C4ControlMsgBoardReply::CompileFunc(StdCompiler *pComp)
 // *** C4ControlMsgBoardCmd
 void C4ControlMsgBoardCmd::Execute() const
 {
-	C4Player *source_player = ::Players.Get(player);
-
 	// don't handle this if the game isn't actually running
 	if (!::Game.IsRunning) return;
 
@@ -543,6 +542,44 @@ void C4ControlPlayerCommand::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(mkIntPackAdapt(iAddMode), "AddMode", 0));
 	C4ControlPacket::CompileFunc(pComp);
 }
+// *** C4ControlMenuCommand
+
+C4ControlMenuCommand::C4ControlMenuCommand(int32_t actionID, int32_t player, int32_t menuID, int32_t subwindowID, C4Object *target, int32_t actionType)
+	: actionID(actionID), player(player), menuID(menuID), subwindowID(subwindowID), target(target ? target->Number : 0), actionType(actionType)
+{
+
+}
+
+void C4ControlMenuCommand::Execute() const
+{
+	// invalid action? The action needs to be in bounds!
+	if (actionType < 0 || actionType >= C4ScriptGuiWindowPropertyName::_lastProp)
+	{
+		// this could only come from a malicious attempt to crash the engine!
+		Log("Warning: invalid action type for C4ControlMenuCommand!");
+		return;
+	}
+	C4ScriptGuiWindow *menu = ::Game.ScriptGuiRoot->GetChildByID(menuID);
+	// menu was closed?
+	if (!menu) return;
+
+	C4Object *obj = target ? ::Objects.ObjectPointer(target) : 0;
+	// target has been removed in the meantime? abort now
+	if (target && !obj) return;
+
+	menu->ExecuteCommand(actionID, player, subwindowID, actionType, obj);
+}
+
+void C4ControlMenuCommand::CompileFunc(StdCompiler *pComp)
+{
+	pComp->Value(mkNamingAdapt(mkIntPackAdapt(actionID), "ID", -1));
+	pComp->Value(mkNamingAdapt(mkIntPackAdapt(player), "Player", -1));
+	pComp->Value(mkNamingAdapt(mkIntPackAdapt(menuID), "Menu", 0));
+	pComp->Value(mkNamingAdapt(mkIntPackAdapt(subwindowID), "Window", 0));
+	pComp->Value(mkNamingAdapt(mkIntPackAdapt(actionType), "Action", 0));
+	pComp->Value(mkNamingAdapt(target, "Target", 0));
+	C4ControlPacket::CompileFunc(pComp);
+}
 
 // *** C4ControlPlayerAction
 C4ControlPlayerAction::C4ControlPlayerAction(const C4Player *source)
@@ -803,7 +840,7 @@ void C4ControlSyncCheck::Execute() const
 		LogFatal("Network: Synchronization loss!");
 		LogFatal(FormatString("Network: %s Frm %i Ctrl %i Rnc %i Cpx %i PXS %i MMi %i Obc %i Oei %i Sct %i", szThis, Frame,ControlTick,RandomCount,AllCrewPosX,PXSCount,MassMoverIndex,ObjectCount,ObjectEnumerationIndex, SectShapeSum).getData());
 		LogFatal(FormatString("Network: %s Frm %i Ctrl %i Rnc %i Cpx %i PXS %i MMi %i Obc %i Oei %i Sct %i", szOther, SyncCheck.Frame,SyncCheck.ControlTick,SyncCheck.RandomCount,SyncCheck.AllCrewPosX,SyncCheck.PXSCount,SyncCheck.MassMoverIndex,SyncCheck.ObjectCount,SyncCheck.ObjectEnumerationIndex, SyncCheck.SectShapeSum).getData());
-		StartSoundEffect("SyncError");
+		StartSoundEffect("UI::SyncError");
 #ifdef _DEBUG
 		// Debug safe
 		C4GameSaveNetwork SaveGame(false);
@@ -881,7 +918,7 @@ void C4ControlClientJoin::CompileFunc(StdCompiler *pComp)
 void C4ControlClientUpdate::Execute() const
 {
 	// host only
-	if (iByClient != C4ClientIDHost) return;
+	if (iByClient != C4ClientIDHost && eType != CUT_SetReady) return;
 	// find client
 	C4Client *pClient = Game.Clients.getClientByID(iID);
 	if (!pClient) return;
@@ -914,6 +951,26 @@ void C4ControlClientUpdate::Execute() const
 		// remove all players ("soft kick")
 		::Players.RemoveAtClient(iID, true);
 		break;
+	case CUT_SetReady:
+		{
+		// nothing to do?
+		if (pClient->isLobbyReady() == !!iData) break;
+		// ready/unready (while keeping track of time)
+		time_t last_change_time = MinReadyAnnouncementDelay;
+		pClient->SetLobbyReady(!!iData, &last_change_time);
+		// log to others, but don't spam
+		if (last_change_time >= MinReadyAnnouncementDelay)
+		{
+			if (!pClient->isLocal())
+			{
+				LogF(LoadResStr(iData ? "IDS_NET_CLIENT_READY" : "IDS_NET_CLIENT_UNREADY"), strClient.getData(), pClient->getName());
+			}
+			// Also update icons
+			C4GameLobby::MainDlg *lobby = ::Network.GetLobby();
+			if (lobby) lobby->OnClientReadyStateChange();
+		}
+		break;
+		}
 	}
 }
 
@@ -923,6 +980,8 @@ void C4ControlClientUpdate::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(mkIntPackAdapt(iID), "ClientID", C4ClientIDUnknown));
 	if (eType == CUT_Activate)
 		pComp->Value(mkNamingAdapt(mkIntPackAdapt(iData), "Data", 0));
+	if (eType == CUT_SetReady)
+		pComp->Value(mkNamingAdapt(mkIntPackAdapt(iData), "Data", 0));
 	C4ControlPacket::CompileFunc(pComp);
 }
 
@@ -1371,9 +1430,10 @@ void C4ControlEMMoveObject::CompileFunc(StdCompiler *pComp)
 
 C4ControlEMDrawTool::C4ControlEMDrawTool(C4ControlEMDrawAction eAction, int32_t iMode,
     int32_t iX, int32_t iY, int32_t iX2, int32_t iY2, int32_t iGrade,
-    bool fIFT, const char *szMaterial, const char *szTexture)
+    const char *szMaterial, const char *szTexture, const char *szBackMaterial, const char *szBackTexture)
 		: eAction(eAction), iMode(iMode), iX(iX), iY(iY), iX2(iX2), iY2(iY2), iGrade(iGrade),
-		fIFT(fIFT), Material(szMaterial, true), Texture(szTexture, true)
+		Material(szMaterial, true), Texture(szTexture, true),
+		BackMaterial(szBackMaterial, true), BackTexture(szBackTexture, true)
 {
 
 }
@@ -1392,21 +1452,23 @@ void C4ControlEMDrawTool::Execute() const
 	// assert validity of parameters
 	if (!Material.getSize()) return;
 	const char *szMaterial = Material.getData(),
-	                         *szTexture = Texture.getData();
+	           *szTexture = Texture.getData();
+	const char *szBackMaterial = BackMaterial.getData(),
+	           *szBackTexture = BackTexture.getData();
 	// perform action
 	switch (eAction)
 	{
 	case EMDT_Brush: // brush tool
 		if (!Texture.getSize()) break;
-		::Landscape.DrawBrush(iX, iY, iGrade, szMaterial, szTexture, fIFT);
+		::Landscape.DrawBrush(iX, iY, iGrade, szMaterial, szTexture, szBackMaterial, szBackTexture);
 		break;
 	case EMDT_Line: // line tool
 		if (!Texture.getSize()) break;
-		::Landscape.DrawLine(iX,iY,iX2,iY2, iGrade, szMaterial, szTexture, fIFT);
+		::Landscape.DrawLine(iX,iY,iX2,iY2, iGrade, szMaterial, szTexture, szBackMaterial, szBackTexture);
 		break;
 	case EMDT_Rect: // rect tool
 		if (!Texture.getSize()) break;
-		::Landscape.DrawBox(iX,iY,iX2,iY2, iGrade, szMaterial, szTexture, fIFT);
+		::Landscape.DrawBox(iX,iY,iX2,iY2, iGrade, szMaterial, szTexture, szBackMaterial, szBackTexture);
 		break;
 	case EMDT_Fill: // fill tool
 	{
@@ -1435,9 +1497,10 @@ void C4ControlEMDrawTool::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(iX2, "X2", 0));
 	pComp->Value(mkNamingAdapt(iY2, "Y2", 0));
 	pComp->Value(mkNamingAdapt(mkIntPackAdapt(iGrade),  "Grade", 0));
-	pComp->Value(mkNamingAdapt(fIFT, "IFT", false));
 	pComp->Value(mkNamingAdapt(Material, "Material", ""));
 	pComp->Value(mkNamingAdapt(Texture, "Texture", ""));
+	pComp->Value(mkNamingAdapt(BackMaterial, "BackMaterial", ""));
+	pComp->Value(mkNamingAdapt(BackTexture, "BackTexture", ""));
 	C4ControlPacket::CompileFunc(pComp);
 }
 
@@ -1451,7 +1514,6 @@ void C4ControlMessage::Execute() const
 	// security
 	if (pPlr && pPlr->AtClient != iByClient) return;
 	// do not record message as control, because it is not synced!
-	//if (pPlr) pPlr->CountControl(C4Player::PCID_Message, Message.GetHash());
 	// get lobby to forward to
 	C4GameLobby::MainDlg *pLobby = ::Network.GetLobby();
 	StdStrBuf str;
@@ -1695,7 +1757,7 @@ void C4ControlVote::Execute() const
 		int32_t iPositive = 0, iNegative = 0, iVotes = 0;
 		// If there are no teams, count as if all were in the same team
 		// (which happens to be equivalent to "everyone is in his own team" here)
-		for (int32_t i = 0; i < Max<int32_t>(Game.Teams.GetTeamCount(), 1); i++)
+		for (int32_t i = 0; i < std::max<int32_t>(Game.Teams.GetTeamCount(), 1); i++)
 		{
 			C4Team *pTeam = Game.Teams.GetTeamByIndex(i);
 			// Votes for this team
