@@ -272,6 +272,19 @@ public:
 	DECLARE_C4CONTROL_VIRTUALS
 };
 
+class C4ControlMenuCommand : public C4ControlPacket // sync
+{
+public:
+	C4ControlMenuCommand()
+			: menuID(0), subwindowID(0) { }
+	C4ControlMenuCommand(int32_t actionID, int32_t player, int32_t menuID, int32_t subwindowID,
+	                       C4Object *target, int32_t actionType);
+protected:
+	int32_t actionID, player, menuID, subwindowID, target, actionType;
+public:
+	DECLARE_C4CONTROL_VIRTUALS
+};
+
 class C4ControlPlayerAction : public C4ControlPacket // sync
 {
 public:
@@ -369,7 +382,7 @@ public:
 
 enum C4ControlClientUpdType
 {
-	CUT_None = -1, CUT_Activate = 0, CUT_SetObserver = 1
+	CUT_None = -1, CUT_Activate = 0, CUT_SetObserver = 1, CUT_SetReady = 2
 };
 
 class C4ControlClientUpdate : public C4ControlPacket // sync, lobby
@@ -379,6 +392,8 @@ public:
 	C4ControlClientUpdate(int32_t iID, C4ControlClientUpdType eType, int32_t iData = 0)
 			: iID(iID), eType(eType), iData(iData)
 	{ }
+private:
+	static const int32_t MinReadyAnnouncementDelay = 1; // seconds that need to pass between ready-state announcements to prevent spam
 public:
 	int32_t iID;
 	C4ControlClientUpdType eType;
@@ -487,17 +502,19 @@ enum C4ControlEMDrawAction
 class C4ControlEMDrawTool : public C4ControlPacket // sync
 {
 public:
-	C4ControlEMDrawTool() : eAction(EMDT_SetMode), iX(0), iY(0), iX2(0), iY2(0), iGrade(0), fIFT(false) { }
+	C4ControlEMDrawTool() : eAction(EMDT_SetMode), iX(0), iY(0), iX2(0), iY2(0), iGrade(0) { }
 	C4ControlEMDrawTool(C4ControlEMDrawAction eAction, int32_t iMode,
 	                    int32_t iX=-1, int32_t iY=-1, int32_t iX2=-1, int32_t iY2=-1, int32_t iGrade=-1,
-	                    bool fIFT=true, const char *szMaterial=NULL, const char *szTexture=NULL);
+	                    const char *szMaterial=NULL, const char *szTexture=NULL,
+	                    const char *szBackMaterial=NULL, const char *szBackTexture=NULL);
 protected:
 	C4ControlEMDrawAction eAction;  // action to be performed
 	int32_t iMode;        // new mode, or mode action was performed in (action will fail if changed)
 	int32_t iX,iY,iX2,iY2,iGrade; // drawing parameters
-	bool fIFT;        // sky/tunnel-background
 	StdStrBuf Material; // used material
 	StdStrBuf Texture;  // used texture
+	StdStrBuf BackMaterial; // used background material
+	StdStrBuf BackTexture;  // used background texture
 public:
 	DECLARE_C4CONTROL_VIRTUALS
 };
