@@ -47,11 +47,14 @@ void C4ToolsDlg::Clear()
 
 void C4ToolsDlg::Default()
 {
-	state->Default();
 	Active = false;
 	Tool = SelectedTool = C4TLS_Brush;
 	Grade = C4TLS_GradeDefault;
 	ModeIFT = true;
 	SCopy("Earth",Material);
 	SCopy("earth",Texture);
+	ModeBack = false;
+	SCopy("Tunnel", BackMaterial);
+	SCopy("tunnel", BackTexture);
+	state->Default();
 }
