@@ -26,25 +26,14 @@
 
 C4TransferZone::C4TransferZone()
 {
-	Default();
+	Object = NULL;
+	X = Y = Wdt = Hgt = 0;
+	Next = NULL;
+	Used = false;
 }
 
 C4TransferZone::~C4TransferZone()
 {
-	Clear();
-}
-
-void C4TransferZone::Default()
-{
-	Object=NULL;
-	X=Y=Wdt=Hgt=0;
-	Next=NULL;
-	Used=false;
-}
-
-void C4TransferZone::Clear()
-{
-
 }
 
 C4TransferZones::C4TransferZones()
