@@ -27,7 +27,6 @@ extern C4Draw *pDraw;
 
 inline void DwTo4UB(DWORD dwClr, unsigned char (&r)[4])
 {
-	//unsigned char r[4];
 	r[0] = (unsigned char)( (dwClr >> 16) & 0xff);
 	r[1] = (unsigned char)( (dwClr >>  8) & 0xff);
 	r[2] = (unsigned char)( (dwClr      ) & 0xff);
@@ -115,35 +114,6 @@ struct C4BltVertex
 	float ftx,fty,ftz; // blit positions
 };
 
-// This structure is used by StdGL, too
-typedef struct _GAMMARAMP
-{
-	WORD                red  [256];
-	WORD                green[256];
-	WORD                blue [256];
-} GAMMARAMP;
-
-// gamma ramp control
-class C4GammaControl
-{
-private:
-	void SetClrChannel(WORD *pBuf, BYTE c1, BYTE c2, int c3); // set color channel ramp
-
-protected:
-	GAMMARAMP ramp;
-
-public:
-	C4GammaControl() { Default(); } // ctor
-	void Default() { Set(0x000000, 0x808080, 0xffffff); } // set default ramp
-
-	void Set(DWORD dwClr1, DWORD dwClr2, DWORD dwClr3); // set color ramp
-
-	DWORD ApplyTo(DWORD dwClr);   // apply gamma to color value
-
-	friend class C4Draw;
-	friend class CStdGL;
-};
-
 // helper struct
 struct ZoomData
 {
@@ -153,23 +123,28 @@ struct ZoomData
 
 class C4FoWRegion;
 
+// Shader combinations
+static const int C4SSC_MOD2 = 1; // signed addition instead of multiplication for clrMod
+static const int C4SSC_BASE = 2; // use a base texture instead of just a single color
+static const int C4SSC_OVERLAY = 4; // use a colored overlay on top of base texture
+static const int C4SSC_LIGHT = 8; // use dynamic+ambient lighting
+static const int C4SSC_NORMAL = 16; // extract normals from normal map instead of (0,0,1)
+
 // direct draw encapsulation
 class C4Draw
 {
 public:
-	static const StdMeshMatrix OgreToClonk;
+	enum DrawOperation { OP_POINTS, OP_TRIANGLES };
 
 	C4Draw(): MaxTexSize(0) { }
 	virtual ~C4Draw() { pDraw=NULL; }
 public:
 	C4AbstractApp * pApp; // the application
 	bool Active;                    // set if device is ready to render, etc.
-	C4GammaControl Gamma;            // gamma
-	C4GammaControl DefRamp;            // default gamma ramp
-	uint32_t dwGamma[C4MaxGammaRamps*3];    // gamma ramps
+	float gamma[C4MaxGammaRamps][3]; // input gammas
+	float gammaOut[3]; // combined gamma
 	int MaxTexSize;
 protected:
-	bool fSetGamma;     // must gamma ramp be reassigned?
 	BYTE                byByteCnt;    // bytes per pixel (2 or 4)
 	float fClipX1,fClipY1,fClipX2,fClipY2; // clipper in unzoomed coordinates
 	float fStClipX1,fStClipY1,fStClipX2,fStClipY2; // stored clipper in unzoomed coordinates
@@ -195,8 +170,6 @@ public:
 	virtual CStdGLCtx *CreateContext(HWND, C4AbstractApp *) { return NULL; }
 #endif
 	virtual bool OnResolutionChanged(unsigned int iXRes, unsigned int iYRes) = 0; // reinit clipper for new resolution
-	virtual bool IsOpenGL() { return false; }
-	virtual bool IsShaderific() { return false; }
 	// Clipper
 	bool GetPrimaryClipper(int &rX1, int &rY1, int &rX2, int &rY2);
 	bool SetPrimaryClipper(int iX1, int iY1, int iX2, int iY2);
@@ -216,6 +189,7 @@ public:
 	void PrimaryUnlocked() { PrimaryLocked=false; }
 	virtual bool PrepareMaterial(StdMeshMatManager& mat_manager, StdMeshMaterialLoader& loader, StdMeshMaterial& mat) = 0; // Find best technique, fail if there is none
 	virtual bool PrepareRendering(C4Surface * sfcToSurface) = 0; // check if/make rendering possible to given surface
+	virtual bool PrepareSpriteShader(C4Shader& shader, const char* name, int ssc, C4GroupSet* pGroups, const char* const* additionalDefines, const char* const* additionalSlices) = 0; // create sprite shader
 	// Blit
 	virtual void BlitLandscape(C4Surface * sfcSource, float fx, float fy,
 	                           C4Surface * sfcTarget, float tx, float ty, float wdt, float hgt);
@@ -236,32 +210,28 @@ public:
 	                C4Surface * sfcTarget, int tx, int ty, int twdt, int thgt,
 	                bool fTransparency=true);
 	bool BlitSurface(C4Surface * sfcSurface, C4Surface * sfcTarget, int tx, int ty, bool fBlitBase);
-	bool BlitSurfaceTile(C4Surface * sfcSurface, C4Surface * sfcTarget, float iToX, float iToY, float iToWdt, float iToHgt, float iOffsetX=0, float iOffsetY=0, bool fSrcColKey=false);
-	bool BlitSurfaceTile2(C4Surface * sfcSurface, C4Surface * sfcTarget, float iToX, float iToY, float iToWdt, float iToHgt, float iOffsetX=0, float iOffsetY=0, bool fSrcColKey=false);
+	bool BlitSurfaceTile(C4Surface * sfcSurface, C4Surface * sfcTarget, float iToX, float iToY, float iToWdt, float iToHgt, float iOffsetX, float iOffsetY, C4ShaderCall* shader_call);
 	virtual void FillBG(DWORD dwClr=0) = 0;
 	// Text
 	enum { DEFAULT_MESSAGE_COLOR = 0xffffffff };
 	bool TextOut(const char *szText, CStdFont &rFont, float fZoom, C4Surface * sfcDest, float iTx, float iTy, DWORD dwFCol=0xffffffff, BYTE byForm=ALeft, bool fDoMarkup=true);
 	bool StringOut(const char *szText, CStdFont &rFont, float fZoom, C4Surface * sfcDest, float iTx, float iTy, DWORD dwFCol=0xffffffff, BYTE byForm=ALeft, bool fDoMarkup=true);
 	// Drawing
-	virtual void PerformMultiPix(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices) = 0;
-	virtual void PerformMultiLines(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, float width) = 0;
-	virtual void PerformMultiTris(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, const C4BltTransform* pTransform, C4TexRef* pTex, C4TexRef* pOverlay, C4TexRef* pNormal, DWORD dwOverlayClrMod) = 0; // blit the same texture many times
+	virtual void PerformMultiPix(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, C4ShaderCall* shader_call) = 0;
+	virtual void PerformMultiLines(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, float width, C4ShaderCall* shader_call) = 0;
+	virtual void PerformMultiTris(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, const C4BltTransform* pTransform, C4TexRef* pTex, C4TexRef* pOverlay, C4TexRef* pNormal, DWORD dwOverlayClrMod, C4ShaderCall* shader_call) = 0; // blit the same texture many times
 	// Convenience drawing functions
 	void DrawBoxDw(C4Surface * sfcDest, int iX1, int iY1, int iX2, int iY2, DWORD dwClr); // calls DrawBoxFade
-	void DrawBoxFade(C4Surface * sfcDest, float iX, float iY, float iWdt, float iHgt, DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, DWORD dwClr4, int iBoxOffX, int iBoxOffY); // calls DrawQuadDw
+	void DrawBoxFade(C4Surface * sfcDest, float iX, float iY, float iWdt, float iHgt, DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, DWORD dwClr4, C4ShaderCall* shader_call); // calls DrawQuadDw
 	void DrawPatternedCircle(C4Surface * sfcDest, int x, int y, int r, BYTE col, C4Pattern & Pattern, CStdPalette &rPal);
 	void DrawFrameDw(C4Surface * sfcDest, int x1, int y1, int x2, int y2, DWORD dwClr);
-	void DrawQuadDw(C4Surface * sfcTarget, float *ipVtx, DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, DWORD dwClr4);
+	void DrawQuadDw(C4Surface * sfcTarget, float *ipVtx, DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, DWORD dwClr4, C4ShaderCall* shader_call);
 	void DrawPix(C4Surface * sfcDest, float tx, float ty, DWORD dwCol); // Consider using PerformMultiPix if you draw more than one pixel
 	void DrawLineDw(C4Surface * sfcTarget, float x1, float y1, float x2, float y2, DWORD dwClr, float width = 1.0f); // Consider using PerformMultiLines if you draw more than one line
 	// gamma
-	void SetGamma(DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, int32_t iRampIndex);  // set gamma ramp
-	void ResetGamma();                                        // reset all gamma ramps to default
-	void ApplyGamma();                                        // apply gamma ramp to ddraw
-	void DisableGamma();                                      // temporarily reset app gamma to default
-	void EnableGamma();                                       // set current gamma ramp in app
-	DWORD ApplyGammaTo(DWORD dwClr);                          // apply gamma to given color
+	void SetGamma(float r, float g, float b, int32_t iRampIndex);  // set gamma
+	void ResetGamma(); // reset gamma to default
+	DWORD ApplyGammaTo(DWORD dwClr); // apply gamma to given color
 	// blit states
 	void ActivateBlitModulation(DWORD dwWithClr) { BlitModulated=true; BlitModulateClr=dwWithClr; } // modulate following blits with a given color
 	void DeactivateBlitModulation() { BlitModulated=false; }  // stop color modulation of blits
@@ -288,12 +258,6 @@ protected:
 	bool CreatePrimaryClipper(unsigned int iXRes, unsigned int iYRes);
 	virtual bool CreatePrimarySurfaces(unsigned int iXRes, unsigned int iYRes, int iColorDepth, unsigned int iMonitor) = 0;
 	virtual bool Error(const char *szMsg);
-	void DebugLog(const char *szMsg)
-	{
-#ifdef _DEBUG
-		Log(szMsg);
-#endif
-	}
 
 	friend class C4Surface;
 	friend class C4TexRef;
