@@ -92,7 +92,7 @@ namespace
 		const char *msg_type = MsgTypeToStr(type);
 		const char *msg_severity = MsgSeverityToStr(severity);
 
-		DebugLogF("  gl: %s severity %s %s: %s", msg_severity, msg_source, msg_type, message);
+		LogSilentF("  gl: %s severity %s %s: %s", msg_severity, msg_source, msg_type, message);
 #ifdef USE_WIN32_WINDOWS
 		if (IsDebuggerPresent() && severity == GL_DEBUG_SEVERITY_HIGH_ARB)
 			BREAKPOINT_HERE;
@@ -103,8 +103,9 @@ namespace
 #undef USERPARAM_CONST
 
 CStdGL::CStdGL():
-		pMainCtx(0)
+	pMainCtx(0), CurrentVBO(0)
 {
+	GenericVBOs[0] = 0;
 	Default();
 	byByteCnt=4;
 	// global ptr
@@ -121,7 +122,7 @@ CStdGL::~CStdGL()
 void CStdGL::Clear()
 {
 	NoPrimaryClipper();
-	//if (pTexMgr) pTexMgr->IntUnlock(); // cannot do this here or we can't preserve textures across GL reinitialization as required when changing multisampling
+	// cannot unlock TexMgr here or we can't preserve textures across GL reinitialization as required when changing multisampling
 	InvalidateDeviceObjects();
 	NoPrimaryClipper();
 	RenderTarget = NULL;
@@ -169,14 +170,7 @@ bool CStdGL::UpdateClipper()
 	ClipAll=false;
 	// set it
 	glViewport(clipRect.x, RenderTarget->Hgt-clipRect.y-clipRect.Hgt, clipRect.Wdt, clipRect.Hgt);
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-
-	// Set clipping plane to -1000 and 1000 so that large meshes are not
-	// clipped away.
-	//glOrtho((GLdouble) iX, (GLdouble) (iX+iWdt), (GLdouble) (iY+iHgt), (GLdouble) iY, -1000.0f, 1000.0f);
-	gluOrtho2D((GLdouble) clipRect.x, (GLdouble) (clipRect.x + clipRect.Wdt), (GLdouble) (clipRect.y + clipRect.Hgt), (GLdouble) clipRect.y);
-	//gluOrtho2D((GLdouble) 0, (GLdouble) xRes, (GLdouble) yRes, (GLdouble) yRes-iHgt);
+	ProjectionMatrix = StdProjectionMatrix::Orthographic(clipRect.x, clipRect.x + clipRect.Wdt, clipRect.y + clipRect.Hgt, clipRect.y);
 	return true;
 }
 
@@ -186,7 +180,6 @@ bool CStdGL::PrepareRendering(C4Surface * sfcToSurface)
 	if (!pApp || !pApp->AssertMainThread()) return false;
 	// not ready?
 	if (!Active)
-		//if (!RestoreDeviceObjects())
 		return false;
 	// target?
 	if (!sfcToSurface) return false;
@@ -209,21 +202,97 @@ bool CStdGL::PrepareRendering(C4Surface * sfcToSurface)
 	return true;
 }
 
+
+bool CStdGL::PrepareSpriteShader(C4Shader& shader, const char* name, int ssc, C4GroupSet* pGroups, const char* const* additionalDefines, const char* const* additionalSlices)
+{
+	const char* uniformNames[C4SSU_Count + 1];
+	uniformNames[C4SSU_ProjectionMatrix] = "projectionMatrix";
+	uniformNames[C4SSU_ModelViewMatrix] = "modelviewMatrix";
+	uniformNames[C4SSU_NormalMatrix] = "normalMatrix";
+	uniformNames[C4SSU_ClrMod] = "clrMod";
+	uniformNames[C4SSU_Gamma] = "gamma";
+	uniformNames[C4SSU_BaseTex] = "baseTex";
+	uniformNames[C4SSU_OverlayTex] = "overlayTex";
+	uniformNames[C4SSU_OverlayClr] = "overlayClr";
+	uniformNames[C4SSU_LightTex] = "lightTex";
+	uniformNames[C4SSU_LightTransform] = "lightTransform";
+	uniformNames[C4SSU_NormalTex] = "normalTex";
+	uniformNames[C4SSU_AmbientTex] = "ambientTex";
+	uniformNames[C4SSU_AmbientTransform] = "ambientTransform";
+	uniformNames[C4SSU_AmbientBrightness] = "ambientBrightness";
+	uniformNames[C4SSU_Bones] = "bones"; // unused
+	uniformNames[C4SSU_CullMode] = "cullMode"; // unused
+	uniformNames[C4SSU_Count] = NULL;
+
+	const char* attributeNames[C4SSA_Count + 1];
+	attributeNames[C4SSA_Position] = "oc_Position";
+	attributeNames[C4SSA_Normal] = "oc_Normal"; // unused
+	attributeNames[C4SSA_TexCoord] = "oc_TexCoord"; // only used if C4SSC_Base is set
+	attributeNames[C4SSA_Color] = "oc_Color";
+	attributeNames[C4SSA_BoneIndices0] = "oc_BoneIndices0"; // unused
+	attributeNames[C4SSA_BoneIndices1] = "oc_BoneIndices1"; // unused
+	attributeNames[C4SSA_BoneWeights0] = "oc_BoneWeights0"; // unused
+	attributeNames[C4SSA_BoneWeights1] = "oc_BoneWeights1"; // unused
+	attributeNames[C4SSA_Count] = NULL;
+
+	// Clear previous content
+	shader.Clear();
+	shader.ClearSlices();
+
+	// Start with #defines
+	shader.AddDefine("OPENCLONK");
+	shader.AddDefine("OC_SPRITE");
+	if (ssc & C4SSC_MOD2) shader.AddDefine("OC_CLRMOD_MOD2");
+	if (ssc & C4SSC_NORMAL) shader.AddDefine("OC_WITH_NORMALMAP");
+	if (ssc & C4SSC_LIGHT) shader.AddDefine("OC_DYNAMIC_LIGHT");
+	if (ssc & C4SSC_BASE) shader.AddDefine("OC_HAVE_BASE");
+	if (ssc & C4SSC_OVERLAY) shader.AddDefine("OC_HAVE_OVERLAY");
+
+	if (additionalDefines)
+		for (const char* const* define = additionalDefines; *define != NULL; ++define)
+			shader.AddDefine(*define);
+
+	// Then load slices for fragment and vertex shader
+	shader.LoadVertexSlices(pGroups, "SpriteVertexShader.glsl");
+	shader.LoadFragmentSlices(pGroups, "CommonShader.glsl");
+	shader.LoadFragmentSlices(pGroups, "ObjectShader.glsl");
+
+	if (additionalSlices)
+		for (const char* const* slice = additionalSlices; *slice != NULL; ++slice)
+			shader.LoadFragmentSlices(pGroups, *slice);
+
+	if (!shader.Init(name, uniformNames, attributeNames))
+	{
+		shader.ClearSlices();
+		return false;
+	}
+
+	return true;
+}
+
 CStdGLCtx *CStdGL::CreateContext(C4Window * pWindow, C4AbstractApp *pApp)
 {
-	DebugLog("  gl: Create Context...");
 	// safety
 	if (!pWindow) return NULL;
 
 	// create it
 	CStdGLCtx *pCtx = new CStdGLCtx();
 	bool first_ctx = !pMainCtx;
-	if (first_ctx) pMainCtx = pCtx;
+	if (first_ctx)
+	{
+		pMainCtx = pCtx;
+		Log("  gl: Create first context...");
+	}
 	bool success = pCtx->Init(pWindow, pApp);
 	if (Config.Graphics.DebugOpenGL && glDebugMessageCallbackARB)
 	{
+		if (first_ctx) Log("  gl: Setting OpenGLDebugProc callback");
 		glDebugMessageCallbackARB(&OpenGLDebugProc, nullptr);
 		glEnable(GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB);
+#ifdef GL_KHR_debug
+		if (GLEW_KHR_debug)
+			glEnable(GL_DEBUG_OUTPUT);
+#endif
 	}
 	// First context: Log some information about hardware/drivers
 	// Must log after context creation to get valid results
@@ -260,15 +329,6 @@ CStdGLCtx *CStdGL::CreateContext(C4Window * pWindow, C4AbstractApp *pApp)
 				LogSilentF("GLExt: %s", gl_extensions ? gl_extensions : "");
 			}
 		}
-
-		// Check which workarounds we have to apply
-		{
-			// If we have less than 2048 uniform components available, we
-			// need to upload bone matrices in a different way
-			GLint count;
-			glGetIntegerv(GL_MAX_VERTEX_UNIFORM_COMPONENTS, &count);
-			Workarounds.LowMaxVertexUniformCount = count < 2048;
-		}
 	}
 	if (!success)
 	{
@@ -329,7 +389,7 @@ bool CStdGL::CreatePrimarySurfaces(unsigned int, unsigned int, int iColorDepth,
 	return ok;
 }
 
-void CStdGL::SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform, GLuint baseTex, GLuint overlayTex, GLuint normalTex, DWORD dwOverlayModClr)
+void CStdGL::SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform, GLuint baseTex, GLuint overlayTex, GLuint normalTex, DWORD dwOverlayModClr, StdProjectionMatrix* out_modelview)
 {
 	// Initialize multi blit shader.
 	int iAdditive = dwBlitMode & C4GFXBLIT_ADDITIVE;
@@ -347,16 +407,17 @@ void CStdGL::SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform,
 	};
 
 	call.SetUniform4fv(C4SSU_ClrMod, 1, fMod);
+	call.SetUniform3fv(C4SSU_Gamma, 1, gammaOut);
 
 	if(baseTex != 0)
 	{
-		call.AllocTexUnit(C4SSU_BaseTex, GL_TEXTURE_2D);
+		call.AllocTexUnit(C4SSU_BaseTex);
 		glBindTexture(GL_TEXTURE_2D, baseTex);
 	}
 
 	if(overlayTex != 0)
 	{
-		call.AllocTexUnit(C4SSU_OverlayTex, GL_TEXTURE_2D);
+		call.AllocTexUnit(C4SSU_OverlayTex);
 		glBindTexture(GL_TEXTURE_2D, overlayTex);
 
 		const float fOverlayModClr[4] = {
@@ -371,7 +432,7 @@ void CStdGL::SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform,
 
 	if(pFoW != NULL && normalTex != 0)
 	{
-		call.AllocTexUnit(C4SSU_NormalTex, GL_TEXTURE_2D);
+		call.AllocTexUnit(C4SSU_NormalTex);
 		glBindTexture(GL_TEXTURE_2D, normalTex);
 	}
 
@@ -382,15 +443,15 @@ void CStdGL::SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform,
 		const FLOAT_RECT vpRect = pFoW->getViewportRegion();
 
 		// Dynamic Light
-		call.AllocTexUnit(C4SSU_LightTex, GL_TEXTURE_2D);
-		glBindTexture(GL_TEXTURE_2D, pFoW->getSurface()->textures[0].texName);
+		call.AllocTexUnit(C4SSU_LightTex);
+		glBindTexture(GL_TEXTURE_2D, pFoW->getSurfaceName());
 
 		float lightTransform[6];
 		pFoW->GetFragTransform(ClipRect, OutRect, lightTransform);
 		call.SetUniformMatrix2x3fv(C4SSU_LightTransform, 1, lightTransform);
 
 		// Ambient Light
-		call.AllocTexUnit(C4SSU_AmbientTex, GL_TEXTURE_2D);
+		call.AllocTexUnit(C4SSU_AmbientTex);
 		glBindTexture(GL_TEXTURE_2D, pFoW->getFoW()->Ambient.Tex);
 		call.SetUniform1f(C4SSU_AmbientBrightness, pFoW->getFoW()->Ambient.GetBrightness());
 
@@ -399,73 +460,96 @@ void CStdGL::SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform,
 		call.SetUniformMatrix2x3fv(C4SSU_AmbientTransform, 1, ambientTransform);
 	}
 
+	call.SetUniform1f(C4SSU_CullMode, 0.0f);
+
+	// The primary reason we use a 4x4 matrix for the modelview matrix is that
+	// that the C4BltTransform pTransform parameter can have projection components
+	// (see SetObjDrawTransform2). Still, for sprites the situation is a bit
+	// unsatisfactory because there's no distinction between modelview and projection
+	// components in the BltTransform. Object rotation is part of the BltTransform
+	// for sprites, which should be part of the modelview matrix, so that lighting
+	// is correct for rotated sprites. This is much more common than projection
+	// components in the BltTransform, and therefore we turn the BltTransform into
+	// the modelview matrix and not the projection matrix.
+	StdProjectionMatrix default_modelview = StdProjectionMatrix::Identity();
+	StdProjectionMatrix& modelview = out_modelview ? *out_modelview : default_modelview;
+
 	// Apply zoom and transform
-	glPushMatrix();
-	glTranslatef(ZoomX, ZoomY, 0.0f);
+	Translate(modelview, ZoomX, ZoomY, 0.0f);
 	// Scale Z as well so that we don't distort normals.
-	glScalef(Zoom, Zoom, Zoom);
-	glTranslatef(-ZoomX, -ZoomY, 0.0f);
+	Scale(modelview, Zoom, Zoom, Zoom);
+	Translate(modelview, -ZoomX, -ZoomY, 0.0f);
 
 	if(pTransform)
 	{
-		// Decompose scale factors and scale Z accordingly to X and Y, again to avoid distorting normals
-		// We could instead work around this by using the projection matrix, but then for object rotations (SetR)
-		// the normals would not be correct.
-		const float sx = sqrt(pTransform->mat[0]*pTransform->mat[0] + pTransform->mat[1]*pTransform->mat[1]);
-		const float sy = sqrt(pTransform->mat[3]*pTransform->mat[3] + pTransform->mat[4]*pTransform->mat[4]);
-		const float sz = sqrt(sx * sy);
-		const GLfloat transform[16] = { pTransform->mat[0], pTransform->mat[3], 0, pTransform->mat[6], pTransform->mat[1], pTransform->mat[4], 0, pTransform->mat[7], 0, 0, sz, 0, pTransform->mat[2], pTransform->mat[5], 0, pTransform->mat[8] };
-		glMultMatrixf(transform);
+		float sz = 1.0f;
+		if (pFoW != NULL && normalTex != 0)
+		{
+			// Decompose scale factors and scale Z accordingly to X and Y, again to avoid distorting normals
+			// We could instead work around this by using the projection matrix, but then for object rotations (SetR)
+			// the normals would not be correct.
+			const float sx = sqrt(pTransform->mat[0]*pTransform->mat[0] + pTransform->mat[1]*pTransform->mat[1]);
+			const float sy = sqrt(pTransform->mat[3]*pTransform->mat[3] + pTransform->mat[4]*pTransform->mat[4]);
+			sz = sqrt(sx * sy);
+		}
+
+		// Multiply modelview matrix with transform
+		StdProjectionMatrix transform;
+		transform(0, 0) = pTransform->mat[0];
+		transform(0, 1) = pTransform->mat[1];
+		transform(0, 2) = 0.0f;
+		transform(0, 3) = pTransform->mat[2];
+		transform(1, 0) = pTransform->mat[3];
+		transform(1, 1) = pTransform->mat[4];
+		transform(1, 2) = 0.0f;
+		transform(1, 3) = pTransform->mat[5];
+		transform(2, 0) = 0.0f;
+		transform(2, 1) = 0.0f;
+		transform(2, 2) = sz;
+		transform(2, 3) = 0.0f;
+		transform(3, 0) = pTransform->mat[6];
+		transform(3, 1) = pTransform->mat[7];
+		transform(3, 2) = 0.0f;
+		transform(3, 3) = pTransform->mat[8];
+		modelview *= transform;
 	}
+
+	call.SetUniformMatrix4x4(C4SSU_ProjectionMatrix, ProjectionMatrix);
+	call.SetUniformMatrix4x4(C4SSU_ModelViewMatrix, modelview);
+
+	if (pFoW != NULL && normalTex != 0)
+		call.SetUniformMatrix3x3Transpose(C4SSU_NormalMatrix, StdMeshMatrix::Inverse(StdProjectionMatrix::Upper3x4(modelview)));
 }
 
-void CStdGL::ResetMultiBlt()
+void CStdGL::PerformMultiPix(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, C4ShaderCall* shader_call)
 {
-	glPopMatrix();
-}
-
-void CStdGL::PerformMultiPix(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices)
-{
-	// Only direct rendering
-	assert(sfcTarget->IsRenderTarget());
-	if(!PrepareRendering(sfcTarget)) return;
-
 	// Draw on pixel center:
-	glPushMatrix();
-	glTranslatef(0.5f, 0.5f, 0.0f);
-
-	// Feed the vertices to the GL
-	C4ShaderCall call(GetSpriteShader(false, false, false));
-	SetupMultiBlt(call, NULL, 0, 0, 0, 0);
-
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
+	StdProjectionMatrix transform = StdProjectionMatrix::Translate(0.5f, 0.5f, 0.0f);
 
 	// This is a workaround. Instead of submitting the whole vertex array to the GL, we do it
 	// in batches of 256 vertices. The intel graphics driver on Linux crashes with
 	// significantly larger arrays, such as 400. It's not clear to me why, maybe POINT drawing
 	// is just not very well tested.
 	const unsigned int BATCH_SIZE = 256;
-	for(unsigned int i = 0; i < n_vertices; i += BATCH_SIZE)
+
+	// Feed the vertices to the GL
+	if (!shader_call)
 	{
-		glVertexPointer(2, GL_FLOAT, sizeof(C4BltVertex), &vertices[i].ftx);
-		glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(C4BltVertex), &vertices[i].color[0]);
-		glDrawArrays(GL_POINTS, 0, std::min(n_vertices - i, BATCH_SIZE));
+		C4ShaderCall call(GetSpriteShader(false, false, false));
+		SetupMultiBlt(call, NULL, 0, 0, 0, 0, &transform);
+		for(unsigned int i = 0; i < n_vertices; i += BATCH_SIZE)
+			PerformMultiBlt(sfcTarget, OP_POINTS, &vertices[i], std::min(n_vertices - i, BATCH_SIZE), false, &call);
+	}
+	else
+	{
+		SetupMultiBlt(*shader_call, NULL, 0, 0, 0, 0, &transform);
+		for(unsigned int i = 0; i < n_vertices; i += BATCH_SIZE)
+			PerformMultiBlt(sfcTarget, OP_POINTS, &vertices[i], std::min(n_vertices - i, BATCH_SIZE), false, shader_call);
 	}
-
-	glDisableClientState(GL_VERTEX_ARRAY);
-	glDisableClientState(GL_COLOR_ARRAY);
-	glPopMatrix();
-
-	ResetMultiBlt();
 }
 
-void CStdGL::PerformMultiLines(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, float width)
+void CStdGL::PerformMultiLines(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, float width, C4ShaderCall* shader_call)
 {
-	// Only direct rendering
-	assert(sfcTarget->IsRenderTarget());
-	if(!PrepareRendering(sfcTarget)) return;
-
 	// In a first step, we transform the lines array to a triangle array, so that we can draw
 	// the lines with some thickness.
 	// In principle, this step could be easily (and probably much more efficiently) performed
@@ -509,113 +593,92 @@ void CStdGL::PerformMultiLines(C4Surface* sfcTarget, const C4BltVertex* vertices
 	}
 
 	// Then, feed the vertices to the GL
-	C4ShaderCall call(GetSpriteShader(true, false, false));
-	SetupMultiBlt(call, NULL, lines_tex, 0, 0, 0);
+	if (!shader_call)
+	{
+		C4ShaderCall call(GetSpriteShader(true, false, false));
+		SetupMultiBlt(call, NULL, lines_tex, 0, 0, 0, NULL);
+		PerformMultiBlt(sfcTarget, OP_TRIANGLES, tri_vertices, n_vertices * 3, true, &call);
+	}
+	else
+	{
+		SetupMultiBlt(*shader_call, NULL, lines_tex, 0, 0, 0, NULL);
+		PerformMultiBlt(sfcTarget, OP_TRIANGLES, tri_vertices, n_vertices * 3, true, shader_call);
+	}
 
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
-	glClientActiveTexture(GL_TEXTURE0);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-	glTexCoordPointer(2, GL_FLOAT, sizeof(C4BltVertex), &tri_vertices[0].tx);
-	glVertexPointer(2, GL_FLOAT, sizeof(C4BltVertex), &tri_vertices[0].ftx);
-	glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(C4BltVertex), &tri_vertices[0].color[0]);
-	glDrawArrays(GL_TRIANGLES, 0, n_vertices * 3);
 	delete[] tri_vertices;
-
-	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	glDisableClientState(GL_VERTEX_ARRAY);
-	glDisableClientState(GL_COLOR_ARRAY);
-	ResetMultiBlt();
 }
 
-void CStdGL::PerformMultiTris(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, const C4BltTransform* pTransform, C4TexRef* pTex, C4TexRef* pOverlay, C4TexRef* pNormal, DWORD dwOverlayModClr)
+void CStdGL::PerformMultiTris(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, const C4BltTransform* pTransform, C4TexRef* pTex, C4TexRef* pOverlay, C4TexRef* pNormal, DWORD dwOverlayModClr, C4ShaderCall* shader_call)
+{
+	// Feed the vertices to the GL
+	if (!shader_call)
+	{
+		C4ShaderCall call(GetSpriteShader(pTex != NULL, pOverlay != NULL, pNormal != NULL));
+		SetupMultiBlt(call, pTransform, pTex ? pTex->texName : 0, pOverlay ? pOverlay->texName : 0, pNormal ? pNormal->texName : 0, dwOverlayModClr, NULL);
+		PerformMultiBlt(sfcTarget, OP_TRIANGLES, vertices, n_vertices, pTex != NULL, &call);
+	}
+	else
+	{
+		SetupMultiBlt(*shader_call, pTransform, pTex ? pTex->texName : 0, pOverlay ? pOverlay->texName : 0, pNormal ? pNormal->texName : 0, dwOverlayModClr, NULL);
+		PerformMultiBlt(sfcTarget, OP_TRIANGLES, vertices, n_vertices, pTex != NULL, shader_call);
+	}
+}
+
+void CStdGL::PerformMultiBlt(C4Surface* sfcTarget, DrawOperation op, const C4BltVertex* vertices, unsigned int n_vertices, bool has_tex, C4ShaderCall* shader_call)
 {
 	// Only direct rendering
 	assert(sfcTarget->IsRenderTarget());
 	if(!PrepareRendering(sfcTarget)) return;
 
-	// Feed the vertices to the GL
-	C4ShaderCall call(GetSpriteShader(pTex != NULL, pOverlay != NULL, pNormal != NULL));
-	SetupMultiBlt(call, pTransform, pTex ? pTex->texName : 0, pOverlay ? pOverlay->texName : 0, pNormal ? pNormal->texName : 0, dwOverlayModClr);
+	// Select a buffer
+	const unsigned int vbo_index = CurrentVBO;
+	CurrentVBO = (CurrentVBO + 1) % N_GENERIC_VBOS;
 
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_COLOR_ARRAY);
-
-	if(pTex)
+	// Upload data into the buffer, resize buffer if necessary
+	glBindBuffer(GL_ARRAY_BUFFER, GenericVBOs[vbo_index]);
+	if (GenericVBOSizes[vbo_index] < n_vertices)
 	{
-		// We use the texture coordinate array in texture0 for
-		// the base, overlay and normal textures
-		glClientActiveTexture(GL_TEXTURE0);
-		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-		glTexCoordPointer(2, GL_FLOAT, sizeof(C4BltVertex), &vertices[0].tx);
+		GenericVBOSizes[vbo_index] = n_vertices;
+		glBufferData(GL_ARRAY_BUFFER, n_vertices * sizeof(C4BltVertex), vertices, GL_STREAM_DRAW);
+	}
+	else
+	{
+		glBufferSubData(GL_ARRAY_BUFFER, 0, n_vertices * sizeof(C4BltVertex), vertices);
 	}
 
-	glVertexPointer(2, GL_FLOAT, sizeof(C4BltVertex), &vertices[0].ftx);
-	glColorPointer(4, GL_UNSIGNED_BYTE, sizeof(C4BltVertex), &vertices[0].color[0]);
-	glDrawArrays(GL_TRIANGLES, 0, n_vertices);
+	const GLuint position = shader_call->GetAttribute(C4SSA_Position);
+	const GLuint color = shader_call->GetAttribute(C4SSA_Color);
+	const GLuint texcoord = has_tex ? shader_call->GetAttribute(C4SSA_TexCoord) : 0;
 
-	if(pTex) glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	glDisableClientState(GL_VERTEX_ARRAY);
-	glDisableClientState(GL_COLOR_ARRAY);
+	glEnableVertexAttribArray(position);
+	glEnableVertexAttribArray(color);
 
-	ResetMultiBlt();
-}
-
-bool CStdGL::CreateSpriteShader(C4Shader& shader, const char* name, int ssc, C4GroupSet* pGroups)
-{
-	static const char vertexSlice[] = 
-		"  gl_FrontColor = gl_Color;"
-		"  gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;";
-
-	const char* uniformNames[C4SSU_Count + 1];
-	uniformNames[C4SSU_ClrMod] = "clrMod";
-	uniformNames[C4SSU_BaseTex] = "baseTex";
-	uniformNames[C4SSU_OverlayTex] = "overlayTex";
-	uniformNames[C4SSU_OverlayClr] = "overlayClr";
-	uniformNames[C4SSU_LightTex] = "lightTex";
-	uniformNames[C4SSU_LightTransform] = "lightTransform";
-	uniformNames[C4SSU_NormalTex] = "normalTex";
-	uniformNames[C4SSU_AmbientTex] = "ambientTex";
-	uniformNames[C4SSU_AmbientTransform] = "ambientTransform";
-	uniformNames[C4SSU_AmbientBrightness] = "ambientBrightness";
-	uniformNames[C4SSU_Bones] = "bones";
-	uniformNames[C4SSU_Count] = NULL;
-
-	// Clear previous content
-	shader.Clear();
-	shader.ClearSlices();
-
-	shader.AddVertexSlice(C4Shader_Vertex_PositionPos, vertexSlice);
-
-	// Add texture coordinate if we have base texture, overlay, or normal map
-	if ( (ssc & (C4SSC_BASE | C4SSC_OVERLAY | C4SSC_NORMAL)) != 0)
-		shader.AddTexCoord("texcoord");
-
-	// Then load slices for fragment shader
-	shader.AddFragmentSlice(-1, "#define OPENCLONK");
-	if (ssc & C4SSC_MOD2) shader.AddFragmentSlice(-1, "#define CLRMOD_MOD2");
-	if (ssc & C4SSC_NORMAL) shader.AddFragmentSlice(-1, "#define HAVE_NORMALMAP");
-	if (ssc & C4SSC_LIGHT) shader.AddFragmentSlice(-1, "#define HAVE_LIGHT");
-	shader.LoadSlices(pGroups, "UtilShader.glsl");
-	shader.LoadSlices(pGroups, "ObjectBaseShader.glsl");
-
-	if (ssc & C4SSC_BASE) shader.LoadSlices(pGroups, "SpriteTextureShader.glsl");
-	if (ssc & C4SSC_OVERLAY) shader.LoadSlices(pGroups, "SpriteOverlayShader.glsl");
-
-	// In case light is disabled, these shaders use a default light source
-	// (typically ambient light everywhere).
-	shader.LoadSlices(pGroups, "ObjectLightShader.glsl");
-	shader.LoadSlices(pGroups, "LightShader.glsl");
-	shader.LoadSlices(pGroups, "AmbientShader.glsl");
-
-	if (!shader.Init(name, uniformNames))
+	if(has_tex)
 	{
-		shader.ClearSlices();
-		return false;
+		glEnableVertexAttribArray(texcoord);
+		glVertexAttribPointer(texcoord, 2, GL_FLOAT, GL_FALSE, sizeof(C4BltVertex), reinterpret_cast<const uint8_t*>(offsetof(C4BltVertex, tx)));
 	}
 
-	return true;
+	glVertexAttribPointer(position, 2, GL_FLOAT, GL_FALSE, sizeof(C4BltVertex), reinterpret_cast<const uint8_t*>(offsetof(C4BltVertex, ftx)));
+	glVertexAttribPointer(color, 4, GL_UNSIGNED_BYTE, GL_TRUE, sizeof(C4BltVertex), reinterpret_cast<const uint8_t*>(offsetof(C4BltVertex, color)));
+
+	switch (op)
+	{
+	case OP_POINTS:
+		glDrawArrays(GL_POINTS, 0, n_vertices);
+		break;
+	case OP_TRIANGLES:
+		glDrawArrays(GL_TRIANGLES, 0, n_vertices);
+		break;
+	default:
+		assert(false);
+		break;
+	}
+
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	if(has_tex) glDisableVertexAttribArray(texcoord);
+	glDisableVertexAttribArray(position);
+	glDisableVertexAttribArray(color);
 }
 
 C4Shader* CStdGL::GetSpriteShader(bool haveBase, bool haveOverlay, bool haveNormal)
@@ -673,38 +736,38 @@ C4Shader* CStdGL::GetSpriteShader(int ssc)
 bool CStdGL::InitShaders(C4GroupSet* pGroups)
 {
 	// Create sprite blitting shaders
-	if(!CreateSpriteShader(SpriteShader, "sprite", 0, pGroups))
+	if(!PrepareSpriteShader(SpriteShader, "sprite", 0, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderMod2, "spriteMod2", C4SSC_MOD2, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderMod2, "spriteMod2", C4SSC_MOD2, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderBase, "spriteBase", C4SSC_BASE, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderBase, "spriteBase", C4SSC_BASE, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderBaseMod2, "spriteBaseMod2", C4SSC_MOD2 | C4SSC_BASE, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderBaseMod2, "spriteBaseMod2", C4SSC_MOD2 | C4SSC_BASE, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderBaseOverlay, "spriteBaseOverlay", C4SSC_BASE | C4SSC_OVERLAY, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderBaseOverlay, "spriteBaseOverlay", C4SSC_BASE | C4SSC_OVERLAY, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderBaseOverlayMod2, "spriteBaseOverlayMod2", C4SSC_MOD2 | C4SSC_BASE | C4SSC_OVERLAY, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderBaseOverlayMod2, "spriteBaseOverlayMod2", C4SSC_MOD2 | C4SSC_BASE | C4SSC_OVERLAY, pGroups, NULL, NULL))
 		return false;
 
-	if(!CreateSpriteShader(SpriteShaderLight, "spriteLight", C4SSC_LIGHT, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLight, "spriteLight", C4SSC_LIGHT, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightMod2, "spriteLightMod2", C4SSC_LIGHT | C4SSC_MOD2, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightMod2, "spriteLightMod2", C4SSC_LIGHT | C4SSC_MOD2, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBase, "spriteLightBase", C4SSC_LIGHT | C4SSC_BASE, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBase, "spriteLightBase", C4SSC_LIGHT | C4SSC_BASE, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBaseMod2, "spriteLightBaseMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_MOD2, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBaseMod2, "spriteLightBaseMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_MOD2, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBaseOverlay, "spriteLightBaseOverlay", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBaseOverlay, "spriteLightBaseOverlay", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBaseOverlayMod2, "spriteLightBaseOverlayMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY | C4SSC_MOD2, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBaseOverlayMod2, "spriteLightBaseOverlayMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY | C4SSC_MOD2, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBaseNormal, "spriteLightBaseNormal", C4SSC_LIGHT | C4SSC_BASE | C4SSC_NORMAL, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBaseNormal, "spriteLightBaseNormal", C4SSC_LIGHT | C4SSC_BASE | C4SSC_NORMAL, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBaseNormalMod2, "spriteLightBaseNormalMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_NORMAL | C4SSC_MOD2, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBaseNormalMod2, "spriteLightBaseNormalMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_NORMAL | C4SSC_MOD2, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBaseNormalOverlay, "spriteLightBaseNormalOverlay", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY | C4SSC_NORMAL, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBaseNormalOverlay, "spriteLightBaseNormalOverlay", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY | C4SSC_NORMAL, pGroups, NULL, NULL))
 		return false;
-	if(!CreateSpriteShader(SpriteShaderLightBaseNormalOverlayMod2, "spriteLightBaseNormalOverlayMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY | C4SSC_NORMAL | C4SSC_MOD2, pGroups))
+	if(!PrepareSpriteShader(SpriteShaderLightBaseNormalOverlayMod2, "spriteLightBaseNormalOverlayMod2", C4SSC_LIGHT | C4SSC_BASE | C4SSC_OVERLAY | C4SSC_NORMAL | C4SSC_MOD2, pGroups, NULL, NULL))
 		return false;
 
 	return true;
@@ -729,16 +792,24 @@ bool CStdGL::RestoreDeviceObjects()
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	const char * linedata = byByteCnt == 2 ? "\xff\xf0\xff\xff" : "\xff\xff\xff\x00\xff\xff\xff\xff";
-	glTexImage2D(GL_TEXTURE_2D, 0, 4, 1, 2, 0, GL_BGRA, byByteCnt == 2 ? GL_UNSIGNED_SHORT_4_4_4_4_REV : GL_UNSIGNED_INT_8_8_8_8_REV, linedata);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1, 2, 0, GL_BGRA, byByteCnt == 2 ? GL_UNSIGNED_SHORT_4_4_4_4_REV : GL_UNSIGNED_INT_8_8_8_8_REV, linedata);
 
 	MaxTexSize = 64;
 	GLint s = 0;
 	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &s);
 	if (s>0) MaxTexSize = s;
 
-	// restore gamma if active
-	if (Active)
-		EnableGamma();
+	// Generic VBOs
+	glGenBuffers(N_GENERIC_VBOS, GenericVBOs);
+	for (unsigned int i = 0; i < N_GENERIC_VBOS; ++i)
+	{
+		GenericVBOSizes[i] = GENERIC_VBO_SIZE;
+		glBindBuffer(GL_ARRAY_BUFFER, GenericVBOs[i]);
+		glBufferData(GL_ARRAY_BUFFER, GenericVBOSizes[i] * sizeof(C4BltVertex), NULL, GL_STREAM_DRAW);
+	}
+
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+
 	// reset blit states
 	dwBlitMode = 0;
 
@@ -749,10 +820,6 @@ bool CStdGL::RestoreDeviceObjects()
 bool CStdGL::InvalidateDeviceObjects()
 {
 	bool fSuccess=true;
-	// clear gamma
-#ifndef USE_SDL_MAINLOOP
-	DisableGamma();
-#endif
 	// deactivate
 	Active=false;
 	// invalidate font objects
@@ -762,6 +829,34 @@ bool CStdGL::InvalidateDeviceObjects()
 		glDeleteTextures(1, &lines_tex);
 		lines_tex = 0;
 	}
+
+	// invalidate generic VBOs
+	if (GenericVBOs[0] != 0)
+		glDeleteBuffers(N_GENERIC_VBOS, GenericVBOs);
+	GenericVBOs[0] = 0;
+	CurrentVBO = 0;
+
+	// invalidate shaders
+
+	// TODO: We don't do this here because we cannot re-validate them in
+	// RestoreDeviceObjects. This should be refactored.
+
+	/*SpriteShader.Clear();
+	SpriteShaderMod2.Clear();
+	SpriteShaderBase.Clear();
+	SpriteShaderBaseMod2.Clear();
+	SpriteShaderBaseOverlay.Clear();
+	SpriteShaderBaseOverlayMod2.Clear();
+	SpriteShaderLight.Clear();
+	SpriteShaderLightMod2.Clear();
+	SpriteShaderLightBase.Clear();
+	SpriteShaderLightBaseMod2.Clear();
+	SpriteShaderLightBaseOverlay.Clear();
+	SpriteShaderLightBaseOverlayMod2.Clear();
+	SpriteShaderLightBaseNormal.Clear();
+	SpriteShaderLightBaseNormalMod2.Clear();
+	SpriteShaderLightBaseNormalOverlay.Clear();
+	SpriteShaderLightBaseNormalOverlayMod2.Clear();*/
 	return fSuccess;
 }
 
@@ -792,23 +887,28 @@ bool CStdGL::Error(const char *szMsg)
 	return r;
 }
 
+const char* CStdGL::GLErrorString(GLenum code)
+{
+	switch (code)
+	{
+	case GL_NO_ERROR: return "No error";
+	case GL_INVALID_ENUM: return "An unacceptable value is specified for an enumerated argument";
+	case GL_INVALID_VALUE: return "A numeric argument is out of range";
+	case GL_INVALID_OPERATION: return "The specified operation is not allowed in the current state";
+	case GL_INVALID_FRAMEBUFFER_OPERATION: return "The framebuffer object is not complete";
+	case GL_OUT_OF_MEMORY: return "There is not enough memory left to execute the command";
+	case GL_STACK_UNDERFLOW: return "An attempt has been made to perform an operation that would cause an internal stack to underflow";
+	case GL_STACK_OVERFLOW: return "An attempt has been made to perform an operation that would cause an internal stack to overflow";
+	default: assert(false); return "";
+	}
+}
+
 bool CStdGL::CheckGLError(const char *szAtOp)
 {
 	GLenum err = glGetError();
 	if (!err) return true;
 
-#ifdef USE_WIN32_WINDOWS
-	StdStrBuf err_buf(gluErrorUnicodeStringEXT(err));
-#else
-	// gluErrorString returns latin-1 strings. Our code expects UTF-8, so convert
-	// Also for some reason gluErrorString returns const GLubyte* instead of a more
-	// reasonable const char *, so cast it - C-style cast required here to match
-	// both unsigned and signed char
-	StdStrBuf err_buf((const char*)gluErrorString(err));
-	err_buf.EnsureUnicode();
-#endif
-
-	LogF("GL error with %s: %d - %s", szAtOp, err, err_buf.getData());
+	LogF("GL error with %s: %d - %s", szAtOp, err, GLErrorString(err));
 	return false;
 }
 
