@@ -23,6 +23,7 @@
 #ifdef _WIN32
 #include <C4windowswrapper.h>
 #endif
+
 #include <GL/glew.h>
 
 #ifdef USE_COCOA
@@ -45,17 +46,16 @@ private:
 	StdCopyStrBuf Buf;
 };
 
-// Shader combinations
-static const int C4SSC_MOD2 = 1; // signed addition instead of multiplication for clrMod
-static const int C4SSC_BASE = 2; // use a base texture instead of just a single color
-static const int C4SSC_OVERLAY = 4; // use a colored overlay on top of base texture
-static const int C4SSC_LIGHT = 8; // use dynamic+ambient lighting
-static const int C4SSC_NORMAL = 16; // extract normals from normal map instead of (0,0,1)
-
 // Uniform data we give the sprite shader (constants from its viewpoint)
 enum C4SS_Uniforms
 {
+	C4SSU_ProjectionMatrix, // 4x4
+	C4SSU_ModelViewMatrix,  // 4x4
+	C4SSU_NormalMatrix,     // 3x3, transpose-inverse of modelview matrix
+
 	C4SSU_ClrMod, // always
+	C4SSU_Gamma, // always
+
 	C4SSU_BaseTex, // C4SSC_BASE
 	C4SSU_OverlayTex, // C4SSC_OVERLAY
 	C4SSU_OverlayClr, // C4SSC_OVERLAY
@@ -69,10 +69,28 @@ enum C4SS_Uniforms
 	C4SSU_AmbientBrightness, // C4SSC_LIGHT
 
 	C4SSU_Bones, // for meshes
+	C4SSU_CullMode, // for meshes
 
 	C4SSU_Count
 };
 
+// Attribute data for sprites and meshes
+enum C4SS_Attributes
+{
+	C4SSA_Position, // 2d for sprites, 3d for meshes
+	C4SSA_Normal,  // meshes only
+	C4SSA_TexCoord, // 2d
+	C4SSA_Color,    // sprites only, 4d
+
+	C4SSA_BoneIndices0,
+	C4SSA_BoneIndices1,
+
+	C4SSA_BoneWeights0,
+	C4SSA_BoneWeights1,
+
+	C4SSA_Count
+};
+
 // one OpenGL context
 class CStdGLCtx
 #ifdef USE_COCOA
@@ -97,12 +115,14 @@ public:
 
 	bool PageFlip();            // present scene
 
+	static void Reinitialize();
+
 protected:
 	void SelectCommon();
 	// this handles are declared as pointers to structs
 	C4Window * pWindow; // window to draw in
 #ifdef USE_WIN32_WINDOWS
-	HGLRC hrc;                  // rendering context
+	static HGLRC hrc;                  // rendering context
 	HWND hWindow; // used if pWindow==NULL
 	HDC hDC;                    // device context handle
 	static bool InitGlew(HINSTANCE hInst);
@@ -130,6 +150,10 @@ protected:
 	int iClrDpt;                // color depth
 	// texture for smooth lines
 	GLuint lines_tex;
+
+	// The orthographic projection matrix
+	StdProjectionMatrix ProjectionMatrix;
+
 	// programs for drawing points, lines, quads
 
 	// Sprite shaders -- there is a variety of shaders to avoid
@@ -151,31 +175,46 @@ protected:
 	C4Shader SpriteShaderLightBaseNormalMod2;
 	C4Shader SpriteShaderLightBaseNormalOverlay;
 	C4Shader SpriteShaderLightBaseNormalOverlayMod2;
+
+	// Generic VBOs for rendering arbitrary points, lines and
+	// triangles, used by PerformMultiBlt. Use more than one VBO, so that
+	// two PerformMultiBlt calls in quick succession can use two different
+	// buffers. Otherwise, the second call would need to wait until the
+	// rendering pipeline has actually drained the buffer. Each buffer
+	// starts with a fixed size. If more than this many vertices need to
+	// be rendered (can happen e.g. for PXS), then the buffer is resized.
+	static const unsigned int N_GENERIC_VBOS = 16;
+	static const unsigned int GENERIC_VBO_SIZE = 3 * 64; // vertices
+	GLuint GenericVBOs[N_GENERIC_VBOS];
+	unsigned int GenericVBOSizes[N_GENERIC_VBOS];
+	unsigned int CurrentVBO;
+
 public:
 	// General
 	void Clear();
 	void Default();
-	virtual bool IsOpenGL() { return true; }
-	virtual bool IsShaderific() { return true; }
 	virtual bool OnResolutionChanged(unsigned int iXRes, unsigned int iYRes); // reinit clipper for new resolution
 	// Clipper
 	bool UpdateClipper(); // set current clipper to render target
-	bool PrepareMaterial(StdMeshMatManager& mat_manager, StdMeshMaterialLoader& loader, StdMeshMaterial& mat);
+	const StdProjectionMatrix& GetProjectionMatrix() const { return ProjectionMatrix; }
+	virtual bool PrepareMaterial(StdMeshMatManager& mat_manager, StdMeshMaterialLoader& loader, StdMeshMaterial& mat);
 	// Surface
-	bool PrepareRendering(C4Surface * sfcToSurface); // check if/make rendering possible to given surface
+	virtual bool PrepareRendering(C4Surface * sfcToSurface); // check if/make rendering possible to given surface
+	virtual bool PrepareSpriteShader(C4Shader& shader, const char* name, int ssc, C4GroupSet* pGroups, const char* const* additionalDefines, const char* const* additionalSlices);
+
 	virtual CStdGLCtx *CreateContext(C4Window * pWindow, C4AbstractApp *pApp);
 #ifdef USE_WIN32_WINDOWS
 	virtual CStdGLCtx *CreateContext(HWND hWindow, C4AbstractApp *pApp);
 #endif
 	// Blit
-	void SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform, GLuint baseTex, GLuint overlayTex, GLuint normalTex, DWORD dwOverlayModClr);
-	void ResetMultiBlt();
+	void SetupMultiBlt(C4ShaderCall& call, const C4BltTransform* pTransform, GLuint baseTex, GLuint overlayTex, GLuint normalTex, DWORD dwOverlayModClr, StdProjectionMatrix* out_modelview);
 	virtual void PerformMesh(StdMeshInstance &instance, float tx, float ty, float twdt, float thgt, DWORD dwPlayerColor, C4BltTransform* pTransform);
 	void FillBG(DWORD dwClr=0);
 	// Drawing
-	virtual void PerformMultiPix(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices);
-	virtual void PerformMultiLines(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, float width);
-	virtual void PerformMultiTris(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, const C4BltTransform* pTransform, C4TexRef* pTex, C4TexRef* pOverlay, C4TexRef* pNormal, DWORD dwOverlayClrMod);
+	virtual void PerformMultiPix(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, C4ShaderCall* shader_call);
+	virtual void PerformMultiLines(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, float width, C4ShaderCall* shader_call);
+	virtual void PerformMultiTris(C4Surface* sfcTarget, const C4BltVertex* vertices, unsigned int n_vertices, const C4BltTransform* pTransform, C4TexRef* pTex, C4TexRef* pOverlay, C4TexRef* pNormal, DWORD dwOverlayClrMod, C4ShaderCall* shader_call);
+	void PerformMultiBlt(C4Surface* sfcTarget, DrawOperation op, const C4BltVertex* vertices, unsigned int n_vertices, bool has_tex, C4ShaderCall* shader_call);
 	// device objects
 	bool RestoreDeviceObjects();    // restore device dependent objects
 	bool InvalidateDeviceObjects(); // free device dependent objects
@@ -193,10 +232,9 @@ protected:
 	bool CreatePrimarySurfaces(unsigned int iXRes, unsigned int iYRes, int iColorDepth, unsigned int iMonitor);
 
 	bool CheckGLError(const char *szAtOp);
+	const char* GLErrorString(GLenum code);
 	virtual bool Error(const char *szMsg);
 
-	bool CreateSpriteShader(C4Shader& shader, const char* name, int ssc, C4GroupSet* pGroups);
-
 	friend class C4Surface;
 	friend class C4TexRef;
 	friend class C4Pattern;
@@ -205,6 +243,7 @@ protected:
 	friend class C4FullScreen;
 	friend class C4Window;
 	friend class C4ShaderCall;
+	friend class C4FoWRegion;
 };
 
 // Global access pointer
