@@ -403,7 +403,7 @@ void C4Facet::DrawEnergyLevelEx(int32_t iLevel, int32_t iRange, const C4Facet &g
 	// draw energy level using graphics
 	if (!pDraw || !gfx.Surface) return;
 	int32_t h=gfx.Hgt;
-	int32_t yBar = Hgt - Clamp<int32_t>(iLevel,0,iRange) * Hgt / Max<int32_t>(iRange,1);
+	int32_t yBar = Hgt - Clamp<int32_t>(iLevel,0,iRange) * Hgt / std::max<int32_t>(iRange,1);
 	int32_t iY = 0, vidx=0;
 	C4Facet gfx_draw = gfx;
 	bool filled = false;
@@ -460,13 +460,6 @@ void C4Facet::Set(C4Surface &rSfc)
 	Set(&rSfc,0,0,rSfc.Wdt,rSfc.Hgt);
 }
 
-void C4Facet::DrawTile(C4Surface * sfcTarget, int32_t iX, int32_t iY, int32_t iWdt, int32_t iHgt)
-{
-	if (!pDraw || !Surface || !Wdt || !Hgt) return;
-	// Blits whole source surface, not surface facet area
-	pDraw->BlitSurfaceTile(Surface,sfcTarget,iX,iY,iWdt,iHgt,0,0,true);
-}
-
 void C4Facet::Expand(int32_t iLeft, int32_t iRight, int32_t iTop, int32_t iBottom)
 {
 	X-=iLeft; Wdt+=iLeft;
@@ -564,7 +557,7 @@ C4Facet C4Facet::GetFraction(int32_t percentWdt, int32_t percentHgt, int32_t ali
 	// Simple spec for square fractions
 	if (percentHgt == 0) percentHgt = percentWdt;
 	// Alignment
-	int iX = X, iY = Y, iWdt = Max(Wdt*percentWdt/100, 1.0f), iHgt = Max(Hgt*percentHgt/100, 1.0f);
+	int iX = X, iY = Y, iWdt = std::max(Wdt*percentWdt/100, 1.0f), iHgt = std::max(Hgt*percentHgt/100, 1.0f);
 	if (alignX & C4FCT_Right) iX += Wdt - iWdt;
 	if (alignX & C4FCT_Center)  iX += Wdt/2 - iWdt/2;
 	if (alignY & C4FCT_Bottom) iY += Hgt - iHgt;
