@@ -46,12 +46,12 @@ public:
 		// set pix in local copy without bounds or surface checks
 		Bits[iY*Pitch+iX]=byCol;
 	}
-	BYTE GetPix(int iX, int iY) // get pixel
+	BYTE GetPix(int iX, int iY) const // get pixel
 	{
 		if (iX<0 || iY<0 || iX>=Wdt || iY>=Hgt) return 0;
 		return Bits ? Bits[iY*Pitch+iX] : 0;
 	}
-	inline BYTE _GetPix(int x, int y) // get pixel (bounds not checked)
+	inline BYTE _GetPix(int x, int y) const // get pixel (bounds not checked)
 	{
 		return Bits[y*Pitch+x];
 	}
