@@ -44,21 +44,36 @@ void C4GameMessage::Init(int32_t iType, const StdStrBuf & sText, C4Object *pTarg
 	// Set data
 	Text.Copy(sText);
 	Target=pTarget;
-	X=iX; Y=iY; Wdt=width;
+	X=iX; Y=iY; Wdt=width; Hgt=0;
 	Player=iPlayer;
 	ColorDw=dwClr;
 	Type=iType;
-	Delay=Max<int32_t>(C4GM_MinDelay, Text.getLength() * (Target ? ObjectMsgDelayFactor : GlobalMsgDelayFactor));
+	Delay=std::max<int32_t>(C4GM_MinDelay, Text.getLength() * (Target ? ObjectMsgDelayFactor : GlobalMsgDelayFactor));
 	DecoID=idDecoID;
 	this->dwFlags=dwFlags;
 	PictureDef=NULL;
 	PictureDefVal.Set0();
 	if (pSrc)
-		if (pSrc->GetDef() || pSrc->GetObject() || pSrc->GetPropertyPropList(P_Source))
+	{
+		// retain special width/height properties when using a message box on an object-local message
+		if (Target)
+		{
+			C4Value val;
+			if (pSrc->GetProperty(P_Wdt, &val))
+				Wdt = val.getInt();
+			if (pSrc->GetProperty(P_Hgt, &val))
+				Hgt = val.getInt();
+		}
+
+		// retain object or definition from the proplist
+		PictureDef = pSrc->GetObject();
+		if (!PictureDef) PictureDef = pSrc->GetDef();
+		if (!PictureDef && pSrc->GetPropertyPropList(P_Source))
 		{
 			PictureDef = pSrc;
 			PictureDefVal.SetPropList(pSrc);
 		}
+	}
 	// Permanent message
 	if ('@' == Text[0])
 	{
@@ -93,10 +108,6 @@ void C4GameMessage::Append(const char *szText, bool fNoDuplicates)
 
 bool C4GameMessage::Execute()
 {
-	// Position by target
-	// currently done in C4GameMessage::Draw for parallaxity
-	/*if (Target)
-	  { X=Target->x; Y=Target->y-Target->Def->Shape.Hgt/2-5; }*/
 	// Delay / removal
 	if (Delay>0) Delay--;
 	if (Delay==0) return false;
@@ -124,9 +135,9 @@ void C4GameMessage::Draw(C4TargetFacet &cgo, int32_t iPlayer)
 			// Word wrap to cgo width
 			if (PictureDef)
 			{
-				if (!wdt) wdt = Clamp<int32_t>(cgo.Wdt/2, 50, Min<int32_t>(500, cgo.Wdt-10));
+				if (!wdt) wdt = Clamp<int32_t>(cgo.Wdt/2, 50, std::min<int32_t>(500, cgo.Wdt-10));
 				int32_t iUnbrokenTextWidth = ::GraphicsResource.FontRegular.GetTextWidth(Text.getData(), true);
-				wdt = Min<int32_t>(wdt, iUnbrokenTextWidth+10);
+				wdt = std::min<int32_t>(wdt, iUnbrokenTextWidth+10);
 			}
 			else
 			{
@@ -158,7 +169,7 @@ void C4GameMessage::Draw(C4TargetFacet &cgo, int32_t iPlayer)
 			if (pFrameDeco)
 			{
 				iDrawX *= cgo.Zoom; iDrawY *= cgo.Zoom;
-				C4Rect rect(iDrawX-cgo.TargetX, iDrawY-cgo.TargetY, iTextWdt + PictureWidth + PictureIndent + pFrameDeco->iBorderLeft + pFrameDeco->iBorderRight, Max(iTextHgt, PictureWidth) + pFrameDeco->iBorderTop + pFrameDeco->iBorderBottom);
+				C4Rect rect(iDrawX-cgo.TargetX, iDrawY-cgo.TargetY, iTextWdt + PictureWidth + PictureIndent + pFrameDeco->iBorderLeft + pFrameDeco->iBorderRight, std::max(iTextHgt, PictureWidth) + pFrameDeco->iBorderTop + pFrameDeco->iBorderBottom);
 				if (dwFlags & C4GM_Bottom) { rect.y -= rect.Hgt; iDrawY -= rect.Hgt; }
 				else if (dwFlags & C4GM_VCenter) { rect.y -= rect.Hgt/2; iDrawY -= rect.Hgt/2; }
 				if (dwFlags & C4GM_Right) { rect.x -= rect.Wdt; iDrawX -= rect.Wdt; }
@@ -171,7 +182,7 @@ void C4GameMessage::Draw(C4TargetFacet &cgo, int32_t iPlayer)
 				iDrawY -= iTextHgt;
 			// draw picture
 			// can only be def or object because has been checked on assignment
-			
+
 			C4Facet facet(cgo.Surface, iDrawX, iDrawY, PictureWidth, PictureWidth);
 			if(PictureDef->GetObject())
 				PictureDef->GetObject()->DrawPicture(facet);
@@ -198,50 +209,88 @@ void C4GameMessage::Draw(C4TargetFacet &cgo, int32_t iPlayer)
 		// adjust position by object; care about parallaxity
 		float iMsgX, iMsgY, newzoom;
 		Target->GetDrawPosition(cgo, iMsgX, iMsgY, newzoom);
-		iMsgY -= Target->Def->Shape.Hgt/2+5;
+		if(~dwFlags & C4GM_YRel)
+			iMsgY -= Target->Def->Shape.Hgt/2+5;
 		iMsgX+=X; iMsgY+=Y;
 		// check output bounds
-		if (Inside<float>((iMsgX - cgo.X) * newzoom, 0, cgo.Wdt * cgo.Zoom - 1))
-			if (Inside<float>((iMsgY - cgo.Y) * newzoom, 0, cgo.Hgt * cgo.Zoom - 1))
+		if (Inside<float>((iMsgX - cgo.X) * newzoom, 0, cgo.Wdt * cgo.Zoom - 1) || (dwFlags & C4GM_XRel))
+			if (Inside<float>((iMsgY - cgo.Y) * newzoom, 0, cgo.Hgt * cgo.Zoom - 1) || (dwFlags & C4GM_YRel))
 			{
 				// if the message is attached to an object and the object
 				// is invisible for that player, the message won't be drawn
 				if (Type == C4GM_Target)
 					if (!Target->IsVisible(iPlayer, false))
 						return;
-				// check fog of war
-				C4Player *pPlr = ::Players.Get(iPlayer);
-				if (pPlr && pPlr->fFogOfWar)
-				{
-					// TODO: Should we render the message with a shader to apply lighting to it?
-					// Otherwise use pDraw->GetFoW() to check for visibility...
-					//if (!pPlr->FoWIsVisible(iMsgX + cgo.TargetX - cgo.X, iMsgY + cgo.TargetY - cgo.Y))
-					if(false)
-					{
-						// special: Target objects that ignore FoW should display the message even if within FoW
-						if (Type != C4GM_Target && Type != C4GM_TargetPlayer) return;
-						if (~Target->Category & C4D_IgnoreFoW) return;
-					}
-				}
 				// Word wrap to cgo width
 				StdStrBuf sText;
 				if (~dwFlags & C4GM_NoBreak)
-					::GraphicsResource.FontRegular.BreakMessage(Text.getData(), Clamp<int32_t>(cgo.Wdt * cgo.Zoom, 50, 200), &sText, true);
+				{
+					// standard break width
+					int breakWdt = Clamp<int32_t>(cgo.Wdt * cgo.Zoom, 50, 200);
+
+					// user supplied width?
+					if (Wdt)
+						breakWdt = Wdt;
+
+					::GraphicsResource.FontRegular.BreakMessage(Text.getData(), breakWdt, &sText, true);
+				}
 				else
 					sText.Ref(Text);
-				// Adjust position by output boundaries
-				float iTX,iTY;
-				int iTWdt,iTHgt;
+
+				// vertical placement
+				if (dwFlags & C4GM_Bottom)
+					iMsgY += Hgt; // iTHgt will be substracted below
+				else if (dwFlags & C4GM_Top)
+					;
+				else if (dwFlags & C4GM_VCenter)
+					iMsgY += Hgt / 2;
+
+				// horizontal placement
+				int alignment = ACenter;
+
+				if (dwFlags & C4GM_Left)
+					alignment = ALeft;
+				else if (dwFlags & C4GM_Right)
+				{
+					alignment = ARight;
+					iMsgX += Wdt;
+				}
+				else if (dwFlags & C4GM_HCenter)
+					iMsgX += Wdt / 2;
+
+				// calculate display position and adjust position by output boundaries
+				float iTX, iTY;
+				iTX = (iMsgX - cgo.X) * newzoom;
+				iTY = (iMsgY - cgo.Y) * newzoom;
+				int iTWdt, iTHgt;
 				::GraphicsResource.FontRegular.GetTextExtent(sText.getData(),iTWdt,iTHgt,true);
+
+				// adjust zoom if needed
+				float zoom = 1.0;
+				if(dwFlags & C4GM_Zoom)
+					zoom = cgo.Zoom;
+
+				if (dwFlags & C4GM_Bottom)
+					iTY -= zoom * float(iTHgt);
+				else if (dwFlags & C4GM_VCenter)
+					iTY -= zoom * float(iTHgt/2);
+				else if (~dwFlags & C4GM_Top)
+					iTY -= zoom * float(iTHgt); // above object is standard
+
+				if (dwFlags & C4GM_Right)
+					iTX += 0.25f * float(iTWdt) * (1.0f - zoom);
+
+				// adjustment for objects at border of screen?
 				// +0.5f for proper rounding; avoids oscillations near pixel border:
-				iTX = Clamp<float>((iMsgX - cgo.X) * newzoom, iTWdt/2, cgo.Wdt * cgo.Zoom - iTWdt / 2) + 0.5f;
-				iTY = Clamp<float>((iMsgY - cgo.Y) * newzoom - iTHgt, 0, cgo.Hgt * cgo.Zoom - iTHgt) + 0.5f;
+				if (~dwFlags & C4GM_XRel) iTX = Clamp<float>(iTX, iTWdt/2, cgo.Wdt * cgo.Zoom - iTWdt / 2) + 0.5f;
+				if (~dwFlags & C4GM_YRel) iTY = Clamp<float>(iTY, 0, cgo.Hgt * cgo.Zoom - iTHgt) + 0.5f;
+
 				// Draw
-				pDraw->TextOut(sText.getData(), ::GraphicsResource.FontRegular, 1.0,
+				pDraw->TextOut(sText.getData(), ::GraphicsResource.FontRegular, zoom,
 				                           cgo.Surface,
 				                           cgo.X + iTX,
 				                           cgo.Y + iTY,
-				                           ColorDw,ACenter);
+				                           ColorDw, alignment);
 				return;
 			}
 	}
