@@ -261,7 +261,7 @@ namespace C4GUI
 		if (pActiveSheet)
 		{
 			// effect
-			if (fByUser) GUISound("Command");
+			if (fByUser) GUISound("UI::Select");
 			// update in sheet
 			pActiveSheet->OnShown(fByUser);
 		}
@@ -290,7 +290,7 @@ namespace C4GUI
 				int32_t iTabWidth, iTabHeight;
 				pSheet->GetCaptionSize(&iTabWidth, &iTabHeight, HasLargeCaptions(), pSheet == pActiveSheet, pfctClip, pfctIcons, pSheetCaptionFont);
 				iTabWidth += (eTabPos == tbLeft) ? 20 : iSheetSpacing;
-				iMaxTabWidth = Max(iTabWidth, iMaxTabWidth);
+				iMaxTabWidth = std::max(iTabWidth, iMaxTabWidth);
 				if (eTabPos == tbLeft)
 				{
 					iTotalHgt += iTabHeight;
@@ -393,7 +393,6 @@ namespace C4GUI
 		// scrolling in captions
 		int32_t iScrollSize = GetTopSize();
 		if (fScrollingLeft) d -= iCaptionScrollPos + iScrollSize;
-		//DrawBar(cgo, ::GraphicsResource.fctCaptionLeft, ::GraphicsResource.fctCaptionBar, ::GraphicsResource.fctCaptionRight);
 		// tabs
 		for (Sheet *pSheet = (Sheet *) GetFirst(); pSheet; pSheet = (Sheet *) pSheet->GetNext())
 		{
@@ -422,8 +421,9 @@ namespace C4GUI
 					vtx[6] = d+iTabWidth; vtx[7] = y0;
 				}
 				DWORD dwClr = (pSheet == pActiveSheet) ? C4GUI_ActiveTabBGColor : C4GUI_StandardBGColor;
-				pDraw->DrawQuadDw(cgo.Surface, vtx, dwClr, dwClr, dwClr, dwClr);
+				pDraw->DrawQuadDw(cgo.Surface, vtx, dwClr, dwClr, dwClr, dwClr, NULL);
 				// draw caption frame
+				// TODO: Switch to PerformMultiLines
 				pDraw->DrawLineDw(cgo.Surface, (float)vtx[0]-1     , (float)vtx[1]      , (float)vtx[2]-1    ,(float)vtx[3]        , C4GUI_BorderColorA1);
 				pDraw->DrawLineDw(cgo.Surface, (float)vtx[2]-1     , (float)vtx[3]      , (float)vtx[4]-fLeft,(float)vtx[5]        , C4GUI_BorderColorA1);
 				pDraw->DrawLineDw(cgo.Surface, (float)vtx[4]       , (float)vtx[5]      , (float)vtx[6]      ,(float)vtx[7]        , C4GUI_BorderColorA1);
@@ -494,7 +494,7 @@ namespace C4GUI
 					if (iButton == C4MC_Button_LeftDown && fScrollingRight && Inside(iX, rcBounds.Wdt-iScrollSize,rcBounds.Wdt))
 					{
 						fProcessed = fScrollingRightDown = true;
-						GUISound("Command");
+						GUISound("UI::Select");
 						DoCaptionScroll(+1);
 					}
 					else if (fScrollingLeft)
@@ -502,7 +502,7 @@ namespace C4GUI
 						if (iButton == C4MC_Button_LeftDown && Inside(iX, d,d+iScrollSize))
 						{
 							fProcessed = fScrollingLeftDown = true;
-							GUISound("Command");
+							GUISound("UI::Select");
 							DoCaptionScroll(-1);
 						}
 						d -= iCaptionScrollPos + iScrollSize;
@@ -564,7 +564,7 @@ namespace C4GUI
 		if (fScrollingLeftDown || fScrollingRightDown)
 		{
 			// stop scrolling
-			GUISound("Command");
+			GUISound("UI::Select");
 			fScrollingLeftDown = fScrollingRightDown = false;
 		}
 	}
