@@ -19,19 +19,19 @@
 #include <C4Include.h>
 #include <C4MainMenu.h>
 
+#include <C4Application.h>
 #include <C4DefList.h>
 #include <C4FullScreen.h>
 #include <C4Viewport.h>
 #include <C4Player.h>
 #include <C4GameOverDlg.h>
-#include <C4Object.h>
 #include <C4GraphicsResource.h>
-#include <C4GraphicsSystem.h>
 #include <C4Game.h>
 #include <C4PlayerList.h>
 #include <C4GameObjects.h>
 #include <C4Network2.h>
 #include <C4GameControl.h>
+#include <C4Def.h>
 
 // -----------------------------------------------------------
 // C4MainMenu
@@ -804,8 +804,8 @@ bool C4MainMenu::MenuCommand(const char *szCommand, bool fIsCloseCommand)
 	{
 		if (!ValidPlr(Player)) return false; // observers may not look at goal/rule info, because it requires queue activation
 		Close(true);
-		C4Object *pObj; C4ID idItem(szCommand+12);
-		if ((pObj = ::Objects.Find(idItem)))
+		C4Object *pObj; C4ID idItem(szCommand+12); C4Def * pDef = C4Id2Def(idItem);
+		if (pDef && (pObj = ::Objects.Find(pDef)))
 			::Control.DoInput(CID_PlrAction, C4ControlPlayerAction::ActivateGoal(::Players.Get(Player), pObj), CDT_Queue);
 		else
 			return false;
