@@ -32,6 +32,7 @@
 #include <C4GraphicsResource.h>
 #include <C4PlayerList.h>
 #include <C4GameControl.h>
+#include <C4ScriptGuiWindow.h>
 
 const int32_t C4MC_Drag_None            = 0,
               C4MC_Drag_Script          = 6,
@@ -50,7 +51,7 @@ const int32_t C4MC_Cursor_Select      = 0,		// click cursor to select/click stuf
               C4MC_Cursor_DownRight   = 10,
               C4MC_Cursor_Passive     = 11;		// passive cursor in records and and fog of war and outside viewport
 
-const int32_t C4MC_Time_on_Target     = 10;
+const int32_t C4MC_Tooltip_Delay = 20;
 
 C4MouseControl::C4MouseControl()
 {
@@ -87,7 +88,6 @@ void C4MouseControl::Default()
 	Drag=C4MC_Drag_None;
 	Selection.Default();
 	TargetObject=DownTarget=NULL;
-	TimeOnTargetObject=0;
 	ControlDown=false;
 	ShiftDown=false;
 	AltDown=false;
@@ -232,12 +232,6 @@ void C4MouseControl::Move(int32_t iButton, int32_t iX, int32_t iY, DWORD dwKeyFl
 	{
 		iX = Viewport->ViewWdt/2;
 		iY = Viewport->ViewHgt/2;
-		if (!Application.isEditor)
-		{
-			//int32_t iMidX = Viewport->OutX + iX;
-			//int32_t iMidY = Viewport->OutY + iY;
-			//FIXMESetCursorPos(iMidX, iMidY);
-		}
 		InitCentered = true;
 	}
 #endif
@@ -312,20 +306,31 @@ void C4MouseControl::Move(int32_t iButton, int32_t iX, int32_t iY, DWORD dwKeyFl
 	case C4MC_Button_Wheel: Wheel(dwKeyFlags); break;
 	}
 
-	// script handling of mouse control for everything but regular movement (which is sent at control frame intervals only)
-	if (iButton != C4MC_Button_None)
-		// not if blocked by selection object
-		if (!TargetObject)
-			// safety (can't really happen in !IsPassive, but w/e
-			if (pPlayer && pPlayer->ControlSet)
-			{
-				if (pPlayer->ControlSet->IsMouseControlAssigned(iButton))
+	// are custom menus active?
+	bool menuProcessed = false;
+	if (pPlayer)
+		// adjust by viewport X/Y because the GUI windows calculate their positions (and thus check input) based on that
+		menuProcessed = ::Game.ScriptGuiRoot->MouseInput(iButton, iX, iY, dwKeyFlags);
+
+	if (menuProcessed)
+		Cursor = C4MC_Cursor_Select;
+
+	// if not caught by a menu
+	if (!menuProcessed)
+		// script handling of mouse control for everything but regular movement (which is sent at control frame intervals only)
+		if (iButton != C4MC_Button_None)
+			// not if blocked by selection object
+			if (!TargetObject)
+				// safety (can't really happen in !IsPassive, but w/e
+				if (pPlayer && pPlayer->ControlSet)
 				{
-					int wheel_dir = 0;
-					if (iButton == C4MC_Button_Wheel) wheel_dir = (short)(dwKeyFlags >> 16);
-					pPlayer->Control.DoMouseInput(0 /* only 1 mouse supported so far */, iButton, GameX, GameY, GuiX, GuiY, (dwKeyFlags & MK_CONTROL) != 0, (dwKeyFlags & MK_SHIFT) != 0, (dwKeyFlags & MK_ALT) != 0, wheel_dir);
+					if (!menuProcessed && pPlayer->ControlSet->IsMouseControlAssigned(iButton))
+					{
+						int wheel_dir = 0;
+						if (iButton == C4MC_Button_Wheel) wheel_dir = (short)(dwKeyFlags >> 16);
+						pPlayer->Control.DoMouseInput(0 /* only 1 mouse supported so far */, iButton, GameX, GameY, GuiX, GuiY, (dwKeyFlags & MK_CONTROL) != 0, (dwKeyFlags & MK_SHIFT) != 0, (dwKeyFlags & MK_ALT) != 0, wheel_dir);
+					}
 				}
-			}
 }
 
 void C4MouseControl::DoMoveInput()
@@ -434,7 +439,7 @@ void C4MouseControl::Draw(C4TargetFacet &cgo, const ZoomData &GameZoom)
 				uint32_t BlitMode = DragImageObject->BlitMode;
 				DragImageObject->ColorMod = (Drag == C4MC_Drag_Script) ? 0x7fffffff : (/*DragImagePhase*/0 ? 0x8f7f0000 : 0x1f007f00);
 				DragImageObject->BlitMode = C4GFXBLIT_MOD2;
-	
+
 				DragImageObject->DrawPicture(ccgo, false, NULL);
 
 				DragImageObject->ColorMod = ColorMod;
@@ -518,27 +523,43 @@ void C4MouseControl::UpdateCursorTarget()
 		if (IsPassive())
 			Cursor=C4MC_Cursor_Passive;
 
-		// Time on target: caption
-		if (TargetObject && Cursor == C4MC_Cursor_Select)
+		// update tooltip information
+		if (OldTargetObject != TargetObject)
 		{
-			TimeOnTargetObject++;
-			if (TimeOnTargetObject>=C4MC_Time_on_Target)
+			C4String *newTooltip = nullptr;
+			if (TargetObject && (Cursor == C4MC_Cursor_Select) && (TargetObject->Category & C4D_MouseSelect) && (newTooltip = TargetObject->GetPropertyStr(P_Tooltip)))
 			{
-				if (TargetObject->Category & C4D_MouseSelect)
-				{
-					// set caption
-					if (!KeepCaption)
-					{
-						C4String *tooltip = TargetObject->GetPropertyStr(P_Tooltip);
-						if(tooltip) {
-							Caption = TargetObject->GetPropertyStr(P_Tooltip)->GetData();
-						}
-					}
-				}
+				float objX, objY;
+				TargetObject->GetViewPos(objX, objY, -fctViewportGUI.X, -fctViewportGUI.Y, fctViewportGUI);
+				objX += TargetObject->Shape.x;
+				objY += TargetObject->Shape.y - TargetObject->addtop();
+				SetTooltipRectangle(C4Rect(objX, objY, TargetObject->Shape.Wdt, TargetObject->Shape.Hgt + TargetObject->addtop()));
+				SetTooltipText(StdStrBuf(newTooltip->GetCStr()));
+			}
+			else
+			{
+				SetTooltipRectangle(C4Rect(0, 0, 0, 0));
+			}
+		}
+
+		if (!KeepCaption
+			&& ToolTipRectangle.Wdt != 0
+			&& Inside(GuiX, ToolTipRectangle.x, ToolTipRectangle.x + ToolTipRectangle.Wdt)
+			&& Inside(GuiY, ToolTipRectangle.y, ToolTipRectangle.y + ToolTipRectangle.Hgt))
+		{
+			++TimeInTooltipRectangle;
+
+			if (TimeInTooltipRectangle >= C4MC_Tooltip_Delay)
+			{
+				Caption = TooltipText;
 			}
 		}
 		else
-			TimeOnTargetObject=0;
+		{
+			// disable tooltip pop-up; whatever set it in the first place will set it again on the next mouse-enter
+			TimeInTooltipRectangle = 0;
+			ToolTipRectangle.Wdt = 0;
+		}
 	}
 
 	// Make a script callback if the object being hovered changes
@@ -706,21 +727,6 @@ void C4MouseControl::RightUp()
 
 void C4MouseControl::Wheel(DWORD dwFlags)
 {
-	//short iDelta = (short)(dwFlags >> 16);
-
-	// Normal wheel: control zoom
-	if (!ControlDown)
-	{
-		//if(iDelta > 0) Viewport->ChangeZoom(C4GFX_ZoomStep);
-		//if(iDelta < 0) Viewport->ChangeZoom(1.0f/C4GFX_ZoomStep);
-	}
-	// Ctrl + Wheel: pass to player control (might be used for inventory or such)
-	else
-	{
-		// TODO
-		//if(iDelta > 0) Game.LocalPlayerControl(Player, COM_WheelUp);
-		//if(iDelta < 0) Game.LocalPlayerControl(Player, COM_WheelDown);
-	}
 }
 
 bool C4MouseControl::IsValidMenu(C4Menu *pMenu)
@@ -878,6 +884,20 @@ const char *C4MouseControl::GetCaption()
 	return Caption.getData();
 }
 
+void C4MouseControl::SetTooltipRectangle(const C4Rect &rectangle)
+{
+	// Set the tooltip rectangle slightly larger than originally requested.
+	// The tooltip will be removed when the cursor leaves the rectangle, so make sure that the tooltip is not already disabled when the cursor moves to the border-pixel of the GUI item
+	// in case the GUI item uses a different check for bounds (< vs <=) than the tooltip rectangle.
+	ToolTipRectangle = C4Rect(rectangle.x - 2, rectangle.y - 2, rectangle.Wdt + 4, rectangle.Hgt + 4);
+	TimeInTooltipRectangle = 0;
+}
+
+void C4MouseControl::SetTooltipText(const StdStrBuf &text)
+{
+	TooltipText = text;
+}
+
 C4Object *C4MouseControl::GetTargetObject()
 {
 	// find object
