@@ -26,10 +26,6 @@
 #include <CSurface8.h>
 #include <C4Material.h>
 
-const uint8_t IFT = 0x80;
-
-const uint8_t CSkyDef1=104,CSkyDef2=123;
-
 const int32_t C4MaxMaterial = 125;
 
 const int32_t C4LSC_Undefined = 0,
@@ -48,7 +44,6 @@ public:
 	int32_t Mode;
 	int32_t Width,Height;
 	int32_t MapWidth,MapHeight,MapZoom;
-	CSurface8 * Map;
 	DWORD MatCount[C4MaxMaterial]; // NoSave //
 	DWORD EffectiveMatCount[C4MaxMaterial]; // NoSave //
 
@@ -62,20 +57,30 @@ public:
 	C4MapCreatorS2 *pMapCreator; // map creator for script-generated maps
 	bool fMapChanged;
 	BYTE *pInitial; // Initial landscape after creation - used for diff
+	BYTE *pInitialBkg; // Initial bkg landscape after creation - used for diff
 	class C4FoW *pFoW;
 
 private:
 	CSurface8 * Surface8;
+	CSurface8 * Surface8Bkg; // Background material
+	CSurface8 * Map;
+	CSurface8 * MapBkg;
 	class C4LandscapeRender *pLandscapeRender;
 	uint8_t *TopRowPix, *BottomRowPix; // array size of landscape width: Filled with 0s for border pixels that are open and MCVehic for pixels that are closed
-	int32_t Pix2Mat[256], Pix2Dens[256], Pix2Place[256];
-	bool Pix2Light[256];
+	int32_t Pix2Mat[C4M_MaxTexIndex], Pix2Dens[C4M_MaxTexIndex], Pix2Place[C4M_MaxTexIndex];
+	bool Pix2Light[C4M_MaxTexIndex];
 	int32_t PixCntPitch;
 	uint8_t *PixCnt;
 	C4Rect Relights[C4LS_MaxRelights];
-	mutable uint8_t *BridgeMatConversion[128]; // NoSave //
+	mutable uint8_t *BridgeMatConversion[C4M_MaxTexIndex]; // NoSave //
 
 public:
+	// Use this with the various drawing functions to keep current material for
+	// either foreground or background map. We can use C4M_MaxTexIndex as a value
+	// here because this value is reserved anyway for the differential landscape
+	// encoding.
+	static const uint8_t Transparent = C4M_MaxTexIndex;
+
 	void Default();
 	void Clear(bool fClearMapCreator=true, bool fClearSky=true, bool fClearRenderer=true);
 	void Execute();
@@ -92,6 +97,7 @@ public:
 	void RaiseTerrain(int32_t tx, int32_t ty, int32_t wdt);
 	void FindMatTop(int32_t mat, int32_t &x, int32_t &y, bool distant_first) const;
 	BYTE GetMapIndex(int32_t iX, int32_t iY) const;
+	BYTE GetBackMapIndex(int32_t iX, int32_t iY) const;
 	bool Load(C4Group &hGroup, bool fLoadSky, bool fSavegame);
 	bool Save(C4Group &hGroup) const;
 	bool SaveDiff(C4Group &hGroup, bool fSyncSave) const;
@@ -99,24 +105,26 @@ public:
 	bool SaveInitial();
 	bool SaveTextures(C4Group &hGroup) const;
 	bool Init(C4Group &hGroup, bool fOverloadCurrent, bool fLoadSky, bool &rfLoaded, bool fSavegame);
+	bool HasMap() const { return Map != NULL && MapBkg != NULL; }
 	bool MapToLandscape();
 	bool ApplyDiff(C4Group &hGroup);
 	bool SetMode(int32_t iMode);
-	bool SetPix(int32_t x, int32_t y, BYTE npix); // set landscape pixel (bounds checked)
-	bool _SetPix(int32_t x, int32_t y, BYTE npix); // set landsape pixel (bounds not checked)
-	bool _SetPixIfMask(int32_t x, int32_t y, BYTE npix, BYTE nMask) ; // set landscape pixel, if it matches nMask color (no bound-checks)
+	bool SetPix2(int32_t x, int32_t y, BYTE fgPix, BYTE bgPix); // set landscape pixel (bounds checked)
+	bool _SetPix2(int32_t x, int32_t y, BYTE fgPix, BYTE bgPix); // set landsape pixel (bounds not checked)
+	void _SetPix2Tmp(int32_t x, int32_t y, BYTE fgPix, BYTE bgPix); // set landsape pixel (bounds not checked, no material count updates, no landscape relighting). Material must be reset to original value with this function before modifying landscape in any other way. Only used for temporary pixel changes by SolidMask (C4SolidMask::RemoveTemporary, C4SolidMask::PutTemporary).
+	bool InsertMaterialOutsideLandscape(int32_t tx, int32_t ty, int32_t mdens); // return whether material insertion would be successful on an out-of-landscape position. Does not actually insert material.
 	bool InsertMaterial(int32_t mat, int32_t *tx, int32_t *ty, int32_t vx = 0, int32_t vy = 0, bool query_only=false); // modifies tx/ty to actual insertion position
 	bool InsertDeadMaterial(int32_t mat, int32_t tx, int32_t ty);
 	bool FindMatPath(int32_t &fx, int32_t &fy, int32_t ydir, int32_t mdens, int32_t mslide) const;
 	bool FindMatSlide(int32_t &fx, int32_t &fy, int32_t ydir, int32_t mdens, int32_t mslide) const;
 	bool FindMatPathPush(int32_t &fx, int32_t &fy, int32_t mdens, int32_t mslide, bool liquid) const;
-	bool Incinerate(int32_t x, int32_t y);
-	bool DrawBrush(int32_t iX, int32_t iY, int32_t iGrade, const char *szMaterial, const char *szTexture, bool fIFT);
-	bool DrawLine(int32_t iX1, int32_t iY1, int32_t iX2, int32_t iY2, int32_t iGrade, const char *szMaterial, const char *szTexture, bool fIFT);
-	bool DrawBox(int32_t iX1, int32_t iY1, int32_t iX2, int32_t iY2, int32_t iGrade, const char *szMaterial, const char *szTexture, bool fIFT);
+	bool Incinerate(int32_t x, int32_t y, int32_t cause_player);
+	bool DrawBrush(int32_t iX, int32_t iY, int32_t iGrade, const char *szMaterial, const char *szTexture, const char *szBackMaterial, const char *szBackTexture);
+	bool DrawLine(int32_t iX1, int32_t iY1, int32_t iX2, int32_t iY2, int32_t iGrade, const char *szMaterial, const char *szTexture, const char *szBackMaterial, const char *szBackTexture);
+	bool DrawBox(int32_t iX1, int32_t iY1, int32_t iX2, int32_t iY2, int32_t iGrade, const char *szMaterial, const char *szTexture, const char *szBackMaterial, const char *szBackTexture);
 	bool DrawChunks(int32_t tx, int32_t ty, int32_t wdt, int32_t hgt, int32_t icntx, int32_t icnty, const char *szMaterial, const char *szTexture, bool bIFT);
-	bool DrawQuad(int32_t iX1, int32_t iY1, int32_t iX2, int32_t iY2, int32_t iX3, int32_t iY3, int32_t iX4, int32_t iY4, const char *szMaterial, bool bIFT, bool fDrawBridge);
-	bool DrawPolygon(int *vtcs, int length, const char *szMaterial, bool bIFT, bool fDrawBridge);
+	bool DrawQuad(int32_t iX1, int32_t iY1, int32_t iX2, int32_t iY2, int32_t iX3, int32_t iY3, int32_t iX4, int32_t iY4, const char *szMaterial, const char *szBackMaterial, bool fDrawBridge);
+	bool DrawPolygon(int *vtcs, int length, const char *szMaterial, const char *szBackMaterial, bool fDrawBridge);
 	CStdPalette *GetPal() const { return Surface8 ? Surface8->pPal : NULL; }
 	inline BYTE _GetPix(int32_t x, int32_t y) const // get landscape pixel (bounds not checked)
 	{
@@ -173,6 +181,72 @@ public:
 	{
 		return Pix2Place[GetPix(x, y)];
 	}
+
+	inline BYTE _GetBackPix(int32_t x, int32_t y) const // get landscape pixel (bounds not checked)
+	{
+#ifdef _DEBUG
+		if (x<0 || y<0 || x>=Width || y>=Height) { BREAKPOINT_HERE; }
+#endif
+		return Surface8Bkg->_GetPix(x,y);
+	}
+	inline BYTE GetBackPix(int32_t x, int32_t y) const // get landscape pixel (bounds checked)
+	{
+		// Border checks
+		if (x<0)
+		{
+			if (y<LeftOpen) return 0;
+			else return Mat2PixColDefault(MTunnel);
+		}
+		if (static_cast<uint32_t>(x) >= static_cast<uint32_t>(Width))
+		{
+			if (y<RightOpen) return 0;
+			else return Mat2PixColDefault(MTunnel);
+		}
+		if (y<0)
+		{
+			return DefaultBkgMat(TopRowPix[x]);
+		}
+		if (static_cast<uint32_t>(y) >= static_cast<uint32_t>(Height))
+		{
+			return DefaultBkgMat(BottomRowPix[x]);
+		}
+
+		return Surface8Bkg->_GetPix(x,y);
+	}
+	inline int32_t _GetBackMat(int32_t x, int32_t y) const // get landscape material (bounds not checked)
+	{
+		return Pix2Mat[_GetBackPix(x, y)];
+	}
+	inline int32_t _GetBackDensity(int32_t x, int32_t y) const // get landscape density (bounds not checked)
+	{
+		return Pix2Dens[_GetBackPix(x, y)];
+	}
+	inline int32_t _GetBackPlacement(int32_t x, int32_t y) const // get landscape material placement (bounds not checked)
+	{
+		return Pix2Place[_GetBackPix(x, y)];
+	}
+	inline int32_t GetBackMat(int32_t x, int32_t y) const // get landscape material (bounds checked)
+	{
+		return Pix2Mat[GetBackPix(x, y)];
+	}
+	inline int32_t GetBackDensity(int32_t x, int32_t y) const // get landscape density (bounds checked)
+	{
+		return Pix2Dens[GetBackPix(x, y)];
+	}
+	inline int32_t GetBackPlacement(int32_t x, int32_t y) const // get landscape material placement (bounds checked)
+	{
+		return Pix2Place[GetBackPix(x, y)];
+	}
+
+	inline bool GetLight(int32_t x, int32_t y)
+	{
+		return GetBackPix(x, y) == 0 || Pix2Light[GetPix(x, y)];
+	}
+	inline bool _GetLight(int32_t x, int32_t y)
+	{
+		return _GetBackPix(x, y) == 0 || Pix2Light[_GetPix(x, y)];
+	}
+
 	inline bool _FastSolidCheck(int32_t x, int32_t y) const // checks whether there *might* be something solid at the point
 	{
 		return PixCnt[(x / 17) * PixCntPitch + (y / 15)] > 0;
@@ -183,7 +257,6 @@ public:
 	}
 	inline int32_t GetPixMat(BYTE byPix) const { return Pix2Mat[byPix]; }
 	inline int32_t GetPixDensity(BYTE byPix) const { return Pix2Dens[byPix]; }
-	inline bool GetPixLight(BYTE byPix) const { return Pix2Light[byPix];  }
 	bool _PathFree(int32_t x, int32_t y, int32_t x2, int32_t y2) const; // quickly checks wether there *might* be pixel in the path.
 	int32_t GetMatHeight(int32_t x, int32_t y, int32_t iYDir, int32_t iMat, int32_t iMax) const;
 
@@ -208,33 +281,31 @@ private:
 	void ExecuteScan();
 	int32_t DoScan(int32_t x, int32_t y, int32_t mat, int32_t dir);
 	uint32_t ChunkyRandom(uint32_t &iOffset, uint32_t iRange) const; // return static random value, according to offset and MapSeed
-	void DrawChunk(int32_t tx, int32_t ty, int32_t wdt, int32_t hgt, int32_t mcol, C4MaterialCoreShape Shape, uint32_t cro);
-	void DrawSmoothOChunk(int32_t tx, int32_t ty, int32_t wdt, int32_t hgt, int32_t mcol, int flip, uint32_t cro);
-	void DrawCustomShapePoly(const C4MaterialShape::Poly &poly, int32_t off_x, int32_t off_y, int32_t mcol);
-	void DrawCustomShape(CSurface8 * sfcMap, C4MaterialShape *shape, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, int32_t iTexture, int32_t mcol, int32_t iOffX, int32_t iOffY);
-	void ChunkOZoom(CSurface8 * sfcMap, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, int32_t iTexture,int32_t iOffX=0,int32_t iOffY=0);
-	bool GetTexUsage(CSurface8 * sfcMap, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, DWORD *dwpTextureUsage) const;
-	bool TexOZoom(CSurface8 * sfcMap, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, DWORD *dwpTextureUsage, int32_t iToX=0,int32_t iToY=0);
-	bool MapToSurface(CSurface8 * sfcMap, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, int32_t iToX, int32_t iToY, int32_t iToWdt, int32_t iToHgt, int32_t iOffX, int32_t iOffY);
-	bool MapToLandscape(CSurface8 * sfcMap, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, int32_t iOffsX = 0, int32_t iOffsY = 0, bool noClear = false); // zoom map segment to surface (or sector surfaces)
+	void DrawChunk(int32_t tx, int32_t ty, int32_t wdt, int32_t hgt, uint8_t mcol, uint8_t mcolBkg, C4MaterialCoreShape Shape, uint32_t cro);
+	void DrawSmoothOChunk(int32_t tx, int32_t ty, int32_t wdt, int32_t hgt, uint8_t mcol, uint8_t mcolBkg, int flip, uint32_t cro);
+	void ChunkOZoom(CSurface8 * sfcMap, CSurface8* sfcMapBkg, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, uint8_t iTexture, int32_t iOffX=0,int32_t iOffY=0);
+	bool GetTexUsage(CSurface8 * sfcMap, CSurface8* sfcMapBkg, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, DWORD *dwpTextureUsage) const;
+	bool TexOZoom(CSurface8 * sfcMap, CSurface8* sfcMapBkg, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, DWORD *dwpTextureUsage, int32_t iToX=0,int32_t iToY=0);
+	bool MapToSurface(CSurface8 * sfcMap, CSurface8* sfcMapBkg, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, int32_t iToX, int32_t iToY, int32_t iToWdt, int32_t iToHgt, int32_t iOffX, int32_t iOffY);
+	bool MapToLandscape(CSurface8 * sfcMap, CSurface8* sfcMapBkg, int32_t iMapX, int32_t iMapY, int32_t iMapWdt, int32_t iMapHgt, int32_t iOffsX = 0, int32_t iOffsY = 0, bool noClear = false); // zoom map segment to surface (or sector surfaces)
 	bool InitTopAndBottomRowPix(); // inti out-of-landscape pixels for bottom side
-	bool GetMapColorIndex(const char *szMaterial, const char *szTexture, bool fIFT, BYTE &rbyCol) const;
+	bool GetMapColorIndex(const char *szMaterial, const char *szTexture, BYTE &rbyCol) const;
 	bool SkyToLandscape(int32_t iToX, int32_t iToY, int32_t iToWdt, int32_t iToHgt, int32_t iOffX, int32_t iOffY);
-	CSurface8 * CreateMap(); // create map by landscape attributes
-	CSurface8 * CreateMapS2(C4Group &ScenFile); // create map by def file
+	bool CreateMap(CSurface8*& sfcMap, CSurface8*& sfcMapBkg); // create map by landscape attributes
+	bool CreateMapS2(C4Group &ScenFile, CSurface8*& sfcMap, CSurface8*& sfcMapBkg); // create map by def file
 	bool Mat2Pal(); // assign material colors to landscape palette
 	void UpdatePixCnt(const class C4Rect &Rect, bool fCheck = false);
 	void UpdateMatCnt(C4Rect Rect, bool fPlus);
 	void PrepareChange(C4Rect BoundingBox);
 	void FinishChange(C4Rect BoundingBox);
-	bool DrawLineLandscape(int32_t iX, int32_t iY, int32_t iGrade, uint8_t line_color);
-	bool DrawLineMap(int32_t iX, int32_t iY, int32_t iRadius, uint8_t line_color);
+	bool DrawLineLandscape(int32_t iX, int32_t iY, int32_t iGrade, uint8_t line_color, uint8_t line_color_bkg);
+	bool DrawLineMap(int32_t iX, int32_t iY, int32_t iRadius, uint8_t line_color, uint8_t line_color_bkg);
 	uint8_t *GetBridgeMatConversion(int32_t for_material_col) const;
 	bool SaveInternal(C4Group &hGroup) const;
 	bool SaveDiffInternal(C4Group &hGroup, bool fSyncSave) const;
 
 	int32_t ForPolygon(int *vtcs, int length, bool (C4Landscape::*fnCallback)(int32_t, int32_t),
-	                C4MaterialList *mats_count = NULL, int col = 0, uint8_t *conversion_table = NULL);
+	                C4MaterialList *mats_count = NULL, uint8_t col = 0, uint8_t colBkg = 0, uint8_t *conversion_table = NULL);
 
 public:
 	int32_t DigFreeShape(int *vtcs, int length, C4Object *by_object = NULL, bool no_dig2objects = false, bool no_instability_check = false);
@@ -252,10 +323,7 @@ public:
 	bool ClearPix(int32_t tx, int32_t ty);	// also used by mass mover (corrode)
 
 private:
-	std::vector<int32_t> GetRoundPolygon(int32_t x, int32_t y, int32_t size, int32_t smoothness) const;
-	std::vector<int32_t> GetRectangle(int32_t tx, int32_t ty, int32_t wdt, int32_t hgt) const;
-	C4Rect getBoundingBox(int *vtcs, int length) const;
-
+	CSurface8* CreateDefaultBkgSurface(CSurface8& sfcFg, bool msbAsIft) const;
 	void DigMaterial2Objects(int32_t tx, int32_t ty, C4MaterialList *mat_list, C4Object *pCollect = NULL);
 	void BlastMaterial2Objects(int32_t tx, int32_t ty, C4MaterialList *mat_list, int32_t caused_by, int32_t str, C4ValueArray *out_objects);
 
@@ -266,10 +334,10 @@ private:
 
 	C4ValueArray *PrepareFreeShape(C4Rect &BoundingBox, C4Object *by_object);
 	void PostFreeShape(C4ValueArray *dig_objects, C4Object *by_object);
+	BYTE DefaultBkgMat(BYTE fg) const;
 
 public:
 	void CompileFunc(StdCompiler *pComp); // without landscape bitmaps and sky
-	bool DebugSave(const char *szFilename) const;
 };
 
 extern C4Landscape Landscape;
@@ -296,11 +364,8 @@ int32_t PixCol2Mat(BYTE pixc);
 #define GBackWdt ::Landscape.Width
 #define GBackHgt ::Landscape.Height
 #define GBackPix ::Landscape.GetPix
-#define SBackPix ::Landscape.SetPix
 #define ClearBackPix ::Landscape.ClearPix
 #define _GBackPix ::Landscape._GetPix
-#define _SBackPix ::Landscape._SetPix
-#define _SBackPixIfMask ::Landscape._SetPixIfMask
 
 inline bool DensitySolid(int32_t dens)
 {
@@ -317,24 +382,12 @@ inline bool DensityLiquid(int32_t dens)
 	return ((dens>=C4M_Liquid) && (dens<C4M_Solid));
 }
 
-inline BYTE PixColIFT(BYTE pixc)
-{
-	return pixc & IFT;
-}
-
 inline int32_t PixCol2Tex(BYTE pixc)
 {
-	// Remove IFT
-	int32_t iTex = int32_t(pixc & (IFT - 1));
 	// Validate
-	if (iTex >= C4M_MaxTexIndex) return 0;
+	if (pixc >= C4M_MaxTexIndex) return 0;
 	// Done
-	return iTex;
-}
-
-inline BYTE GBackIFT(int32_t x, int32_t y)
-{
-	return PixColIFT(GBackPix(x,y));
+	return pixc;
 }
 
 inline int32_t GBackMat(int32_t x, int32_t y)
