@@ -55,6 +55,7 @@ C4LandscapeRenderGL::C4LandscapeRenderGL()
 {
 	ZeroMem(Surfaces, sizeof(Surfaces));
 	ZeroMem(hMaterialTexture, sizeof(hMaterialTexture));
+	hVBO = 0;
 }
 
 C4LandscapeRenderGL::~C4LandscapeRenderGL()
@@ -99,6 +100,12 @@ bool C4LandscapeRenderGL::Init(int32_t iWidth, int32_t iHeight, C4TextureMap *pT
 		return false;
 	}
 
+	if (!InitVBO())
+	{
+		LogFatal("[!] Could not initialize landscape VBO!");
+		return false;
+	}
+
 	return true;
 }
 
@@ -135,27 +142,31 @@ void C4LandscapeRenderGL::Clear()
 		delete Surfaces[i];
 		Surfaces[i] = NULL;
 	}
+
 	glDeleteTextures(C4LR_MipMapCount, hMaterialTexture);
 	std::fill_n(hMaterialTexture, C4LR_MipMapCount, 0);
+
+	glDeleteBuffers(1, &hVBO);
+	hVBO = 0;
 }
 
 bool C4LandscapeRenderGL::InitLandscapeTexture()
 {
-
-	// Round up to nearest power of two
-	int iSfcWdt = 1, iSfcHgt = 1;
-	while(iSfcWdt < iWidth) iSfcWdt *= 2;
-	while(iSfcHgt < iHeight) iSfcHgt *= 2;
-
-	// One bit more of information to safe more space
-	if(iSfcWdt * 3 / 4 >= iWidth) iSfcWdt = iSfcWdt * 3 / 4;
-	if(iSfcHgt * 3 / 4 >= iHeight) iSfcHgt = iSfcHgt * 3 / 4;
+	// Don't round up to the nearest power-of-two here. Modern
+	// hardware should handle NPOT-textures just fine, and they
+	// are already used in other places in Clonk. This avoids
+	// accessing one row or column of pixels below the actually
+	// used surface in the shader, which can lead to strange
+	// one-pixel edges at the bottom or right of the landscape
+	// (see e.g. http://bugs.openclonk.org/view.php?id=771).
+	int iSfcWdt = iWidth;
+	int iSfcHgt = iHeight;
 
 	// Create our surfaces
 	for(int i = 0; i < C4LR_SurfaceCount; i++)
 	{
 		Surfaces[i] = new C4Surface();
-		if(!Surfaces[i]->Create(iSfcWdt, iSfcHgt))
+		if(!Surfaces[i]->Create(iSfcWdt, iSfcHgt, false, 0, 0))
 			return false;
 	}
 
@@ -190,8 +201,8 @@ bool C4LandscapeRenderGL::InitMaterialTexture(C4TextureMap *pTexs)
 	glGetIntegerv(GL_MAX_3D_TEXTURE_SIZE, &iMaxTexSize);
 	if (iTexWdt > iMaxTexSize || iTexHgt > iMaxTexSize)
 	{
-		iTexWdt = Min(iTexWdt, iMaxTexSize);
-		iTexHgt = Min(iTexHgt, iMaxTexSize);
+		iTexWdt = std::min(iTexWdt, iMaxTexSize);
+		iTexHgt = std::min(iTexHgt, iMaxTexSize);
 		LogF("   gl: Material textures too large, GPU only supports %dx%d! Cropping might occur!", iMaxTexSize, iMaxTexSize);
 	}
 	if(iMaterialTextureDepth >= iMaxTexSize)
@@ -233,9 +244,13 @@ bool C4LandscapeRenderGL::InitMaterialTexture(C4TextureMap *pTexs)
 				Log("   gl: Halp! Material texture is fragmented!");
 			else
 			{
-				// Size recheck
+				// Size recheck. It's fine if this texture's size is a divisor
+				// of the maximum texture size, because then we can just tile
+				// the smaller texture.
 				if(pSurface->Wdt != iTexWdt || pSurface->Hgt != iTexHgt)
-					LogF("   gl: texture %s size mismatch (%dx%d vs %dx%d)!", Texture.getData(), pSurface->Wdt, pSurface->Hgt, iTexWdt, iTexHgt);
+					if (iTexWdt % pSurface->Wdt != 0 || iTexHgt % pSurface->Hgt != 0)
+						LogF("   gl: texture %s size mismatch (%dx%d vs %dx%d)!", Texture.getData(), pSurface->Wdt, pSurface->Hgt, iTexWdt, iTexHgt);
+
 				// Copy bytes
 				DWORD *texdata = reinterpret_cast<DWORD *>(p);
 				pSurface->Lock();
@@ -259,7 +274,15 @@ bool C4LandscapeRenderGL::InitMaterialTexture(C4TextureMap *pTexs)
 		}
 		// If we didn't "continue" yet, we haven't written the texture yet.
 		// Make color texture transparent, and normal texture flat.
-		memset(p, fNormal ? 127 : 0, iTexSize);
+		if (fNormal)
+		{
+			DWORD *texdata = reinterpret_cast<DWORD *>(p);
+			for (int y = 0; y < iTexHgt; ++y)
+				for (int x = 0; x < iTexWdt; ++x)
+					*texdata++ = RGBA(127, 127, 255, 255);
+		}
+		else
+			memset(p, 0, iTexSize);
 	}
 
 	// Clear error error(s?)
@@ -371,20 +394,22 @@ void C4LandscapeRenderGL::Update(C4Rect To, C4Landscape *pSource)
 		Surfaces[i]->ClearBoxDw(To.x, To.y, To.Wdt, To.Hgt);
 	}
 
-	// Initialize up & down placement arrays:
-	// Calculate the placement sums for the first line in the rectangle only. For the consecutive lines, it is updated 
-	// for each line in the below for-loop.
+	// Initialize up & down placement arrays. These arrays are always updated
+	// so that they contain the placement sums of C4LR_BiasDistanceY pixels
+	// above and below the current row.
 	int x, y;
-	int *placementSumsUp = new int [To.Wdt * 2];
-	int *placementSumsDown = placementSumsUp + To.Wdt;
-	for(x = 0; x < To.Wdt; x++)
+	int placementSumsWidth = C4LR_BiasDistanceX * 2 + To.Wdt;
+	int *placementSumsUp = new int [placementSumsWidth * 2];
+	int *placementSumsDown = placementSumsUp + placementSumsWidth;
+	for(x = 0; x < placementSumsWidth; x++)
 	{
 		placementSumsUp[x] = 0;
 		placementSumsDown[x] = 0;
-		for(y = 1; y <= Min(C4LR_BiasDistanceY, To.y); y++)
-			placementSumsUp[x] += pSource->_GetPlacement(To.x+x, To.y-y);
-		for(y = 1; y <= Min(C4LR_BiasDistanceY, iHeight - 1 - To.y); y++)
-			placementSumsDown[x] += pSource->_GetPlacement(To.x+x, To.y+y);
+		if (To.x + x - C4LR_BiasDistanceX < 0 || To.x + x - C4LR_BiasDistanceX >= iWidth) continue;
+		for(y = 1; y <= std::min(C4LR_BiasDistanceY, To.y); y++)
+			placementSumsUp[x] += pSource->_GetPlacement(To.x+x-C4LR_BiasDistanceX, To.y-y);
+		for(y = 1; y <= std::min(C4LR_BiasDistanceY, iHeight - 1 - To.y); y++)
+			placementSumsDown[x] += pSource->_GetPlacement(To.x+x-C4LR_BiasDistanceX, To.y+y);
 	}
 
 	// Get tex refs (shortcut, we will use them quite heavily)
@@ -396,30 +421,50 @@ void C4LandscapeRenderGL::Update(C4Rect To, C4Landscape *pSource)
 	// Go through it from top to bottom
 	for(y = 0; y < To.Hgt; y++)
 	{
-		// Initialize left & right placement for the left-most pixel. Will be updated in the below loop for every pixel
-		// in the line
-		int placementSumLeft = 0;
-		int placementSumRight = 0;
-		for(x = 1; x <= Min(C4LR_BiasDistanceX, To.x); x++)
-			placementSumLeft += pSource->_GetPlacement(To.x-x,To.y+y);
-		for(x = 1; x <= Min(C4LR_BiasDistanceX, iWidth - 1 - To.x ); x++)
-			placementSumRight += pSource->_GetPlacement(To.x+x,To.y+y);
+		// Initialize left & right placement sums. These are meant to contain
+		// the placement sum of a (C4LR_BiasDistanceX, 2*C4LR_BiasDistanceY+1)
+		// rectangle left/right of the current pixel. So we initialise it to
+		// be correct at x=0. Note that the placementSum arrays don't contain
+		// information about the current row, therefore we need a special case
+		// for those pixels.
+		int sumLeft = 0, sumRight = 0;
+		for(x = 1; x <= std::min(C4LR_BiasDistanceX, To.x); x++)
+			sumLeft += pSource->_GetPlacement(To.x-x,To.y+y);
+		for(x = 1; x <= std::min(C4LR_BiasDistanceX, iWidth - 1 - To.x ); x++)
+			sumRight += pSource->_GetPlacement(To.x+x,To.y+y);
+		for (int i = 1; i <= C4LR_BiasDistanceX; i++) {
+			sumLeft += placementSumsUp[C4LR_BiasDistanceX - i];
+			sumLeft += placementSumsDown[C4LR_BiasDistanceX - i];
+			sumRight += placementSumsUp[C4LR_BiasDistanceX + i];
+			sumRight += placementSumsDown[C4LR_BiasDistanceX + i];
+		}
+
+		// Initialise up & down sums. Same principle as above, but slightly
+		// easier as we do not miss pixels if we just use the placement sums.
+		int sumUp = 0, sumDown = 0;
+		for (int i = -C4LR_BiasDistanceX; i <= C4LR_BiasDistanceX; i++) {
+			sumUp += placementSumsUp[C4LR_BiasDistanceX + i];
+			sumDown += placementSumsDown[C4LR_BiasDistanceX + i];
+		}
 
 		for(x = 0; x < To.Wdt; x++)
 		{
 			int pixel = pSource->_GetPix(To.x+x, To.y+y);
 			int placement = pSource->_GetPlacement(To.x+x, To.y+y);
 
-			int horizontalBias = Max(0, placement * C4LR_BiasDistanceX - placementSumRight) -
-			                     Max(0, placement * C4LR_BiasDistanceX - placementSumLeft);
-			int verticalBias = Max(0, placement * C4LR_BiasDistanceY - placementSumsDown[x]) -
-			                   Max(0, placement * C4LR_BiasDistanceY - placementSumsUp[x]);
+			// Calculate bias. The scale here is the size of the rectangle (see above)
+			const int horizontalFactor = C4LR_BiasDistanceX * (2 * C4LR_BiasDistanceY + 1);
+			int horizontalBias = std::max(0, placement * horizontalFactor - sumRight) -
+			                     std::max(0, placement * horizontalFactor - sumLeft);
+			const int verticalFactor = C4LR_BiasDistanceY * (2 * C4LR_BiasDistanceX + 1);
+			int verticalBias = std::max(0, placement * verticalFactor - sumDown) -
+			                   std::max(0, placement * verticalFactor - sumUp);
 
 			// Maximum placement differences that make a difference in the result,  after which we are at the limits of
 			// what can be packed into a byte
 			const int maximumPlacementDifference = 40;
-			int horizontalBiasScaled = Clamp(horizontalBias * 127 / maximumPlacementDifference / C4LR_BiasDistanceX + 128, 0, 255);
-			int verticalBiasScaled = Clamp(verticalBias * 127 / maximumPlacementDifference / C4LR_BiasDistanceY + 128, 0, 255);
+			int horizontalBiasScaled = Clamp(horizontalBias * 127 / maximumPlacementDifference / horizontalFactor + 128, 0, 255);
+			int verticalBiasScaled = Clamp(verticalBias * 127 / maximumPlacementDifference / verticalFactor + 128, 0, 255);
 
 			// Collect data to save per pixel
 			unsigned char data[C4LR_SurfaceCount * 4];
@@ -435,23 +480,51 @@ void C4LandscapeRenderGL::Update(C4Rect To, C4Landscape *pSource)
 				texture[i]->SetPix4(To.x+x, To.y+y, 
 					RGBA(data[i*4+0], data[i*4+1], data[i*4+2], data[i*4+3]));
 
+			// Update sums (last column would be out-of-bounds, and not
+			// necessary as we will re-initialise it for the next row)
+			if (x < To.Wdt - 1) {
+				sumLeft -= placementSumsUp[x] + placementSumsDown[x];
+				sumLeft += placementSumsUp[x + C4LR_BiasDistanceX] + placementSumsDown[x + C4LR_BiasDistanceX];
+				sumRight -= placementSumsUp[x + C4LR_BiasDistanceX + 1] + placementSumsDown[x + C4LR_BiasDistanceX + 1];
+				sumUp -= placementSumsUp[x];
+				sumDown -= placementSumsDown[x];
+				sumRight += placementSumsUp[x + 2 * C4LR_BiasDistanceX + 1] + placementSumsDown[x + 2 * C4LR_BiasDistanceX + 1];
+				sumUp += placementSumsUp[x + 2 * C4LR_BiasDistanceX + 1];
+				sumDown += placementSumsDown[x + 2 * C4LR_BiasDistanceX + 1];
+			}
+
 			// Update left & right for next pixel in line
 			if(x + To.x + 1 < iWidth)
-				placementSumRight -= pSource->_GetPlacement(To.x+x + 1, To.y+y);
+				sumRight -= pSource->_GetPlacement(To.x+x + 1, To.y+y);
 			if(To.x+x + C4LR_BiasDistanceX + 1 < iWidth)
-				placementSumRight += pSource->_GetPlacement(To.x+x + C4LR_BiasDistanceX + 1, To.y+y);
-			placementSumLeft += placement;
+				sumRight += pSource->_GetPlacement(To.x+x + C4LR_BiasDistanceX + 1, To.y+y);
+			sumLeft += placement;
 			if(To.x+x - C4LR_BiasDistanceX >= 0)
-				placementSumLeft -= pSource->_GetPlacement(To.x+x - C4LR_BiasDistanceX, To.y+y);
+				sumLeft -= pSource->_GetPlacement(To.x+x - C4LR_BiasDistanceX, To.y+y);
 
 			// Update up & down arrays (for next line already)
-			if(To.y+y + 1 < iHeight)
-				placementSumsDown[x] -= pSource->_GetPlacement(To.x+x, To.y+y + 1);
-			if(To.y+y + C4LR_BiasDistanceY + 1 < iHeight)
-				placementSumsDown[x] += pSource->_GetPlacement(To.x+x, To.y+y + C4LR_BiasDistanceY + 1);
-			placementSumsUp[x] += placement;
-			if(To.y+y - C4LR_BiasDistanceY >= 0) {
-				placementSumsUp[x] -= pSource->_GetPlacement(To.x+x, To.y+y - C4LR_BiasDistanceY);
+			if (To.x + x >= C4LR_BiasDistanceX) {
+				if (To.y + y + 1 < iHeight)
+					placementSumsDown[x] -= pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y + 1);
+				if (To.y + y + C4LR_BiasDistanceY + 1 < iHeight)
+					placementSumsDown[x] += pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y + C4LR_BiasDistanceY + 1);
+				if (To.y + y - C4LR_BiasDistanceY >= 0)
+					placementSumsUp[x] -= pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y - C4LR_BiasDistanceY);
+				placementSumsUp[x] += pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y);
+			}
+		}
+
+		// Finish updating up & down arrays for the next line
+		if (To.x + x >= C4LR_BiasDistanceX)
+		{
+			for (; x < std::min(placementSumsWidth, iWidth - To.x + C4LR_BiasDistanceX); x++) {
+				if (To.y + y + 1 < iHeight)
+					placementSumsDown[x] -= pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y + 1);
+				if (To.y + y + C4LR_BiasDistanceY + 1 < iHeight)
+					placementSumsDown[x] += pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y + C4LR_BiasDistanceY + 1);
+				if (To.y + y - C4LR_BiasDistanceY >= 0)
+					placementSumsUp[x] -= pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y - C4LR_BiasDistanceY);
+				placementSumsUp[x] += pSource->_GetPlacement(To.x + x - C4LR_BiasDistanceX, To.y + y);
 			}
 		}
 	}
@@ -531,23 +604,27 @@ const char *C4LandscapeRenderGL::UniformNames[C4LRU_Count+1];
 
 bool C4LandscapeRenderGL::LoadShader(C4GroupSet *pGroups, C4Shader& shader, const char* name, int ssc)
 {
-	// Create vertex shader (hard-coded)
-	shader.AddVertexDefaults();
-	hLandscapeTexCoord = shader.AddTexCoord("landscapeCoord");
-	if(ssc & C4SSC_LIGHT) hLightTexCoord = shader.AddTexCoord("lightCoord");
+	// Setup #defines
+	shader.AddDefine("OPENCLONK");
+	shader.AddDefine("OC_LANDSCAPE");
+	if(ssc & C4SSC_LIGHT) shader.AddDefine("OC_DYNAMIC_LIGHT"); // sample light from light texture
+
+	// Create vertex shader
+	shader.LoadVertexSlices(pGroups, "LandscapeVertexShader.glsl");
 
 	// Then load slices for fragment shader
-	shader.AddFragmentSlice(-1, "#define LANDSCAPE");
-	if(ssc & C4SSC_LIGHT) shader.AddFragmentSlice(-1, "#define HAVE_LIGHT"); // sample light from light texture
+	shader.LoadFragmentSlices(pGroups, "CommonShader.glsl");
+	shader.LoadFragmentSlices(pGroups, "LandscapeShader.glsl");
 
-	shader.LoadSlices(pGroups, "UtilShader.glsl");
-	shader.LoadSlices(pGroups, "LandscapeShader.glsl");
-	shader.LoadSlices(pGroups, "LightShader.glsl");
-	shader.LoadSlices(pGroups, "AmbientShader.glsl");
-	shader.LoadSlices(pGroups, "ScalerShader.glsl");
+	// Make attribute name map
+	const char* AttributeNames[C4LRA_Count + 1];
+	AttributeNames[C4LRA_Position] = "oc_Position";
+	AttributeNames[C4LRA_LandscapeTexCoord] = "oc_LandscapeTexCoord";
+	AttributeNames[C4LRA_LightTexCoord] = "oc_LightTexCoord"; // unused if no dynamic light
+	AttributeNames[C4LRA_Count] = NULL;
 
 	// Initialise!
-	if (!shader.Init(name, UniformNames)) {
+	if (!shader.Init(name, UniformNames, AttributeNames)) {
 		shader.ClearSlices();
 		return false;
 	}
@@ -569,14 +646,15 @@ bool C4LandscapeRenderGL::LoadShaders(C4GroupSet *pGroups)
 
 	// Make uniform name map
 	ZeroMem(UniformNames, sizeof(UniformNames));
+	UniformNames[C4LRU_ProjectionMatrix]  = "projectionMatrix";
 	UniformNames[C4LRU_LandscapeTex]      = "landscapeTex";
 	UniformNames[C4LRU_ScalerTex]         = "scalerTex";
 	UniformNames[C4LRU_MaterialTex]       = "materialTex";
 	UniformNames[C4LRU_LightTex]          = "lightTex";
 	UniformNames[C4LRU_AmbientTex]        = "ambientTex";
+	UniformNames[C4LRU_Gamma]             = "gamma";
 	UniformNames[C4LRU_Resolution]        = "resolution";
 	UniformNames[C4LRU_Center]            = "center";
-	UniformNames[C4LRU_MatMap]            = "matMap";
 	UniformNames[C4LRU_MatMapTex]         = "matMapTex";
 	UniformNames[C4LRU_MaterialDepth]     = "materialDepth";
 	UniformNames[C4LRU_MaterialSize]      = "materialSize";
@@ -591,6 +669,17 @@ bool C4LandscapeRenderGL::LoadShaders(C4GroupSet *pGroups)
 	return true;
 }
 
+bool C4LandscapeRenderGL::InitVBO()
+{
+	// Our VBO needs to hold 4 vertices with 6 floats each.
+	assert(hVBO == 0);
+	glGenBuffers(1, &hVBO);
+	glBindBuffer(GL_ARRAY_BUFFER, hVBO);
+	glBufferData(GL_ARRAY_BUFFER, 24 * sizeof(float), NULL, GL_STREAM_DRAW);
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	return true;
+}
+
 void C4LandscapeRenderGL::ClearShaders()
 {
 	if (Shader.Initialised())
@@ -606,19 +695,13 @@ void C4LandscapeRenderGL::ClearShaders()
 	}
 }
 
-void C4LandscapeRenderGL::RefreshShaders()
-{
-	Shader.Refresh("landscape", UniformNames);
-	ShaderLight.Refresh("landscapeLight", UniformNames);
-}
-
 bool C4LandscapeRenderGL::LoadScaler(C4GroupSet *pGroups)
 {
 	// Search for scaler
 	C4Group *pGroup = pGroups->FindEntry(C4CFN_LandscapeScaler);
 	if(!pGroup) return false;
 	// Load scaler from group
-	if(!fctScaler.Load(*pGroup, C4CFN_LandscapeScaler))
+	if(!fctScaler.Load(*pGroup, C4CFN_LandscapeScaler, C4FCT_Full, C4FCT_Full, false, 0))
 		return false;
 	// Check size
 	const int iOrigWdt = 8 * 3, iOrigHgt = 4 * 8 * 3;
@@ -798,7 +881,7 @@ void C4LandscapeRenderGL::AddTexturesFromMap(C4TextureMap *pMap)
 
 }
 
-void C4LandscapeRenderGL::BuildMatMap(GLfloat *pFMap, GLubyte *pIMap)
+void C4LandscapeRenderGL::BuildMatMap(uint32_t *pTex)
 {
 	// TODO: Still merely an inefficient placeholder for things to come...
 
@@ -810,8 +893,8 @@ void C4LandscapeRenderGL::BuildMatMap(GLfloat *pFMap, GLubyte *pIMap)
 		if(!pEntry->GetTextureName())
 		{
 			// Undefined textures transparent
-			if(pFMap) pFMap[pix] = 0.5 / iMaterialTextureDepth;
-			if(pIMap) pIMap[pix] = 0;
+			pTex[2*pix] = 0;
+			pTex[2*pix+1] = RGBA(0,0,0,255);
 			continue;
 		}
 
@@ -819,8 +902,13 @@ void C4LandscapeRenderGL::BuildMatMap(GLfloat *pFMap, GLubyte *pIMap)
 		int iPhases = 1; const char *p = pEntry->GetTextureName();
 		while((p = strchr(p, '-'))) { p++; iPhases++; }
 		// Hard-coded hack. Fix me!
-		const int iPhaseLength = 300;
-		float phase = (iPhases == 1 ? 0 : float(C4TimeMilliseconds::Now().AsInt() % (iPhases * iPhaseLength)) / iPhaseLength);
+		C4Material *pMaterial = pEntry->GetMaterial();
+		const int iPhaseLength = pMaterial->AnimationSpeed;
+		float phase = 0;
+		if (iPhases > 1) {
+			phase = C4TimeMilliseconds::Now().AsInt() % (iPhases * iPhaseLength);
+			phase /= iPhaseLength;
+		}
 
 		// Find our transition
 		const char *pFrom = pEntry->GetTextureName();
@@ -849,8 +937,17 @@ void C4LandscapeRenderGL::BuildMatMap(GLfloat *pFMap, GLubyte *pIMap)
 		}
 
 		// Assign texture
-		if(pFMap) pFMap[pix] = (gTexCoo + 0.5) / iMaterialTextureDepth;
-		if(pIMap) pIMap[pix] = int((gTexCoo * 256.0 / iMaterialTextureDepth) + 0.5);
+		int iTexCoo = int((gTexCoo * 256.0 / iMaterialTextureDepth) + 0.5);
+		pTex[2*pix] = RGBA(
+			Clamp(pMaterial->LightEmit[0], 0, 255),
+			Clamp(pMaterial->LightEmit[1], 0, 255),
+			Clamp(pMaterial->LightEmit[2], 0, 255),
+			iTexCoo);
+		pTex[2*pix+1] = RGBA(
+			Clamp(pMaterial->LightSpot[0], 0, 255),
+			Clamp(pMaterial->LightSpot[1], 0, 255),
+			Clamp(pMaterial->LightSpot[2], 0, 255),
+			Clamp(pMaterial->LightAngle, 0, 255));
 	}
 }
 
@@ -863,13 +960,6 @@ void C4LandscapeRenderGL::Draw(const C4TargetFacet &cgo, const C4FoWRegion *Ligh
 	C4Surface *sfcTarget = cgo.Surface;
 	if (!pGL->PrepareRendering(sfcTarget)) return;
 
-#ifdef AUTO_RELOAD_SHADERS
-	RefreshShaders();
-#endif // AUTO_RELOAD_SHADERS
-
-	// Clear error(s?)
-	while(glGetError()) {}
-
 	// Choose the right shader depending on whether we have dynamic lighting or not
 	const C4Shader* shader = &Shader;
 	if (Light) shader = &ShaderLight;
@@ -880,19 +970,15 @@ void C4LandscapeRenderGL::Draw(const C4TargetFacet &cgo, const C4FoWRegion *Ligh
 	ShaderCall.Start();
 
 	// Bind data
+	ShaderCall.SetUniformMatrix4x4(C4LRU_ProjectionMatrix, pGL->GetProjectionMatrix());
+	ShaderCall.SetUniform3fv(C4LRU_Gamma, 1, pGL->gammaOut);
 	ShaderCall.SetUniform2f(C4LRU_Resolution, Surfaces[0]->Wdt, Surfaces[0]->Hgt);
 	float centerX = float(cgo.TargetX)+float(cgo.Wdt)/2,
 		  centerY = float(cgo.TargetY)+float(cgo.Hgt)/2;
 	ShaderCall.SetUniform2f(C4LRU_Center,
 	                        centerX / float(Surfaces[0]->Wdt),
 	                        centerY / float(Surfaces[0]->Hgt));
-	if (shader->HaveUniform(C4LRU_MatMap))
-	{
-		GLfloat MatMap[256];
-		BuildMatMap(MatMap, NULL);
-		ShaderCall.SetUniform1fv(C4LRU_MatMap, 256, MatMap);
-	}
-	ShaderCall.SetUniform1i(C4LRU_MaterialDepth, iMaterialTextureDepth);
+	ShaderCall.SetUniform1f(C4LRU_MaterialDepth, float(iMaterialTextureDepth));
 	ShaderCall.SetUniform2f(C4LRU_MaterialSize,
 	                        float(iMaterialWidth) / ::Game.C4S.Landscape.MaterialZoom,
 	                        float(iMaterialHeight) / ::Game.C4S.Landscape.MaterialZoom);
@@ -914,7 +1000,7 @@ void C4LandscapeRenderGL::Draw(const C4TargetFacet &cgo, const C4FoWRegion *Ligh
 		GLint iLandscapeUnits[C4LR_SurfaceCount];
 		for(int i = 0; i < C4LR_SurfaceCount; i++)
 		{
-			iLandscapeUnits[i] = ShaderCall.AllocTexUnit(-1, GL_TEXTURE_2D) - GL_TEXTURE0;
+			iLandscapeUnits[i] = ShaderCall.AllocTexUnit(-1) - GL_TEXTURE0;
 			glBindTexture(GL_TEXTURE_2D, Surfaces[i]->textures[0].texName);
 			if (pGL->Zoom != 1.0)
 			{
@@ -929,23 +1015,23 @@ void C4LandscapeRenderGL::Draw(const C4TargetFacet &cgo, const C4FoWRegion *Ligh
 		}
 		ShaderCall.SetUniform1iv(C4LRU_LandscapeTex, C4LR_SurfaceCount, iLandscapeUnits);
 	}
-	if(Light && ShaderCall.AllocTexUnit(C4LRU_LightTex, GL_TEXTURE_2D))
+	if(Light && ShaderCall.AllocTexUnit(C4LRU_LightTex))
 	{
-		glBindTexture(GL_TEXTURE_2D, Light->getSurface()->textures[0].texName);
+		glBindTexture(GL_TEXTURE_2D, Light->getSurfaceName());
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	}
-	if(Light && ShaderCall.AllocTexUnit(C4LRU_AmbientTex, GL_TEXTURE_2D))
+	if(Light && ShaderCall.AllocTexUnit(C4LRU_AmbientTex))
 	{
 		glBindTexture(GL_TEXTURE_2D, Light->getFoW()->Ambient.Tex);
 	}
-	if(ShaderCall.AllocTexUnit(C4LRU_ScalerTex, GL_TEXTURE_2D))
+	if(ShaderCall.AllocTexUnit(C4LRU_ScalerTex))
 	{
 		glBindTexture(GL_TEXTURE_2D, fctScaler.Surface->textures[0].texName);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	}
-	if(ShaderCall.AllocTexUnit(C4LRU_MaterialTex, GL_TEXTURE_3D))
+	if(ShaderCall.AllocTexUnit(C4LRU_MaterialTex))
 	{
         // Decide which mip-map level to use
 		double z = 0.5; int iMM = 0;
@@ -955,11 +1041,11 @@ void C4LandscapeRenderGL::Draw(const C4TargetFacet &cgo, const C4FoWRegion *Ligh
 		glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	}
-	if(ShaderCall.AllocTexUnit(C4LRU_MatMapTex, GL_TEXTURE_1D))
+	if(ShaderCall.AllocTexUnit(C4LRU_MatMapTex))
 	{
-		GLubyte MatMap[256];
-		BuildMatMap(NULL, MatMap);
-		glTexImage1D(GL_TEXTURE_1D, 0, 1, 256, 0, GL_RED, GL_UNSIGNED_BYTE, MatMap);
+		uint32_t MatMap[2*256];
+		BuildMatMap(MatMap);
+		glTexImage1D(GL_TEXTURE_1D, 0, GL_RGBA8, 2*256, 0, GL_RGBA, GL_UNSIGNED_BYTE, MatMap);
 		glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	}
 
@@ -983,58 +1069,81 @@ void C4LandscapeRenderGL::Draw(const C4TargetFacet &cgo, const C4FoWRegion *Ligh
 	// Blend it
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-	// To the blit
-	glColor3f(1.0, 1.0, 1.0);
-	glBegin(GL_QUADS);
+	// Prepare vertex data
+	float vtxData[24];
+	float* pos = &vtxData[0];
+	float* tex = &vtxData[8];
+	float* lightTex = &vtxData[16];
+
+	pos[0] = tTexBlt.left;
+	pos[1] = tTexBlt.top;
+	pos[2] = tTexBlt.right;
+	pos[3] = tTexBlt.top;
+	pos[4] = tTexBlt.left;
+	pos[5] = tTexBlt.bottom;
+	pos[6] = tTexBlt.right;
+	pos[7] = tTexBlt.bottom;
+
+	tex[0] = fTexBlt.left;
+	tex[1] = fTexBlt.top;
+	tex[2] = fTexBlt.right;
+	tex[3] = fTexBlt.top;
+	tex[4] = fTexBlt.left;
+	tex[5] = fTexBlt.bottom;
+	tex[6] = fTexBlt.right;
+	tex[7] = fTexBlt.bottom;
+
+	unsigned int nFloats = 16;
+	if (Light)
+	{
+		FLOAT_RECT lTexBlt;
+		const C4Rect LightRect = Light->getRegion();
+		int32_t iLightWdt = Light->getSurfaceWidth(),
+			iLightHgt = Light->getSurfaceHeight();
+		lTexBlt.left = (fx - LightRect.x) / iLightWdt;
+		lTexBlt.top = 1.0 - (fy - LightRect.y) / iLightHgt;
+		lTexBlt.right = (fx + cgo.Wdt - LightRect.x) / iLightWdt;
+		lTexBlt.bottom = 1.0 - (fy + cgo.Hgt - LightRect.y) / iLightHgt;
+
+		lightTex[0] = lTexBlt.left;
+		lightTex[1] = lTexBlt.top;
+		lightTex[2] = lTexBlt.right;
+		lightTex[3] = lTexBlt.top;
+		lightTex[4] = lTexBlt.left;
+		lightTex[5] = lTexBlt.bottom;
+		lightTex[6] = lTexBlt.right;
+		lightTex[7] = lTexBlt.bottom;
+		nFloats = 24;
+	}
+
+	// Upload vertex data
+	glBindBuffer(GL_ARRAY_BUFFER, hVBO);
+	glBufferSubData(GL_ARRAY_BUFFER, 0, nFloats * sizeof(float), vtxData);
+
+	// Setup state
+	glEnableVertexAttribArray(shader->GetAttribute(C4LRA_Position));
+	glEnableVertexAttribArray(shader->GetAttribute(C4LRA_LandscapeTexCoord));
+
+	glVertexAttribPointer(shader->GetAttribute(C4LRA_Position), 2, GL_FLOAT, GL_FALSE, 0, 0);
+	glVertexAttribPointer(shader->GetAttribute(C4LRA_LandscapeTexCoord), 2, GL_FLOAT, GL_FALSE, 0, reinterpret_cast<const uint8_t*>(8 * sizeof(float)));
 
 	if (Light)
 	{
-		// Calculate coordinates into light texture
-		FLOAT_RECT lTexBlt;
-		if (Light)
-		{
-			const C4Rect LightRect = Light->getRegion();
-			int32_t iLightWdt = Light->getSurface()->Wdt,
-				iLightHgt = Light->getSurface()->Hgt;
-			lTexBlt.left = (fx - LightRect.x) / iLightWdt;
-			lTexBlt.top = 1.0 - (fy - LightRect.y) / iLightHgt;
-			lTexBlt.right = (fx + cgo.Wdt - LightRect.x) / iLightWdt;
-			lTexBlt.bottom = 1.0 - (fy + cgo.Hgt - LightRect.y) / iLightHgt;
-		}
-
-		#define LVERTEX(x, y) \
-			glMultiTexCoord2f(hLandscapeTexCoord, fTexBlt.x, fTexBlt.y); \
-			glMultiTexCoord2f(hLightTexCoord, lTexBlt.x, lTexBlt.y); \
-			glVertex2f(tTexBlt.x, tTexBlt.y);
-
-		LVERTEX(left, top);
-		LVERTEX(right, top);
-		LVERTEX(right, bottom);
-		LVERTEX(left, bottom);
-
-		#undef LVERTEX
-	}
-	else
-	{
-		#define VERTEX(x, y) \
-			glMultiTexCoord2f(hLandscapeTexCoord, fTexBlt.x, fTexBlt.y); \
-			glVertex2f(tTexBlt.x, tTexBlt.y);
-
-		VERTEX(left, top);
-		VERTEX(right, top);
-		VERTEX(right, bottom);
-		VERTEX(left, bottom);
-
-		#undef VERTEX
+		glEnableVertexAttribArray(shader->GetAttribute(C4LRA_LightTexCoord));
+		glVertexAttribPointer(shader->GetAttribute(C4LRA_LightTexCoord), 2, GL_FLOAT, GL_FALSE, 0, reinterpret_cast<const uint8_t*>(16 * sizeof(float)));
 	}
 
-	
+	// Do the blit
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 
-	glEnd();
+	// Reset state
+	glDisableVertexAttribArray(shader->GetAttribute(C4LRA_Position));
+	glDisableVertexAttribArray(shader->GetAttribute(C4LRA_LandscapeTexCoord));
+	if (Light)
+		glDisableVertexAttribArray(shader->GetAttribute(C4LRA_LightTexCoord));
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 
-	// Remove shader
 	ShaderCall.Finish();
-
 }
 
 #endif // #ifndef USE_CONSOLE
