@@ -32,18 +32,55 @@ static const char *DrawFn_Background_Name  = "Background";
 static const char *DrawFn_Liquid_Name      = "Liquid";
 static const char *DrawFn_Solid_Name       = "Solid";
 
-int32_t FnParTexCol(C4String *mattex, int32_t default_col = -1)
+bool TexColSingle(const char *mattex, uint8_t& col)
+{
+	if (SEqual(mattex, DrawFn_Transparent_Name)) { col = 0; return true; }
+	if (SEqual(mattex, DrawFn_Sky_Name)) { col = C4M_MaxTexIndex; return true; }
+
+	col = ::MapScript.pTexMap->GetIndexMatTex(mattex);
+	if (col == 0) return false;
+
+	return true;
+}
+
+bool FnParTexCol(C4String *mattex, uint8_t& fg, uint8_t& bg)
 {
 	// Return index of material-texture definition for a single color
-	// Defaults to underground (tunnel background) color. Prefix material with ^ to get overground (sky background) color.
-	if (!mattex || !mattex->GetCStr()) return default_col;
-	if (mattex->GetData() == DrawFn_Transparent_Name) return 0;
-	if (mattex->GetData() == DrawFn_Sky_Name) return IFT;
-	const char *cmattex = mattex->GetCStr();
-	int32_t ift = IFT;
-	if (*cmattex == '^') { ift=0; ++cmattex; }
-	int32_t col = ::MapScript.pTexMap->GetIndexMatTex(cmattex);
-	return col ? col|ift : default_col;
+	// Defaults to underground (tunnel background) color. Prefix material with ^ to get overground (sky background) color,
+	// or specify as mattex1:mattex2 for foreground-background pair.
+	if (!mattex || !mattex->GetCStr()) return false;
+
+	int sep_pos = SCharPos(':', mattex->GetCStr());
+	if (sep_pos == -1)
+	{
+		const char *cmattex = mattex->GetCStr();
+		bool ift = true;
+		if (*cmattex == '^') { ift=false; ++cmattex; }
+		
+		uint8_t col;
+		if (!TexColSingle(cmattex, col)) return false;
+
+		fg = col;
+		if (ift)
+			bg = ::MapScript.pTexMap->DefaultBkgMatTex(fg);
+		else
+			bg = C4M_MaxTexIndex; // sky
+
+		return true;
+	}
+	else
+	{
+		const char *cmattex = mattex->GetCStr();
+		std::string fg_mattex(cmattex, cmattex + sep_pos);
+		std::string bg_mattex(cmattex + sep_pos + 1);
+
+		uint8_t fg_col, bg_col;
+		if (!TexColSingle(fg_mattex.c_str(), fg_col)) return false;
+		if (!TexColSingle(bg_mattex.c_str(), bg_col)) return false;
+
+		fg = fg_col; bg = bg_col;
+		return true;
+	}
 }
 
 void C4MapScriptMatTexMask::UnmaskSpec(C4String *spec)
@@ -52,8 +89,8 @@ void C4MapScriptMatTexMask::UnmaskSpec(C4String *spec)
 	// Possible definitions:
 	// Material-Texture - Given material-texture combination (both sky and tunnel background)
 	// Material         - All defined default textures of given material
-	// *                - All materials
-	// Sky              - Index IFT
+	// *                - All materials including sky
+	// Sky              - Index C4M_MaxTexIndex
 	// Transparent      - Index 0
 	// Background       - All tunnel materials plus sky
 	// Liquid           - All liquid materials
@@ -77,40 +114,40 @@ void C4MapScriptMatTexMask::UnmaskSpec(C4String *spec)
 		if (prefix_done) break;
 		++cspec;
 	}
-	std::vector<bool> mat_mask(IFT, false);
+	std::vector<bool> mat_mask(C4M_MaxTexIndex+1, false);
 	if (SEqual(cspec, DrawFn_Transparent_Name))
 	{
 		// "Transparent" is zero index. Force to non-IFT
 		mat_mask[0] = true;
-		bgsky = true;  bgtunnel = false;
 	}
 	else if (SEqual(cspec, DrawFn_Sky_Name))
 	{
-		// Sky material: Force to IFT
-		mat_mask[0] = true;
-		bgsky = false;  bgtunnel = true;
+		// Sky material
+		mat_mask[C4M_MaxTexIndex] = true;
 	}
 	else if (SEqual(cspec, DrawFn_Background_Name))
 	{
 		// All background materials
-		for (int32_t i=0; i<IFT; ++i) if (!DensitySemiSolid(Landscape.GetPixDensity(i))) mat_mask[i] = true;
+		for (int32_t i=1; i<C4M_MaxTexIndex; ++i) if (!DensitySemiSolid(Landscape.GetPixDensity(i))) mat_mask[i] = true;
 		// Background includes sky
-		mat_mask[0] = true;
+		mat_mask[C4M_MaxTexIndex] = true;
 	}
 	else if (SEqual(cspec, DrawFn_Liquid_Name))
 	{
 		// All liquid materials
-		for (int32_t i=0; i<IFT; ++i) if (DensityLiquid(Landscape.GetPixDensity(i))) mat_mask[i] = true;
+		for (int32_t i=1; i<C4M_MaxTexIndex; ++i) if (DensityLiquid(Landscape.GetPixDensity(i))) mat_mask[i] = true;
 	}
 	else if (SEqual(cspec, DrawFn_Solid_Name))
 	{
 		// All solid materials
-		for (int32_t i=0; i<IFT; ++i) if (DensitySolid(Landscape.GetPixDensity(i))) mat_mask[i] = true;
+		for (int32_t i=1; i<C4M_MaxTexIndex; ++i) if (DensitySolid(Landscape.GetPixDensity(i))) mat_mask[i] = true;
 	}
 	else if (SEqual(cspec, "*"))
 	{
 		// All materials
-		for (int32_t i=0; i<IFT; ++i) mat_mask[i] = true;
+		for (int32_t i=1; i<C4M_MaxTexIndex; ++i) mat_mask[i] = true;
+		// Including sky
+		mat_mask[C4M_MaxTexIndex] = true;
 	}
 	else
 	{
@@ -135,13 +172,15 @@ void C4MapScriptMatTexMask::UnmaskSpec(C4String *spec)
 			}
 		}
 	}
-	// 'OR' spec onto this->mask. Apply bgsky, bgtunnel and invert.
-	for (int32_t i=0; i<IFT; ++i)
+
+	// 'OR' spec onto this->sky_mask and this->tunnel_mask. Apply bgsky, bgtunnel and invert.
+	for (int32_t i=0; i<C4M_MaxTexIndex + 1; ++i)
+	{
 		if ((mat_mask[i] && (bgsky || !bgtunnel)) != invert)
-			mask[i] = true;
-	for (int32_t i=0; i<IFT; ++i)
+			sky_mask[i] = true;
 		if ((mat_mask[i] && (!bgsky || bgtunnel)) != invert)
-			mask[i+IFT] = true;
+			tunnel_mask[i] = true;
+	}
 }
 
 void C4MapScriptMatTexMask::Init(const C4Value &spec)
@@ -154,7 +193,7 @@ void C4MapScriptMatTexMask::Init(const C4Value &spec)
 		for (int32_t i=0; i<arr->GetSize(); ++i)
 		{
 			C4String *smask = arr->GetItem(i).getStr();
-			if (!smask) throw new C4AulExecError(FormatString("MatTexMask expected string as %dth element in array.", (int)i).getData());
+			if (!smask) throw C4AulExecError(FormatString("MatTexMask expected string as %dth element in array.", (int)i).getData());
 			UnmaskSpec(smask);
 		}
 	}
@@ -166,10 +205,12 @@ void C4MapScriptMatTexMask::Init(const C4Value &spec)
 			UnmaskSpec(smask);
 		else
 		{
-			if (spec) throw new C4AulExecError("MatTexMask expected string or array of strings.");
+			if (spec) throw C4AulExecError("MatTexMask expected string or array of strings.");
 			// nil defaults to everything except index zero unmasked
-			mask = std::vector<bool>(256, true);
-			mask[0] = false;
+			sky_mask = std::vector<bool>(256, true);
+			tunnel_mask = std::vector<bool>(256, true);
+			sky_mask[0] = false;
+			tunnel_mask[0] = false;
 		}
 	}
 }
@@ -191,12 +232,12 @@ static bool FnLayerDraw(C4PropList * _this, C4String *mattex, C4PropList *mask_a
 {
 	// Layer script function: Draw material mattex in shape of mask_algo in _this layer within bounds given by rect
 	C4MapScriptLayer *layer = _this->GetMapScriptLayer();
-	int32_t icol = FnParTexCol(mattex);
-	if (!layer || icol<0) return false;
+	uint8_t fg, bg;
+	if (!layer || !FnParTexCol(mattex, fg, bg)) return false;
 	C4Rect rcBounds;
 	if (!FnParRect(layer, rect, &rcBounds)) return false;
 	std::unique_ptr<C4MapScriptAlgo> algo(FnParAlgo(mask_algo));
-	return layer->Fill(icol, rcBounds, algo.get());
+	return layer->Fill(fg, bg, rcBounds, algo.get());
 }
 
 static bool FnLayerBlit(C4PropList * _this, C4PropList *mask_algo, C4ValueArray *rect)
@@ -215,8 +256,11 @@ static C4PropList *FnCreateLayer(C4PropList * _this, C4String *mattex_fill, int3
 {
 	// Layer script function: Create new layer filled by mattex_fill of size width,height as sub layer of _this map
 	// Size defaults to _this layer size
-	int32_t icol = FnParTexCol(mattex_fill, 0);
-	if (icol<0) throw new C4AulExecError(FormatString("CreateLayer: Invalid fill material.").getData());
+	uint8_t fg = 0, bg = 0;
+	if (mattex_fill && mattex_fill->GetCStr())
+		if (!FnParTexCol(mattex_fill, fg, bg))
+			throw C4AulExecError(FormatString("CreateLayer: Invalid fill material.").getData());
+
 	C4MapScriptLayer *layer = _this->GetMapScriptLayer();
 	if (!layer) return NULL;
 	if (!width && !height)
@@ -224,11 +268,11 @@ static C4PropList *FnCreateLayer(C4PropList * _this, C4String *mattex_fill, int3
 		width = layer->GetWdt();
 		height = layer->GetHgt();
 	}
-	if (width<=0 || height<=0) throw new C4AulExecError(FormatString("CreateLayer: Invalid size (%d*%d).", (int)width, (int)height).getData());
+	if (width<=0 || height<=0) throw C4AulExecError(FormatString("CreateLayer: Invalid size (%d*%d).", (int)width, (int)height).getData());
 	C4MapScriptMap *map = layer->GetMap();
 	if (!map) return NULL;
 	layer = map->CreateLayer(width, height);
-	if (icol) layer->Fill(icol, layer->GetBounds(), NULL);
+	if (fg != 0 || bg != 0) layer->Fill(fg, bg, layer->GetBounds(), NULL);
 	return layer;
 }
 
@@ -248,6 +292,37 @@ static C4PropList *FnLayerDuplicate(C4PropList * _this, const C4Value &mask_spec
 	return new_layer;
 }
 
+static int32_t FnLayerGetMaterialTextureIndex(C4PropList * _this, C4String* mattex)
+{
+	if (!mattex) return -1;
+
+	uint8_t col;
+	if (!TexColSingle(mattex->GetCStr(), col))
+		return -1;
+
+	return col;
+}
+
+static int32_t FnLayerGetDefaultBackgroundIndex(C4PropList * _this, const C4Value &value)
+{
+	uint8_t fg;
+	C4String* str;
+
+	if ((str = value.getStr()))
+	{
+		if (!TexColSingle(str->GetCStr(), fg))
+			return -1;
+	}
+	else
+	{
+		if (!Inside(value.getInt(), 0, 255))
+			return -1;
+		fg = value.getInt();
+	}
+
+	return ::MapScript.pTexMap->DefaultBkgMatTex(fg);
+}
+
 static int32_t FnLayerGetPixel(C4PropList * _this, int32_t x, int32_t y)
 {
 	// Layer script function: Query pixel at position x,y from _this layer
@@ -256,22 +331,64 @@ static int32_t FnLayerGetPixel(C4PropList * _this, int32_t x, int32_t y)
 	return layer->GetPix(x,y,0);
 }
 
-static bool FnLayerSetPixel(C4PropList * _this, int32_t x, int32_t y, const C4Value &to_value_c4v)
+static int32_t FnLayerGetBackPixel(C4PropList * _this, int32_t x, int32_t y)
+{
+	// Layer script function: Query pixel at position x,y from _this layer
+	C4MapScriptLayer *layer = _this->GetMapScriptLayer();
+	if (!layer) return 0;
+	return layer->GetBackPix(x,y,0);
+}
+
+static bool FnLayerSetPixel(C4PropList * _this, int32_t x, int32_t y, const C4Value& fg_value_c4v, const C4Value& bg_value_c4v)
 {
 	// Layer script function: Set pixel at position x,y to to_value in _this layer
 	C4MapScriptLayer *layer = _this->GetMapScriptLayer();
 	if (!layer) return false;
-	int32_t to_value; C4String *to_value_s;
-	if ((to_value_s = to_value_c4v.getStr()))
+	uint8_t fg, bg;
+
+	if (fg_value_c4v.GetType() == C4V_Nil)
 	{
-		to_value = FnParTexCol(to_value_s);
+		fg = layer->GetPix(x,y,0);
 	}
 	else
 	{
-		to_value = to_value_c4v.getInt();
+		const C4Value& val = fg_value_c4v;
+		C4String *str = val.getStr();
+		if (str != NULL)
+		{
+			if (!TexColSingle(str->GetCStr(), fg))
+				throw C4AulExecError("MapLayer::SetPixel: Trying to set invalid pixel value.");
+		}
+		else
+		{
+			if (!Inside(val.getInt(), 0, 255))
+				throw C4AulExecError("MapLayer::SetPixel: Trying to set invalid pixel value.");
+			fg = val.getInt();
+		}
 	}
-	if (!Inside(to_value, 0, 255)) throw new C4AulExecError("MapLayer::SetPixel: Trying to set invalid pixel value.");
-	return layer->SetPix(x,y,to_value);
+
+	if (bg_value_c4v.GetType() == C4V_Nil)
+	{
+		bg = layer->GetBackPix(x,y,0);
+	}
+	else
+	{
+		const C4Value& val = bg_value_c4v;
+		C4String *str = val.getStr();
+		if (str != NULL)
+		{
+			if (!TexColSingle(str->GetCStr(), bg))
+				throw C4AulExecError("MapLayer::SetPixel: Trying to set invalid pixel value.");
+		}
+		else
+		{
+			if (!Inside(val.getInt(), 0, 255))
+				throw C4AulExecError("MapLayer::SetPixel: Trying to set invalid pixel value.");
+			bg = val.getInt();
+		}
+	}
+			
+	return layer->SetPix(x,y,fg,bg);
 }
 
 static int32_t FnLayerGetPixelCount(C4PropList * _this, const C4Value &mask_spec, C4ValueArray *rect)
@@ -321,15 +438,17 @@ static C4ValueArray *FnLayerCreateMatTexMask(C4PropList * _this, const C4Value &
 {
 	// layer script function: Generate an array 256 bools representing the given mask_spec
 	C4MapScriptMatTexMask mat_mask(mask_spec);
-	C4ValueArray *result = new C4ValueArray(256);
-	for (int32_t i=0; i<256; ++i)
+	C4ValueArray *result = new C4ValueArray(C4M_MaxTexIndex + 1);
+	for (int32_t i=0; i < C4M_MaxTexIndex + 1; ++i)
 	{
-		result->SetItem(i, C4VBool(mat_mask(uint8_t(i))));
+		result->SetItem(i, C4VBool(mat_mask(uint8_t(i), C4M_MaxTexIndex)));
 	}
 	return result;
 }
 
-C4MapScriptLayer::C4MapScriptLayer(C4PropList *prototype, C4MapScriptMap *map) : C4PropListNumbered(prototype), surface(NULL), surface_owned(false), map(map)
+// TODO: CreateBackMatTexMask? Or Return 512 bools?
+
+C4MapScriptLayer::C4MapScriptLayer(C4PropList *prototype, C4MapScriptMap *map) : C4PropListNumbered(prototype), fg_surface(NULL), bg_surface(NULL), surface_owned(false), map(map)
 {
 	// It seems like numbered PropLists need a number. I don't know why.
 	AcquireNumber();
@@ -340,9 +459,10 @@ bool C4MapScriptLayer::CreateSurface(int32_t wdt, int32_t hgt)
 	// Create new surface of given size. Surface is filled with color 0
 	ClearSurface();
 	if (wdt<=0 || hgt<=0) return false;
-	surface = new CSurface8;
+	fg_surface = new CSurface8;
+	bg_surface = new CSurface8;
 	surface_owned = true;
-	if (!surface->Create(wdt, hgt))
+	if (!fg_surface->Create(wdt, hgt) || !bg_surface->Create(wdt, hgt))
 	{
 		ClearSurface();
 		return false;
@@ -354,48 +474,61 @@ bool C4MapScriptLayer::CreateSurface(int32_t wdt, int32_t hgt)
 void C4MapScriptLayer::ClearSurface()
 {
 	// Delete surface if owned or just set to zero if unowned
-	if (surface_owned) delete surface;
-	surface=NULL; surface_owned=false;
+	if (surface_owned) { delete fg_surface; delete bg_surface; }
+	fg_surface=NULL; bg_surface=NULL;
+	surface_owned=false;
 	// if there is no surface, width and height parameters are undefined. no need to update them.
 }
 
 void C4MapScriptLayer::UpdateSurfaceSize()
 {
 	// Called when surface size changes: Update internal property values
-	if (surface)
+	if (fg_surface)
 	{
-		SetProperty(P_Wdt, C4VInt(surface->Wdt));
-		SetProperty(P_Hgt, C4VInt(surface->Hgt));
+		SetProperty(P_Wdt, C4VInt(fg_surface->Wdt));
+		SetProperty(P_Hgt, C4VInt(fg_surface->Hgt));
 	}
 }
 
 void C4MapScriptLayer::ConvertSkyToTransparent()
 {
-	// Convert all sky (color==IFT) pixels to transparent (color==0)
+	// Convert all sky (color==C4M_MaxTexIndex) pixels to transparent (color==0)
 	// Needed because C4Landscape map zoom assumes sky to be 0
 	if (!HasSurface()) return;
-	for (int32_t y=0; y<surface->Hgt; ++y)
-		for (int32_t x=0; x<surface->Wdt; ++x)
-			if (surface->_GetPix(x,y) == IFT)
-				surface->_SetPix(x,y, 0);
+	for (int32_t y=0; y<fg_surface->Hgt; ++y)
+	{
+		for (int32_t x=0; x<fg_surface->Wdt; ++x)
+		{
+			if (fg_surface->_GetPix(x,y) == C4M_MaxTexIndex)
+				fg_surface->_SetPix(x,y, 0);
+
+			if (bg_surface->_GetPix(x,y) == C4M_MaxTexIndex)
+				bg_surface->_SetPix(x,y, 0);
+		}
+	}
 }
 
 C4Rect C4MapScriptLayer::GetBounds() const
 {
 	// Return bounding rectangle of surface. Surface always starts at 0,0.
-	return surface ? C4Rect(0,0,surface->Wdt,surface->Hgt) : C4Rect();
+	return fg_surface ? C4Rect(0,0,fg_surface->Wdt,fg_surface->Hgt) : C4Rect();
 }
 
-bool C4MapScriptLayer::Fill(int col, const C4Rect &rcBounds, const C4MapScriptAlgo *algo)
+bool C4MapScriptLayer::Fill(uint8_t fg, uint8_t bg, const C4Rect &rcBounds, const C4MapScriptAlgo *algo)
 {
 	// safety
+	uint8_t temp_fg, temp_bg;
 	if (!HasSurface()) return false;
-	assert(rcBounds.x>=0 && rcBounds.y>=0 && rcBounds.x+rcBounds.Wdt<=surface->Wdt && rcBounds.y+rcBounds.Hgt<=surface->Hgt);
+	assert(rcBounds.x>=0 && rcBounds.y>=0 && rcBounds.x+rcBounds.Wdt<=fg_surface->Wdt && rcBounds.y+rcBounds.Hgt<=fg_surface->Hgt);
 	// set all non-masked pixels within bounds that fulfill algo
 	for (int32_t y=rcBounds.y; y<rcBounds.y+rcBounds.Hgt; ++y)
 		for (int32_t x=rcBounds.x; x<rcBounds.x+rcBounds.Wdt; ++x)
-			if (!algo || (*algo)(x,y))
-				surface->_SetPix(x,y,col);
+			if (!algo || (*algo)(x,y,temp_fg,temp_bg))
+			{
+				fg_surface->_SetPix(x,y,fg);
+				bg_surface->_SetPix(x,y,bg);
+			}
+
 	return true;
 }
 
@@ -403,14 +536,17 @@ bool C4MapScriptLayer::Blit(const C4Rect &rcBounds, const C4MapScriptAlgo *algo)
 {
 	// safety
 	if (!HasSurface()) return false;
-	assert(rcBounds.x>=0 && rcBounds.y>=0 && rcBounds.x+rcBounds.Wdt<=surface->Wdt && rcBounds.y+rcBounds.Hgt<=surface->Hgt);
+	assert(rcBounds.x>=0 && rcBounds.y>=0 && rcBounds.x+rcBounds.Wdt<=fg_surface->Wdt && rcBounds.y+rcBounds.Hgt<=fg_surface->Hgt);
 	assert(algo);
 	// set all pixels within bounds by algo, if algo is not transparent
-	uint8_t col;
+	uint8_t fg, bg;
 	for (int32_t y=rcBounds.y; y<rcBounds.y+rcBounds.Hgt; ++y)
 		for (int32_t x=rcBounds.x; x<rcBounds.x+rcBounds.Wdt; ++x)
-			if ((col=(*algo)(x,y)))
-				surface->_SetPix(x,y,col);
+			if (((*algo)(x,y,fg,bg)))
+			{
+				if (fg) fg_surface->_SetPix(x,y,fg);
+				if (bg) bg_surface->_SetPix(x,y,bg);
+			}
 	return true;
 }
 
@@ -423,11 +559,19 @@ bool C4MapScriptLayer::Blit(const C4MapScriptLayer *src, const C4Rect &src_rect,
 	// TODO: Fix C4Rect::Contains and check if the sector code still works
 	// assert(src->GetBounds().Contains(src_rect));
 	// copy all pixels that aren't masked
-	uint8_t col;
+	uint8_t fg, bg;
 	for (int32_t y=src_rect.y; y<src_rect.y+src_rect.Hgt; ++y)
 		for (int32_t x=src_rect.x; x<src_rect.x+src_rect.Wdt; ++x)
-			if (col_mask(col=src->surface->_GetPix(x,y)))
-				surface->_SetPix(x-src_rect.x+tx,y-src_rect.y+ty,col);
+		{
+			fg = src->fg_surface->_GetPix(x, y);
+			bg = src->bg_surface->_GetPix(x, y);
+
+			if (col_mask(fg, bg))
+			{
+				fg_surface->_SetPix(x-src_rect.x+tx,y-src_rect.y+ty,fg);
+				bg_surface->_SetPix(x-src_rect.x+tx,y-src_rect.y+ty,bg);
+			}
+		}
 	return true;
 }
 
@@ -435,12 +579,12 @@ int32_t C4MapScriptLayer::GetPixCount(const C4Rect &rcBounds, const C4MapScriptM
 {
 	// safety
 	if (!HasSurface()) return 0;
-	assert(rcBounds.x>=0 && rcBounds.y>=0 && rcBounds.x+rcBounds.Wdt<=surface->Wdt && rcBounds.y+rcBounds.Hgt<=surface->Hgt);
+	assert(rcBounds.x>=0 && rcBounds.y>=0 && rcBounds.x+rcBounds.Wdt<=fg_surface->Wdt && rcBounds.y+rcBounds.Hgt<=fg_surface->Hgt);
 	// count matching pixels in rect
 	int32_t count = 0;
 	for (int32_t y=rcBounds.y; y<rcBounds.y+rcBounds.Hgt; ++y)
 		for (int32_t x=rcBounds.x; x<rcBounds.x+rcBounds.Wdt; ++x)
-			count += col_mask(surface->_GetPix(x,y));
+			count += col_mask(fg_surface->_GetPix(x,y), bg_surface->_GetPix(x, y));
 	return count;
 }
 
@@ -453,22 +597,22 @@ bool C4MapScriptLayer::FindPos(const C4Rect &search_rect, const C4MapScriptMatTe
 	{
 		int32_t x=search_rect.x + Random(search_rect.Wdt);
 		int32_t y=search_rect.y + Random(search_rect.Hgt);
-		if (col_mask(surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
+		if (col_mask(fg_surface->_GetPix(x,y), bg_surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
 	}
 	// Nothing found yet: Start at a random position and search systemically
 	// (this guantuess to find a pixel if there is one, but favours border pixels)
 	int32_t sx=search_rect.x + Random(search_rect.Wdt);
 	int32_t sy=search_rect.y + Random(search_rect.Hgt);
 	for (int32_t x = sx; x < search_rect.x + search_rect.Wdt; ++x)
-		if (col_mask(surface->_GetPix(x,sy))) { *out_x=x; *out_y=sy; return true; }
+		if (col_mask(fg_surface->_GetPix(x,sy), bg_surface->_GetPix(x,sy))) { *out_x=x; *out_y=sy; return true; }
 	for (int32_t y = sy + 1; y<search_rect.y + search_rect.Hgt; ++y)
 		for (int32_t x = search_rect.x; x < search_rect.x + search_rect.Wdt; ++x)
-			if (col_mask(surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
+			if (col_mask(fg_surface->_GetPix(x,y), bg_surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
 	for (int32_t y = search_rect.y; y<sy; ++y)
 		for (int32_t x = search_rect.x; x < search_rect.x + search_rect.Wdt; ++x)
-			if (col_mask(surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
+			if (col_mask(fg_surface->_GetPix(x,y), bg_surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
 	for (int32_t x = search_rect.x; x<sx; ++x)
-		if (col_mask(surface->_GetPix(x,sy))) { *out_x=x; *out_y=sy; return true; }
+		if (col_mask(fg_surface->_GetPix(x,sy), bg_surface->_GetPix(x,sy))) { *out_x=x; *out_y=sy; return true; }
 	// Nothing found
 	return false;
 }
@@ -536,7 +680,10 @@ void C4MapScriptHost::AddEngineFunctions()
 	::AddFunc(this, "Blit", FnLayerBlit);
 	::AddFunc(this, "CreateLayer", FnCreateLayer);
 	::AddFunc(this, "Duplicate", FnLayerDuplicate);
+	::AddFunc(this, "GetMaterialTextureIndex", FnLayerGetMaterialTextureIndex);
+	::AddFunc(this, "GetDefaultBackgroundIndex", FnLayerGetDefaultBackgroundIndex);
 	::AddFunc(this, "GetPixel", FnLayerGetPixel);
+	::AddFunc(this, "GetBackPixel", FnLayerGetBackPixel);
 	::AddFunc(this, "SetPixel", FnLayerSetPixel);
 	::AddFunc(this, "GetPixelCount", FnLayerGetPixelCount);
 	::AddFunc(this, "Resize", FnLayerResize);
@@ -574,10 +721,12 @@ C4MapScriptMap *C4MapScriptHost::CreateMap()
 	return new C4MapScriptMap(MapPrototype);
 }
 
-bool C4MapScriptHost::InitializeMap(C4SLandscape *pLandscape, C4TextureMap *pTexMap, C4MaterialMap *pMatMap, uint32_t iPlayerCount, CSurface8 **pmap_surface)
+bool C4MapScriptHost::InitializeMap(C4SLandscape *pLandscape, C4TextureMap *pTexMap, C4MaterialMap *pMatMap, uint32_t iPlayerCount, CSurface8 **pmap_fg_surface, CSurface8** pmap_bg_surface)
 {
 	// Init scripted map by calling InitializeMap in the proper scripts. If *pmap_surface is given, it will pass the existing map to be modified by script.
-	assert(pmap_surface);
+	assert(pmap_fg_surface);
+	assert(pmap_bg_surface);
+
 	this->pTexMap = pTexMap;
 	this->pMatMap = pMatMap;
 	// Don't bother creating surfaces if the functions aren't defined
@@ -589,13 +738,14 @@ bool C4MapScriptHost::InitializeMap(C4SLandscape *pLandscape, C4TextureMap *pTex
 	// Create proplist as script context
 	std::unique_ptr<C4MapScriptMap> map(CreateMap());
 	// Drawing on existing map or create new?
-	if (*pmap_surface)
+	if (*pmap_fg_surface && *pmap_bg_surface)
 	{
 		// Existing map
-		map->SetSurface(*pmap_surface);
+		map->SetSurface(*pmap_fg_surface, *pmap_bg_surface);
 	}
 	else
 	{
+		assert(!*pmap_fg_surface && !*pmap_bg_surface);
 		// No existing map. Create new.
 		int32_t map_wdt,map_hgt;
 		pLandscape->GetMapSize(map_wdt, map_hgt, iPlayerCount);
@@ -608,7 +758,7 @@ bool C4MapScriptHost::InitializeMap(C4SLandscape *pLandscape, C4TextureMap *pTex
 	if (result)
 	{
 		map->ConvertSkyToTransparent();
-		*pmap_surface = map->ReleaseSurface();
+		map->ReleaseSurface(*pmap_fg_surface, *pmap_bg_surface);
 	}
 	return !!result;
 }
