@@ -30,15 +30,22 @@
 // mattex masks: Array of bools for each possible material-texture index
 class C4MapScriptMatTexMask
 {
-	std::vector<bool> mask; // vector of size 256: true means pixel color is let through; false means it is blocked
+	std::vector<bool> sky_mask; // vector of size C4M_MaxTexIndex + 1: true means pixel color is let through; false means it is blocked
+	std::vector<bool> tunnel_mask; // vector of size C4M_MaxTexIndex + 1: true means pixel color is let through; false means it is blocked
+
+	// sky_mask stores the mask for pixels with sky or transparent background, and tunnel_mask
+	// stores the mask for pixels with non-sky background. We don't allow
+	// different masks for different background materials since this would
+	// need a C4M_MaxTexIndex + 1 x C4M_MaxTexIndex + 1 matrix to account
+	// for each possible combination of foreground and background pixel.
 
 	void UnmaskSpec(C4String *spec);
 public:
-	C4MapScriptMatTexMask() : mask(256,false) { }
-	C4MapScriptMatTexMask(const C4Value &spec) : mask(256,false) { Init(spec); }
+	C4MapScriptMatTexMask() : sky_mask(C4M_MaxTexIndex + 1,false), tunnel_mask(C4M_MaxTexIndex + 1, false) { }
+	C4MapScriptMatTexMask(const C4Value &spec) : sky_mask(C4M_MaxTexIndex + 1, false), tunnel_mask(C4M_MaxTexIndex + 1, false) { Init(spec); }
 	void Init(const C4Value &spec);
 
-	bool operator()(uint8_t mattex) const { return mask[mattex]; }
+	bool operator()(uint8_t fg, uint8_t bg) const { if (bg == C4M_MaxTexIndex) return sky_mask[fg]; else return tunnel_mask[fg]; }
 };
 
 // algorithms may be either indicator functions (int,int)->bool that tell whether a map pixel should be
@@ -49,7 +56,7 @@ class C4MapScriptAlgo
 protected:
 	bool GetXYProps(const C4PropList *props, C4PropertyName k, int32_t *out_xy, bool zero_defaults);
 public:
-	virtual uint8_t operator () (int32_t x, int32_t y) const = 0;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const = 0;
 	virtual ~C4MapScriptAlgo() {}
 };
 
@@ -89,7 +96,7 @@ public:
 	C4MapScriptAlgoLayer(const C4MapScriptLayer *layer) : layer(layer) {}
 	C4MapScriptAlgoLayer(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_RndChecker: checkerboard on which areas are randomly set or unset
@@ -100,7 +107,7 @@ class C4MapScriptAlgoRndChecker : public C4MapScriptAlgo
 public:
 	C4MapScriptAlgoRndChecker(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Rect: 1 for pixels contained in rect, 0 otherwise
@@ -110,7 +117,7 @@ class C4MapScriptAlgoRect : public C4MapScriptAlgo
 public:
 	C4MapScriptAlgoRect(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Ellipsis: 1 for pixels within ellipsis, 0 otherwise
@@ -121,7 +128,7 @@ class C4MapScriptAlgoEllipsis : public C4MapScriptAlgo
 public:
 	C4MapScriptAlgoEllipsis(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Polygon: 1 for pixels within polygon or on border, 0 otherwise
@@ -135,7 +142,7 @@ class C4MapScriptAlgoPolygon : public C4MapScriptAlgo
 public:
 	C4MapScriptAlgoPolygon(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Lines: 1 for pixels on stripes in direction ("X","Y"). Stripe distance "Distance". Optional offset "OffX", "OffY"
@@ -146,7 +153,7 @@ class C4MapScriptAlgoLines : public C4MapScriptAlgo
 public:
 	C4MapScriptAlgoLines(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // base class for algo that takes one or more operands
@@ -166,7 +173,7 @@ class C4MapScriptAlgoAnd : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoAnd(const C4PropList *props) : C4MapScriptAlgoModifier(props) { }
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Or: First nonzero operand
@@ -175,7 +182,7 @@ class C4MapScriptAlgoOr : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoOr(const C4PropList *props) : C4MapScriptAlgoModifier(props) { }
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Not: 1 if operand is 0, 0 otherwise.
@@ -184,7 +191,7 @@ class C4MapScriptAlgoNot : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoNot(const C4PropList *props) : C4MapScriptAlgoModifier(props,1,1) { }
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Xor: If exactly one of the two operands is nonzero, return it. Otherwise, return zero.
@@ -193,7 +200,7 @@ class C4MapScriptAlgoXor : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoXor(const C4PropList *props) : C4MapScriptAlgoModifier(props,2,2) { }
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Offset: Base layer shifted by ox,oy
@@ -203,7 +210,7 @@ class C4MapScriptAlgoOffset : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoOffset(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Scale: Base layer scaled by sx,sy percent from fixed point cx,cy
@@ -213,7 +220,7 @@ class C4MapScriptAlgoScale : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoScale(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Rotate: Base layer rotated by angle r around point ox,oy
@@ -224,7 +231,7 @@ class C4MapScriptAlgoRotate : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoRotate(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Turbulence: move by a random offset iterations times
@@ -234,7 +241,7 @@ class C4MapScriptAlgoTurbulence : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoTurbulence(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Border: Border of operand layer
@@ -245,7 +252,7 @@ class C4MapScriptAlgoBorder : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoBorder(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // MAPALGO_Filter: Original color of operand if it's marked to go through filter. 0 otherwise.
@@ -255,14 +262,15 @@ class C4MapScriptAlgoFilter : public C4MapScriptAlgoModifier
 public:
 	C4MapScriptAlgoFilter(const C4PropList *props);
 
-	virtual uint8_t operator () (int32_t x, int32_t y) const;
+	virtual bool operator () (int32_t x, int32_t y, uint8_t& fg, uint8_t& bg) const;
 };
 
 // layer of a script-controlled map
-// IFT can be used to mark Sky
+// C4M_MaxTexIndex can be used to mark Sky
 class C4MapScriptLayer : public C4PropListNumbered
 {
-	CSurface8 *surface;
+	CSurface8 *fg_surface;
+	CSurface8 *bg_surface;
 	bool surface_owned;
 protected:
 	class C4MapScriptMap *map;
@@ -276,30 +284,31 @@ public:
 	// Surface management
 	bool CreateSurface(int32_t wdt, int32_t hgt);
 	void ClearSurface();
-	CSurface8 *ReleaseSurface() { surface_owned=false; return surface; } // return surface and mark it as not owned by self
-	void SetSurface(CSurface8 *s) { ClearSurface(); surface=s; UpdateSurfaceSize(); }
-	bool HasSurface() const { return surface && surface->Bits; }
+	void ReleaseSurface(CSurface8*& fg, CSurface8*& bg) { surface_owned=false; fg = fg_surface; bg = bg_surface; } // return surface and mark it as not owned by self
+	void SetSurface(CSurface8 *fg, CSurface8* bg) { ClearSurface(); fg_surface=fg; bg_surface=bg; UpdateSurfaceSize(); }
+	bool HasSurface() const { return fg_surface && fg_surface->Bits && bg_surface && bg_surface->Bits; }
 
 	// Size management
 	void UpdateSurfaceSize(); // updates width and height properties by current surface
 	C4Rect GetBounds () const;
-	int32_t GetWdt() const { return surface ? surface->Wdt : 0; }
-	int32_t GetHgt() const { return surface ? surface->Hgt : 0; }
+	int32_t GetWdt() const { return fg_surface ? fg_surface->Wdt : 0; }
+	int32_t GetHgt() const { return fg_surface ? fg_surface->Hgt : 0; }
 
 	// Pixel functions
-	uint8_t GetPix(int32_t x, int32_t y, uint8_t outside_col) const { return (!HasSurface()||x<0||y<0||x>=surface->Wdt||y>=surface->Hgt) ? outside_col : surface->_GetPix(x,y); }
-	bool SetPix(int32_t x, int32_t y, uint8_t col) const { if (!HasSurface()||x<0||y<0||x>=surface->Wdt||y>=surface->Hgt) return false; surface->_SetPix(x,y,col); return true; }
-	bool IsPixMasked(int32_t x, int32_t y) const { return GetPix(x,y,0)!=0; } // masking: If pixel is inside surface and not transparent
-	void ConvertSkyToTransparent(); // change all pixels that are IFT to 0
-	int32_t GetPixCount(const C4Rect &rcBounds, const C4MapScriptMatTexMask &col_mask); // return number of pixels that match mask
+	uint8_t GetPix(int32_t x, int32_t y, uint8_t outside_col) const { return (!HasSurface()||x<0||y<0||x>=fg_surface->Wdt||y>=fg_surface->Hgt) ? outside_col : fg_surface->_GetPix(x,y); }
+	uint8_t GetBackPix(int32_t x, int32_t y, uint8_t outside_col) const { return (!HasSurface()||x<0||y<0||x>=bg_surface->Wdt||y>=bg_surface->Hgt) ? outside_col : bg_surface->_GetPix(x,y); }
+	bool SetPix(int32_t x, int32_t y, uint8_t fg, uint8_t bg) const { if (!HasSurface()||x<0||y<0||x>=bg_surface->Wdt||y>=bg_surface->Hgt) return false; fg_surface->_SetPix(x,y,fg); bg_surface->_SetPix(x,y,bg); return true; }
+	bool IsPixMasked(int32_t x, int32_t y) const { return GetPix(x,y,0) != 0 || GetBackPix(x,y,0) != 0; } // masking: If pixel is inside surface and not transparent
+	void ConvertSkyToTransparent(); // change all pixels that are C4M_MaxTexIndex to 0
+	int32_t GetPixCount(const C4Rect &rcBounds, const C4MapScriptMatTexMask &mask); // return number of pixels that match mask
 
 	// Drawing functions
-	bool Fill(int col, const C4Rect &rcBounds, const C4MapScriptAlgo *algo);
+	bool Fill(uint8_t fg, uint8_t bg, const C4Rect &rcBounds, const C4MapScriptAlgo *algo);
 	bool Blit(const C4Rect &rcBounds, const C4MapScriptAlgo *algo);
-	bool Blit(const C4MapScriptLayer *src, const C4Rect &src_rect, const C4MapScriptMatTexMask &col_mask, int32_t tx, int32_t ty);
+	bool Blit(const C4MapScriptLayer *src, const C4Rect &src_rect, const C4MapScriptMatTexMask &mask, int32_t tx, int32_t ty);
 
 	// Search functions
-	bool FindPos(const C4Rect &search_rect, const C4MapScriptMatTexMask &col_mask, int32_t *out_x, int32_t *out_y, int32_t max_tries);
+	bool FindPos(const C4Rect &search_rect, const C4MapScriptMatTexMask &mask, int32_t *out_x, int32_t *out_y, int32_t max_tries);
 };
 
 class C4MapScriptMap : public C4MapScriptLayer
@@ -331,7 +340,7 @@ public:
 	virtual bool LoadData(const char *, const char *, C4LangStringTable *);
 	void Clear();
 	virtual C4PropListStatic * GetPropList();
-	bool InitializeMap(C4SLandscape *pLandscape, C4TextureMap *pTexMap, C4MaterialMap *pMatMap, uint32_t iPlayerCount, CSurface8 **pmap_surface);
+	bool InitializeMap(C4SLandscape *pLandscape, C4TextureMap *pTexMap, C4MaterialMap *pMatMap, uint32_t iPlayerCount, CSurface8 **pmap_fg_surface, CSurface8** pmap_bg_surface);
 	C4PropListStatic *GetLayerPrototype() { return LayerPrototype; }
 
 	C4TextureMap* pTexMap;
