@@ -20,6 +20,7 @@
 #ifndef INC_C4Texture
 #define INC_C4Texture
 
+#include <C4TextureShape.h>
 #include <C4Surface.h>
 #include <C4Constants.h>
 #include <C4Material.h>
@@ -34,9 +35,12 @@ public:
 
 	void SetAverageColor(uint32_t Color) { AvgColor = Color; }
 	uint32_t GetAverageColor() const { return AvgColor; }
+	void SetMaterialShape(class C4TextureShape *s) { material_shape.reset(s); }
+	class C4TextureShape *GetMaterialShape() const { return material_shape.get(); }
 protected:
 	StdStrBuf Name;
 	uint32_t AvgColor;
+	std::unique_ptr<class C4TextureShape> material_shape;
 	C4Texture *Next;
 };
 
@@ -69,6 +73,7 @@ public:
 	~C4TextureMap();
 protected:
 	C4TexMapEntry Entry[C4M_MaxTexIndex];
+	std::vector<int32_t> Order; // drawing order in map2landscape. Reflects order in MatMap.txt file.
 	C4Texture *FirstTexture;
 	bool fOverloadMaterials;
 	bool fOverloadTextures;
@@ -77,8 +82,7 @@ public:
 	bool fEntriesAdded;
 public:
 	const C4TexMapEntry *GetEntry(int32_t iIndex) const { return Inside<int32_t>(iIndex, 0, C4M_MaxTexIndex-1) ? &Entry[iIndex] : NULL; }
-	void RemoveEntry(int32_t iIndex) { if (Inside<int32_t>(iIndex, 1, C4M_MaxTexIndex-1)) Entry[iIndex].Clear(); }
-	void Default();
+	void RemoveEntry(int32_t iIndex);
 	void Clear();
 	void StoreMapPalette(CStdPalette *, C4MaterialMap &rMaterials);
 	static bool LoadFlags(C4Group &hGroup, const char *szEntryName, bool *pOverloadMaterials, bool *pOverloadTextures);
@@ -96,7 +100,9 @@ public:
 	bool AddEntry(BYTE byIndex, const char *szMaterial, const char *szTexture);
 	bool AddTexture(const char *szTexture, C4Surface * sfcSurface);
 	int32_t GetTextureIndex(const char *pTexName);
+	BYTE DefaultBkgMatTex(BYTE fg) const;
 protected:
+	friend class C4Landscape;
 };
 
 extern C4TextureMap TextureMap;
