@@ -73,7 +73,7 @@ void C4Weather::Execute()
 	// Temperature
 	if (!::Game.iTick35)
 	{
-		int32_t iTemperature=Climate-(int32_t)(TemperatureRange*cos(6.28*(float)Season/100.0));
+		int32_t iTemperature = Climate - fixtoi(Cos(itofix(36 * Season, 10)), TemperatureRange);
 		if (Temperature<iTemperature) Temperature++;
 		else if (Temperature>iTemperature) Temperature--;
 	}
@@ -84,8 +84,6 @@ void C4Weather::Execute()
 		Wind=Clamp<int32_t>(Wind+Sign(TargetWind-Wind),
 		                      Game.C4S.Weather.Wind.Min,
 		                      Game.C4S.Weather.Wind.Max);
-	if (!::Game.iTick10)
-		SoundLevel("Wind",NULL,Max(Abs(Wind)-30,0)*2);
 }
 
 void C4Weather::Clear()
@@ -95,7 +93,7 @@ void C4Weather::Clear()
 
 int32_t C4Weather::GetWind(int32_t x, int32_t y)
 {
-	if (GBackIFT(x,y)) return 0;
+	if (Landscape.GetBackPix(x,y) != 0) return 0;
 	return Wind;
 }
 
@@ -147,12 +145,12 @@ int32_t C4Weather::GetClimate()
 	return Climate;
 }
 
-static DWORD SeasonColors[4][3] =
+static float SeasonColors[4][3] =
 {
-	{ 0x000000, 0x7f7f90, 0xefefff }, // winter: slightly blue; blued out by temperature
-	{ 0x070f00, 0x90a07f, 0xffffdf }, // spring: green to yellow
-	{ 0x000000, 0x808080, 0xffffff }, // summer: regular ramp
-	{ 0x0f0700, 0xa08067, 0xffffdf }  // fall:   dark, brown ramp
+	{ 0.90f, 0.90f, 1.00f }, // winter: slightly blue; blued out by temperature
+	{ 1.00f, 1.05f, 0.90f }, // spring: green to yellow
+	{ 1.00f, 1.00f, 1.00f }, // summer: regular ramp
+	{ 1.00f, 0.95f, 0.90f }  // fall:   dark, brown ramp
 };
 
 void C4Weather::SetSeasonGamma()
@@ -161,28 +159,16 @@ void C4Weather::SetSeasonGamma()
 	// get season num and offset
 	int32_t iSeason1=(Season/25)%4; int32_t iSeason2=(iSeason1+1)%4;
 	int32_t iSeasonOff1=Clamp(Season%25, 5, 19)-5; int32_t iSeasonOff2=15-iSeasonOff1;
-	DWORD dwClr[3]; memset(dwClr, 0, sizeof(DWORD)*3);
+	float gamma[3] = { 0.0f, 0.0f, 0.0f };
 	// interpolate between season colors
-	for (int32_t i=0; i<3; ++i)
-		for (int32_t iChan=0; iChan<24; iChan+=8)
-		{
-			BYTE byC1=BYTE(SeasonColors[iSeason1][i]>>iChan);
-			BYTE byC2=BYTE(SeasonColors[iSeason2][i]>>iChan);
-			int32_t iChanVal=(byC1*iSeasonOff2 + byC2*iSeasonOff1) / 15;
-			// red+green: reduce in winter
-			if (Temperature<0)
-			{
-				if (iChan)
-					iChanVal+=Temperature/2;
-				else
-					// blue channel: emphasize in winter
-					iChanVal-=Temperature/2;
-			}
-			// set channel
-			dwClr[i] |= Clamp<int32_t>(iChanVal,0,255)<<iChan;
-		}
+	for (int32_t iChan=0; iChan<3; iChan+=8)
+	{
+		float c1 = SeasonColors[iSeason1][iChan],
+		      c2 = SeasonColors[iSeason2][iChan];
+		gamma[iChan] = (c1*iSeasonOff2 + c2*iSeasonOff1) / 15;
+	}
 	// apply gamma ramp
-	pDraw->SetGamma(dwClr[0], dwClr[1], dwClr[2], C4GRI_SEASON);
+	pDraw->SetGamma(gamma[0], gamma[1], gamma[2], C4GRI_SEASON);
 }
 
 void C4Weather::CompileFunc(StdCompiler *pComp)
@@ -196,14 +182,25 @@ void C4Weather::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(TemperatureRange, "TemperatureRange",      30));
 	pComp->Value(mkNamingAdapt(Climate,          "Climate",               0));
 	pComp->Value(mkNamingAdapt(NoGamma,          "NoGamma",               0));
-	uint32_t dwGammaDefaults[C4MaxGammaRamps*3];
+
+	int32_t gamma[C4MaxGammaRamps*3],
+	        gammaDefaults[C4MaxGammaRamps*3];
 	for (int32_t i=0; i<C4MaxGammaRamps; ++i)
 	{
-		dwGammaDefaults[i*3+0] = 0x000000;
-		dwGammaDefaults[i*3+1] = 0x808080;
-		dwGammaDefaults[i*3+2] = 0xffffff;
+		gammaDefaults[i*3+0] = 100;
+		gammaDefaults[i*3+1] = 100;
+		gammaDefaults[i*3+2] = 100;
+		gamma[i*3+0] = int(pDraw->gamma[i][0] * 100.0f);
+		gamma[i*3+1] = int(pDraw->gamma[i][1] * 100.0f);
+		gamma[i*3+2] = int(pDraw->gamma[i][2] * 100.0f);
+	}
+	pComp->Value(mkNamingAdapt(mkArrayAdaptM(gamma), "Gamma", gammaDefaults));
+	for (int32_t i=0; i<C4MaxGammaRamps; ++i)
+	{
+		pDraw->gamma[i][0] = float(gamma[i*3+0]) / 100.0f;
+		pDraw->gamma[i][1] = float(gamma[i*3+1]) / 100.0f;
+		pDraw->gamma[i][2] = float(gamma[i*3+2]) / 100.0f;
 	}
-	pComp->Value(mkNamingAdapt(mkArrayAdaptM(pDraw->dwGamma), "Gamma", dwGammaDefaults));
 }
 
 C4Weather Weather;
