@@ -29,16 +29,12 @@ const int C4MaxNameList = 10;
 class C4NameList
 {
 public:
-	C4NameList();
-public:
-	char Name[C4MaxNameList][C4MaxName+1];
-	int32_t Count[C4MaxNameList];
+	char Name[C4MaxNameList][C4MaxName + 1] = { { 0 } };
+	int32_t Count[C4MaxNameList] = { 0 };
 public:
 	void Clear();
 	bool Add(const char *szName, int32_t iCount=0);
 	bool Set(const char *szName, int32_t iCount);
-	bool Read(const char *szSource, int32_t iDefValue=0);
-	bool Write(char *szTarget, bool fValues=true);
 public:
 	bool IsEmpty();
 	bool operator==(const C4NameList& rhs)
