@@ -20,18 +20,9 @@
 #ifndef INC_C4Random
 #define INC_C4Random
 
-#include <ctime>
-
 extern int RandomCount;
-extern unsigned int RandomHold;
 
-inline void FixedRandom(DWORD dwSeed)
-{
-	// for SafeRandom
-	srand((unsigned)time(NULL));
-	RandomHold=dwSeed;
-	RandomCount=0;
-}
+void FixedRandom(DWORD dwSeed);
 
 int Random(int iRange);
 
