@@ -115,12 +115,6 @@ int32_t StrToI32(const char *s, int base, const char **scan_end)
 		result += value;
 	}
 	if (scan_end != 0L) *scan_end = s;
-	if (result < 0)
-	{
-		//overflow
-		// we need 2147483648 (2^31) to be -2147483648 in order for -2147483648 to work
-		//result = INT_MAX;
-	}
 	result *= sign;
 	return result;
 }
@@ -152,7 +146,7 @@ void SCopyUntil(const char *szSource, char *sTarget, char cUntil, int iMaxL, int
 
 void SCopyUntil(const char *szSource, char *sTarget, const char * sUntil, size_t iMaxL)
 {
-	size_t n = Min(strcspn(szSource, sUntil), iMaxL - 1);
+	size_t n = std::min(strcspn(szSource, sUntil), iMaxL - 1);
 	strncpy(sTarget, szSource, n);
 	sTarget[n] = 0;
 }
@@ -293,21 +287,6 @@ bool SCopySegmentEx(const char *szString, int iSegment, char *sTarget,
 	return true;
 }
 
-
-bool SCopyNamedSegment(const char *szString, const char *szName, char *sTarget,
-                       char cSeparator, char cNameSeparator, int iMaxL)
-{
-	// Advance to named segment
-	while (!( SEqual2(szString,szName) && (szString[SLen(szName)]==cNameSeparator) ))
-	{
-		if (SCharPos(cSeparator,szString)==-1) { sTarget[0]=0; return false; } // No more segments
-		szString += SCharPos(cSeparator,szString)+1;
-	}
-	// Copy segment contents
-	SCopyUntil(szString+SLen(szName)+1,sTarget,cSeparator,iMaxL);
-	return true;
-}
-
 unsigned int SCharCount(char cTarget, const char *szInStr, const char *cpUntil)
 {
 	unsigned int iResult=0;
@@ -351,35 +330,6 @@ void SCapitalize(char *str)
 	}
 }
 
-const char *SSearchIdentifier(const char *szString, const char *szIndex)
-{
-	// Does not check whether szIndex itself is an identifier.
-	// Just checks for space in front and back.
-	const char *cscr;
-	size_t indexlen,match=0;
-	bool frontok=true;
-	if (!szString || !szIndex) return NULL;
-	indexlen=SLen(szIndex);
-	for (cscr=szString; cscr && *cscr; cscr++)
-	{
-		// Match length
-		if (*cscr==szIndex[match]) match++;
-		else match=0;
-		// String is matched, front and back ok?
-		if (match>=indexlen)
-			if (frontok)
-				if (!IsIdentifier(*(cscr+1)))
-					return cscr+1;
-		// Currently no match, check for frontok
-		if (match==0)
-		{
-			if (IsIdentifier(*cscr)) frontok=false;
-			else frontok=true;
-		}
-	}
-	return NULL;
-}
-
 const char *SSearch(const char *szString, const char *szIndex)
 {
 	const char *cscr;
@@ -520,71 +470,6 @@ int SLineGetCharacters(const char *szText, const char *cpPosition)
 	return iChars;
 }
 
-void SRemoveComments(char *szScript)
-{
-	const char *pScriptCont;
-	if (!szScript) return;
-	while (*szScript)
-	{
-		// Advance to next slash
-		while (*szScript && (*szScript!='/')) szScript++;
-		if (!(*szScript)) return; // No more comments
-		// Line comment
-		if (szScript[1]=='/')
-		{
-			if ((pScriptCont = SSearch(szScript+2,LineFeed)))
-				SCopy(pScriptCont-SLen(LineFeed),szScript);
-			else
-				szScript[0]=0;
-		}
-		// Block comment
-		else if (szScript[1]=='*')
-		{
-			if ((pScriptCont = SSearch(szScript+2,"*/")))
-				SCopy(pScriptCont,szScript);
-			else
-				szScript[0]=0;
-		}
-		// No comment
-		else
-		{
-			szScript++;
-		}
-	}
-}
-
-bool SCopyPrecedingIdentifier(const char *pBegin, const char *pIdentifier, char *sTarget, int iSize)
-{
-	// Safety
-	if (!pIdentifier || !sTarget || !pBegin) return false;
-	// Empty default
-	sTarget[0]=0;
-	// Identifier is at begin
-	if (!(pIdentifier>pBegin)) return false;
-	// Rewind space
-	const char *cPos;
-	if (!(cPos = SRewindSpace(pIdentifier-1,pBegin))) return false;
-	// Rewind to beginning of identifier
-	while ((cPos>pBegin) && IsIdentifier(cPos[-1])) cPos--;
-	// Copy identifier
-	SCopyIdentifier(cPos,sTarget,iSize);
-	// Success
-	return true;
-}
-
-const char *SSearchFunction(const char *szString, const char *szIndex)
-{
-	// Safety
-	if (!szString || !szIndex) return NULL;
-	// Ignore failsafe
-	if (szIndex[0]=='~') szIndex++;
-	// Buffer to append colon
-	char szDeclaration[256+2];
-	SCopy(szIndex,szDeclaration,256); SAppendChar(':',szDeclaration);
-	// Search identifier
-	return SSearchIdentifier(szString,szDeclaration);
-}
-
 void SInsert(char *szString, const char *szInsert, int iPosition, int iMaxLen)
 {
 	// Safety
@@ -611,7 +496,7 @@ bool SCopyEnclosed(const char *szSource, char cOpen, char cClose, char *sTarget,
 	if (!szSource || !sTarget) return false;
 	if ((iPos = SCharPos(cOpen,szSource)) < 0) return false;
 	if ((iLen = SCharPos(cClose,szSource+iPos+1)) < 0) return false;
-	SCopy(szSource+iPos+1,sTarget,Min(iLen,iSize));
+	SCopy(szSource+iPos+1,sTarget,std::min(iLen,iSize));
 	return true;
 }
 
