@@ -179,7 +179,7 @@ public:
 	void Value(bool &rBool)    { Boolean(rBool); }
 
 	// Compiling/Decompiling (may throw a data format exception!)
-	template <class T> inline void Compile(T RREF rStruct)
+	template <class T> inline void Compile(T &&rStruct)
 	{
 		assert(isCompiler());
 		DoCompilation(rStruct);
@@ -343,7 +343,6 @@ void CompileNewFunc(T *&pStruct, StdCompiler *pComp, const P& rPar)
 	//    behaviour is to construct the object from compiler data
 	std::unique_ptr<T> temp(new T); // exception-safety
 	// Compile
-	//temp->CompileFunc(pComp, rPar);
 	pComp->Value(mkParAdapt(*temp, rPar));
 	pStruct = temp.release();
 }
@@ -374,14 +373,13 @@ void CompileNewFuncCtx(T *&pStruct, StdCompiler *pComp, const ContextT& rCtx, co
 	//    and context
 	std::unique_ptr<T> temp(new T(rCtx));  // exception-safety
 	// Compile
-	//temp->CompileFunc(pComp, rPar);
 	pComp->Value(mkParAdapt(*temp, rPar));
 	pStruct = temp.release();
 }
 
 // Helpers for buffer-based compiling (may throw a data format exception!)
 template <class CompT, class StructT>
-void CompileFromBuf(StructT RREF TargetStruct, const typename CompT::InT &SrcBuf)
+void CompileFromBuf(StructT &&TargetStruct, const typename CompT::InT &SrcBuf)
 {
 	CompT Compiler;
 	Compiler.setInput(SrcBuf.getRef());
@@ -492,7 +490,7 @@ public:
 
 	// Input
 	typedef StdBuf InT;
-	void setInput(InT RREF In) { Buf = std::move(In); }
+	void setInput(InT &&In) { Buf = std::move(In); }
 
 	// Properties
 	virtual bool isCompiler()                     { return true; }
@@ -736,7 +734,7 @@ private:
 void StdCompilerWarnCallback(void *pData, const char *szPosition, const char *szError);
 
 template <class CompT, class StructT>
-bool CompileFromBuf_Log(StructT &TargetStruct, const typename CompT::InT &SrcBuf, const char *szName)
+bool CompileFromBuf_Log(StructT &&TargetStruct, const typename CompT::InT &SrcBuf, const char *szName)
 {
 	try
 	{
@@ -745,13 +743,16 @@ bool CompileFromBuf_Log(StructT &TargetStruct, const typename CompT::InT &SrcBuf
 	}
 	catch (StdCompiler::Exception *pExc)
 	{
-		LogF("ERROR: %s (in %s)", pExc->Msg.getData(), szName);
+		if (!pExc->Pos.getLength())
+			LogF("ERROR: %s (in %s)", pExc->Msg.getData(), szName);
+		else
+			LogF("ERROR: %s (in %s, %s)", pExc->Msg.getData(), pExc->Pos.getData(), szName);
 		delete pExc;
 		return false;
 	}
 }
 template <class CompT, class StructT>
-bool CompileFromBuf_LogWarn(StructT RREF TargetStruct, const typename CompT::InT &SrcBuf, const char *szName)
+bool CompileFromBuf_LogWarn(StructT &&TargetStruct, const typename CompT::InT &SrcBuf, const char *szName)
 {
 	try
 	{
@@ -772,7 +773,7 @@ bool CompileFromBuf_LogWarn(StructT RREF TargetStruct, const typename CompT::InT
 	}
 }
 template <class CompT, class StructT>
-bool DecompileToBuf_Log(StructT RREF TargetStruct, typename CompT::OutT *pOut, const char *szName)
+bool DecompileToBuf_Log(StructT &&TargetStruct, typename CompT::OutT *pOut, const char *szName)
 {
 	if (!pOut) return false;
 	try
