@@ -39,9 +39,33 @@ typedef enum _MapeMapgenError {
   MAPE_MAPGEN_ERROR_MEMORY
 } MapeMapgenError;
 
+/**
+ * MapeMapgenType:
+ * @MAPE_MAPGEN_NONE: Does not represent a map description.
+ * @MAPE_MAPGEN_LANDSCAPE_TXT: Represents a Landscape.txt script.
+ * @MAPE_MAPGEN_MAP_C: Represents a Map.c script.
+ *
+ * Specifies the different types of maps that can be rendered.
+ */
+typedef enum _MapeMapgenType {
+  MAPE_MAPGEN_NONE,
+  MAPE_MAPGEN_LANDSCAPE_TXT,
+  MAPE_MAPGEN_MAP_C
+} MapeMapgenType;
+
+gboolean
+mape_mapgen_init(GError** error);
+
+void
+mape_mapgen_deinit();
+
+void
+mape_mapgen_set_root_group(MapeGroup* group);
+
 GdkPixbuf*
 mape_mapgen_render(const gchar* filename,
                    const gchar* source,
+                   MapeMapgenType type,
                    const gchar* script_path,
                    MapeMaterialMap* material_map,
                    MapeTextureMap* texture_map,
