@@ -169,9 +169,12 @@ void C4GameControlNetwork::DoInput(const C4Control &Input) // by main thread
 				Application.InteractiveThread.ThreadLog("Failed to send control to host!");
 	}
 	// decentral mode: always broadcast to everybody
-	else /*if(eMode == CNM_Decentral)*/
+	else
+	{
+		assert (eMode == CNM_Decentral);
 		if (!pNetwork->Clients.BroadcastMsgToClients(CtrlPkt))
 			Application.InteractiveThread.ThreadLog("Failed to broadcast control!");
+	}
 	// add to list
 	AddCtrl(pCtrl);
 	// ok, control is sent for this control tick
@@ -459,7 +462,7 @@ void C4GameControlNetwork::HandlePacket(char cStatus, const C4PacketBase *pPacke
 
 #define GETPKT(type, name) \
     assert(pPacket); const type &name = \
-      /*dynamic_cast*/ static_cast<const type &>(*pPacket);
+      static_cast<const type &>(*pPacket);
 
 	switch (cStatus)
 	{
@@ -792,7 +795,7 @@ C4GameControlPacket *C4GameControlNetwork::PackCompleteCtrl(int32_t iTick)
 		::Network.Clients.BroadcastMsgToConnClients(MkC4NetIOPacket(PID_Control, *pComplete));
 
 	// advance control request time
-	tNextControlRequest = Max(tNextControlRequest, C4TimeMilliseconds::Now() + C4ControlRequestInterval);
+	tNextControlRequest = std::max(tNextControlRequest, C4TimeMilliseconds::Now() + C4ControlRequestInterval);
 
 	// return
 	return pComplete;
