@@ -150,10 +150,10 @@ C4Network2ClientListBox::ClientListItem::ClientListItem(class C4Network2ClientLi
 		// activate/deactivate and kick btns for clients at host
 		if (!pForDlg->IsStartup())
 		{
-			pActivateBtn = new C4GUI::CallbackButtonEx<C4Network2ClientListBox::ClientListItem, C4GUI::IconButton>(C4GUI::Ico_Active, GetToprightCornerRect(Max(iIconSize, 16),Max(iIconSize, 16),2,1,1), 0, this, &ClientListItem::OnButtonActivate);
+			pActivateBtn = new C4GUI::CallbackButtonEx<C4Network2ClientListBox::ClientListItem, C4GUI::IconButton>(C4GUI::Ico_Active, GetToprightCornerRect(std::max(iIconSize, 16),std::max(iIconSize, 16),2,1,1), 0, this, &ClientListItem::OnButtonActivate);
 			fShownActive = true;
 		}
-		pKickBtn = new  C4GUI::CallbackButtonEx<C4Network2ClientListBox::ClientListItem, C4GUI::IconButton>(C4GUI::Ico_Kick, GetToprightCornerRect(Max(iIconSize, 16),Max(iIconSize, 16),2,1,0), 0, this, &ClientListItem::OnButtonKick);
+		pKickBtn = new  C4GUI::CallbackButtonEx<C4Network2ClientListBox::ClientListItem, C4GUI::IconButton>(C4GUI::Ico_Kick, GetToprightCornerRect(std::max(iIconSize, 16),std::max(iIconSize, 16),2,1,0), 0, this, &ClientListItem::OnButtonKick);
 		pKickBtn->SetToolTip(LoadResStrNoAmp("IDS_NET_KICKCLIENT"));
 	}
 	if (!pForDlg->IsStartup()) if (pClient && !pClient->isLocal())
@@ -325,7 +325,6 @@ C4Network2ClientListBox::ConnectionListItem::ConnectionListItem(class C4Network2
 	ca.ExpandLeft(-iIconSize*2);
 	// create subcomponents
 	// reconnect/disconnect buttons
-	//pReconnectBtn = new C4GUI::CallbackButtonEx<C4Network2ClientListBox::ConnectionListItem, C4GUI::IconButton>(C4GUI::Ico_Notify, ca.GetFromRight(iIconSize, iIconSize), 0, this, &ConnectionListItem::OnButtonReconnect);
 	if (!Game.Parameters.isLeague())
 	{
 		pDisconnectBtn = new  C4GUI::CallbackButtonEx<C4Network2ClientListBox::ConnectionListItem, C4GUI::IconButton>(C4GUI::Ico_Disconnect, ca.GetFromRight(iIconSize, iIconSize), 0, this, &ConnectionListItem::OnButtonDisconnect);
@@ -343,7 +342,6 @@ C4Network2ClientListBox::ConnectionListItem::ConnectionListItem(class C4Network2
 	// add components
 	AddElement(pDesc);
 	AddElement(pPing);
-	//AddElement(pReconnectBtn);
 	if (pDisconnectBtn) AddElement(pDisconnectBtn);
 	// add to listbox (will eventually get moved)
 	pForDlg->AddElement(this);
@@ -575,7 +573,7 @@ C4Network2StartWaitDlg::C4Network2StartWaitDlg()
 	AddElement(pClientListBox = new C4Network2ClientListBox(caAll.GetAll(), true));
 	// place abort button
 	C4GUI::Button *pBtnAbort = new C4GUI::CancelButton(caButtonArea.GetCentered(C4GUI_DefButtonWdt, C4GUI_ButtonHgt));
-	AddElement(pBtnAbort); //pBtnAbort->SetToolTip(LoadResStr("IDS_DLGTIP_CANCEL"));
+	AddElement(pBtnAbort);
 }
 
 
@@ -589,12 +587,12 @@ C4GameOptionButtons::C4GameOptionButtons(const C4Rect &rcBounds, bool fNetwork,
 	SetBounds(rcBounds);
 	// calculate button size from area
 	int32_t iButtonCount = fNetwork ? fHost ? 6 : 3 : 2;
-	int32_t iIconSize = Min<int32_t>(C4GUI_IconExHgt, rcBounds.Hgt), iIconSpacing = rcBounds.Wdt/(rcBounds.Wdt >= 400 ? 64 : 128);
+	int32_t iIconSize = std::min<int32_t>(C4GUI_IconExHgt, rcBounds.Hgt), iIconSpacing = rcBounds.Wdt/(rcBounds.Wdt >= 400 ? 64 : 128);
 	if ((iIconSize+iIconSpacing*2)*iButtonCount > rcBounds.Wdt)
 	{
 		if (iIconSize*iButtonCount <= rcBounds.Wdt)
 		{
-			iIconSpacing = Max<int32_t>(0, (rcBounds.Wdt-iIconSize*iButtonCount)/(iButtonCount*2)-1);
+			iIconSpacing = std::max<int32_t>(0, (rcBounds.Wdt-iIconSize*iButtonCount)/(iButtonCount*2)-1);
 		}
 		else
 		{
@@ -614,8 +612,7 @@ C4GameOptionButtons::C4GameOptionButtons(const C4Rect &rcBounds, bool fNetwork,
 			fIsInternet = true;
 			fIsDisabled = true;
 		}
-		btnInternet = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(fIsInternet ? C4GUI::Ico_Ex_InternetOn : C4GUI::Ico_Ex_InternetOff, caButtons.GetFromLeft(iIconSize, iIconSize), 'I' /* 2do */, &C4GameOptionButtons::OnBtnInternet, this);
-		btnInternet->SetToolTip(LoadResStr("IDS_DLGTIP_STARTINTERNETGAME"));
+		btnInternet = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(fIsInternet ? C4GUI::Ico_Ex_InternetOn : C4GUI::Ico_Ex_InternetOff, caButtons.GetFromLeft(iIconSize, iIconSize), LoadResStr("IDS_DLGTIP_STARTINTERNETGAME"), &C4GameOptionButtons::OnBtnInternet, this);
 		btnInternet->SetEnabled(!fIsDisabled);
 		AddElement(btnInternet);
 	}
@@ -627,25 +624,21 @@ C4GameOptionButtons::C4GameOptionButtons(const C4Rect &rcBounds, bool fNetwork,
 		C4GUI::Icons eLeagueIcon;
 		fIsLeague = fLobby ? Game.Parameters.isLeague() : !!Config.Network.LeagueServerSignUp;
 		eLeagueIcon = fIsLeague ? C4GUI::Ico_Ex_LeagueOn : C4GUI::Ico_Ex_LeagueOff;
-		btnLeague = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(eLeagueIcon, caButtons.GetFromLeft(iIconSize, iIconSize), 'L' /* 2do */, &C4GameOptionButtons::OnBtnLeague, this);
-		btnLeague->SetToolTip(LoadResStr("IDS_DLGTIP_STARTLEAGUEGAME"));
+		btnLeague = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(eLeagueIcon, caButtons.GetFromLeft(iIconSize, iIconSize), LoadResStr("IDS_DLGTIP_STARTLEAGUEGAME"), &C4GameOptionButtons::OnBtnLeague, this);
 		btnLeague->SetEnabled(fHost && !fLobby);
 		AddElement(btnLeague);
 	}
 	else btnLeague=NULL;
 	if (fNetwork && fHost)
 	{
-		btnPassword = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(::Network.isPassworded() ? C4GUI::Ico_Ex_Locked : C4GUI::Ico_Ex_Unlocked, caButtons.GetFromLeft(iIconSize, iIconSize), 'P' /* 2do */, &C4GameOptionButtons::OnBtnPassword, this);
-		btnPassword->SetToolTip(LoadResStr("IDS_NET_PASSWORD_DESC"));
+		btnPassword = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(::Network.isPassworded() ? C4GUI::Ico_Ex_Locked : C4GUI::Ico_Ex_Unlocked, caButtons.GetFromLeft(iIconSize, iIconSize), LoadResStr("IDS_NET_PASSWORD_DESC"), &C4GameOptionButtons::OnBtnPassword, this);
 		AddElement(btnPassword);
-		btnComment = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(C4GUI::Ico_Ex_Comment, caButtons.GetFromLeft(iIconSize, iIconSize), 'M' /* 2do */, &C4GameOptionButtons::OnBtnComment, this);
-		btnComment->SetToolTip(LoadResStr("IDS_DESC_COMMENTDESCRIPTIONFORTHIS"));
+		btnComment = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(C4GUI::Ico_Ex_Comment, caButtons.GetFromLeft(iIconSize, iIconSize), LoadResStr("IDS_DESC_COMMENTDESCRIPTIONFORTHIS"), &C4GameOptionButtons::OnBtnComment, this);
 		AddElement(btnComment);
 	}
 	else btnPassword=btnComment=NULL;
-	btnRecord = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(Game.Record || fIsLeague ? C4GUI::Ico_Ex_RecordOn : C4GUI::Ico_Ex_RecordOff, caButtons.GetFromLeft(iIconSize, iIconSize), 'R' /* 2do */, &C4GameOptionButtons::OnBtnRecord, this);
+	btnRecord = new C4GUI::CallbackButton<C4GameOptionButtons, C4GUI::IconButton>(Game.Record || fIsLeague ? C4GUI::Ico_Ex_RecordOn : C4GUI::Ico_Ex_RecordOff, caButtons.GetFromLeft(iIconSize, iIconSize), LoadResStr("IDS_DLGTIP_RECORD"), &C4GameOptionButtons::OnBtnRecord, this);
 	btnRecord->SetEnabled(!fIsLeague);
-	btnRecord->SetToolTip(LoadResStr("IDS_DLGTIP_RECORD"));
 	AddElement(btnRecord);
 }
 
@@ -730,7 +723,7 @@ void C4GameOptionButtons::OnPasswordSet(const StdStrBuf &rsNewPassword)
 		SCopy(szPass, Config.Network.LastPassword, CFG_MaxString);
 	}
 	// acoustic feedback
-	C4GUI::GUISound("Connect");
+	C4GUI::GUISound("UI::Confirmed");
 }
 
 void C4GameOptionButtons::UpdatePasswordBtn()
@@ -738,7 +731,6 @@ void C4GameOptionButtons::UpdatePasswordBtn()
 	// update icon to reflect if a password is set
 	const char *szPass = ::Network.GetPassword();
 	bool fHasPassword = szPass && *szPass;
-//btnPassword->SetHighlight(fHasPassword);
 	btnPassword->SetIcon(fHasPassword ? C4GUI::Ico_Ex_Locked : C4GUI::Ico_Ex_Unlocked);
 }
 
@@ -761,7 +753,7 @@ void C4GameOptionButtons::OnCommentSet(const StdStrBuf &rsNewComment)
 	// message feedback
 	Log(LoadResStr("IDS_NET_COMMENTCHANGED"));
 	// acoustic feedback
-	C4GUI::GUISound("Connect");
+	C4GUI::GUISound("UI::Confirmed");
 }
 
 void C4GameOptionButtons::SetCountdown(bool fToVal)
@@ -831,7 +823,7 @@ void C4Chart::DrawElement(C4TargetFacet &cgo)
 		iMinVal = ((iMinVal-(iMinVal<0))/ddv+(iMinVal<0))*ddv;
 	ValueType dv=iMaxVal-iMinVal; TimeType dt=iMaxTime-iMinTime;
 	// axis calculations
-	sbuf.Format("-%d", (int) Max(Abs(iMaxVal), Abs(iMinVal)));
+	sbuf.Format("-%d", (int) std::max(Abs(iMaxVal), Abs(iMinVal)));
 	rFont.GetTextExtent(sbuf.getData(), XAxisMinStepWdt, YAxisMinStepHgt, false);
 	YAxisWdt += XAxisMinStepWdt; XAxisHgt += YAxisMinStepHgt;
 	XAxisMinStepWdt += 2; YAxisMinStepHgt += 2;
@@ -847,7 +839,7 @@ void C4Chart::DrawElement(C4TargetFacet &cgo)
 		while ((pSeries = pDisplayGraph->GetSeries(iSeries++)))
 		{
 			rFont.GetTextExtent(pSeries->GetTitle(), W, Q, true);
-			iLegendWdt = Max(iLegendWdt, W);
+			iLegendWdt = std::max(iLegendWdt, W);
 		}
 		tw -= iLegendWdt+1;
 		iSeries = 0;
@@ -894,15 +886,15 @@ void C4Chart::DrawElement(C4TargetFacet &cgo)
 	int iSeries = 0;
 	while (const C4Graph *pSeries = pDisplayGraph->GetSeries(iSeries++))
 	{
-		int iThisMinTime = Max(iMinTime, pSeries->GetStartTime());
-		int iThisMaxTime = Min(iMaxTime, pSeries->GetEndTime());
+		int iThisMinTime = std::max(iMinTime, pSeries->GetStartTime());
+		int iThisMaxTime = std::min(iMaxTime, pSeries->GetEndTime());
 		bool fAnyVal = false;
 		for (iX = 0; iX<tw; ++iX)
 		{
 			iTime = iMinTime + dt*iX/tw;
 			if (!Inside(iTime, iThisMinTime, iThisMaxTime)) continue;
 			int iY2 = int((-pSeries->GetValue(iTime) + iMinVal) * th / dv) + ty+th;
-			if (fAnyVal) pDraw->DrawLineDw(cgo.Surface, (float) (tx+iX-1), (float) iY, (float) (tx+iX), (float) iY2, pSeries->GetColorDw());
+			if (fAnyVal) pDraw->DrawLineDw(cgo.Surface, (float) (tx+iX-1), (float) iY, (float) (tx+iX), (float) iY2, pSeries->GetColorDw() | 0xff000000);
 			iY = iY2;
 			fAnyVal = true;
 		}
