@@ -52,11 +52,16 @@ void C4Network2Reference::InitLocal()
 	Parameters = ::Game.Parameters;
 
 	// Discard player resources (we don't want these infos in the reference)
+	// Add league performance (but only after game end)
 	C4ClientPlayerInfos *pClientInfos; C4PlayerInfo *pPlayerInfo;
 	int32_t i, j;
 	for (i = 0; (pClientInfos = Parameters.PlayerInfos.GetIndexedInfo(i)); i++)
 		for (j = 0; (pPlayerInfo = pClientInfos->GetPlayerInfo(j)); j++)
+		{
 			pPlayerInfo->DiscardResource();
+			if(::Game.GameOver)
+				pPlayerInfo->SetLeaguePerformance(::Game.RoundResults.GetLeaguePerformance(pPlayerInfo->GetID()));
+		}
 
 	// Special additional information in reference
 	Icon = ::Game.C4S.Head.Icon;
@@ -110,7 +115,7 @@ void C4Network2Reference::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(ObservingAllowed,  "ObservingAllowed", true));
 	pComp->Value(mkNamingAdapt(PasswordNeeded,    "PasswordNeeded",   false));
 	pComp->Value(mkNamingAdapt(mkIntPackAdapt(iAddrCnt), "AddressCount", 0));
-	iAddrCnt = Min<uint8_t>(C4ClientMaxAddr, iAddrCnt);
+	iAddrCnt = std::min<uint8_t>(C4ClientMaxAddr, iAddrCnt);
 	pComp->Value(mkNamingAdapt(mkArrayAdapt(Addrs, iAddrCnt, C4Network2Address()), "Address"));
 	pComp->Value(mkNamingAdapt(Game.sEngineName,      "Game",             "None"));
 	pComp->Value(mkNamingAdapt(mkArrayAdaptDM(Game.iVer,0),"Version"    ));
@@ -346,7 +351,7 @@ bool C4Network2HTTPClient::Decompress(StdBuf *pData)
 	size_t iSize = pData->getSize();
 	// Create buffer
 	uint32_t iOutSize = *getBufPtr<uint32_t>(*pData, pData->getSize() - sizeof(uint32_t));
-	iOutSize = Min<uint32_t>(iOutSize, iSize * 1000);
+	iOutSize = std::min<uint32_t>(iOutSize, iSize * 1000);
 	StdBuf Out; Out.New(iOutSize);
 	// Prepare stream
 	z_stream zstrm;
@@ -429,9 +434,9 @@ C4TimeMilliseconds C4Network2HTTPClient::GetNextTick(C4TimeMilliseconds tNow)
 	if (!fBusy)
 		return tNetIOTCPTick;
 
-	C4TimeMilliseconds tHTTPClientTick = tNow + 1000 * Max<time_t>(iRequestTimeout - time(NULL), 0);
+	C4TimeMilliseconds tHTTPClientTick = tNow + 1000 * std::max<time_t>(iRequestTimeout - time(NULL), 0);
 
-	return Max(tNetIOTCPTick, tHTTPClientTick);
+	return std::max(tNetIOTCPTick, tHTTPClientTick);
 }
 
 bool C4Network2HTTPClient::Query(const StdBuf &Data, bool fBinary)
