@@ -18,13 +18,15 @@
 #define INC_C4Network2Upnp
 
 #include "network/C4Network2IO.h"
-#include <boost/noncopyable.hpp>
 
-class C4Network2UPnP : boost::noncopyable
+class C4Network2UPnP
 {
 	struct C4Network2UPnPP *p;
 public:
 	C4Network2UPnP();
+	//noncopyable
+	C4Network2UPnP(const C4Network2UPnP&) = delete;
+	C4Network2UPnP& operator=(const C4Network2UPnP&) = delete;
 	~C4Network2UPnP();
 
 	void AddMapping(enum C4Network2IOProtocol protocol, uint16_t intport, uint16_t extport);
