@@ -25,9 +25,7 @@
 #include <string>
 #include <vector>
 
-//#include <boost/operators.hpp>
-
-class C4ID //: boost::totally_ordered<C4ID, boost::equivalent<C4ID> >
+class C4ID
 {
 public:
 	typedef size_t Handle;
@@ -45,7 +43,6 @@ public:
 		DEPRECATED static const C4ID Flag;
 	DEPRECATED static const C4ID Conkit; // Construction kit
 	static const C4ID Clonk;
-	DEPRECATED static const C4ID Flame;
 	static const C4ID Melee;
 	static const C4ID Bubble;
 
