@@ -23,12 +23,13 @@
 #include <C4Window.h>
 #include <C4Draw.h>
 
+#include "C4App.h"
 #import <Cocoa/Cocoa.h>
+
+#ifndef USE_CONSOLE
 #import "C4WindowController.h"
 #import "C4DrawGLMac.h"
 
-#include "C4App.h"
-
 bool C4AbstractApp::Copy(const StdStrBuf & text, bool fClipboard)
 {
 	NSPasteboard* pasteboard = [NSPasteboard generalPasteboard];
@@ -151,12 +152,7 @@ void C4AbstractApp::RestoreVideoMode()
 {
 }
 
-StdStrBuf C4AbstractApp::GetGameDataPath()
-{
-	return StdCopyStrBuf([[[NSBundle mainBundle] resourcePath] fileSystemRepresentation]);
-}
-
-bool C4AbstractApp::SetVideoMode(unsigned int iXRes, unsigned int iYRes, unsigned int iColorDepth, unsigned int iRefreshRate, unsigned int iMonitor, bool fFullScreen)
+bool C4AbstractApp::SetVideoMode(int iXRes, int iYRes, unsigned int iColorDepth, unsigned int iRefreshRate, unsigned int iMonitor, bool fFullScreen)
 {
 	fFullScreen &= !lionAndBeyond(); // Always false for Lion since then Lion's true(tm) Fullscreen is used
 	C4WindowController* controller = pWindow->objectiveCObject<C4WindowController>();
@@ -190,38 +186,8 @@ bool C4AbstractApp::SetVideoMode(unsigned int iXRes, unsigned int iYRes, unsigne
 	return true;
 }
 
-bool C4AbstractApp::ApplyGammaRamp(struct _GAMMARAMP &ramp, bool fForce)
-{
-	CGGammaValue r[256];
-	CGGammaValue g[256];
-	CGGammaValue b[256];
-	for (int i = 0; i < 256; i++)
-	{
-		r[i] = static_cast<float>(ramp.red[i])/65535.0;
-		g[i] = static_cast<float>(ramp.green[i])/65535.0;
-		b[i] = static_cast<float>(ramp.blue[i])/65535.0;
-	}
-	CGSetDisplayTransferByTable(C4OpenGLView.displayID, 256, r, g, b);
-	return true;
-}
-
-bool C4AbstractApp::SaveDefaultGammaRamp(struct _GAMMARAMP &ramp)
-{
-	CGGammaValue r[256];
-	CGGammaValue g[256];
-	CGGammaValue b[256];
-	uint32_t count;
-	CGGetDisplayTransferByTable(C4OpenGLView.displayID, 256, r, g, b, &count);
-	for (int i = 0; i < 256; i++)
-	{
-		ramp.red[i]   = r[i]*65535;
-		ramp.green[i] = g[i]*65535;
-		ramp.blue[i]  = b[i]*65535;
-	}
-	return true;
-}
-
-#endif
+#endif // USE_COCOA
+#endif // USE_CONSOLE
 
 bool IsGermanSystem()
 {
@@ -246,3 +212,8 @@ bool EraseItemSafe(const char* szFilename)
 		files: [NSArray arrayWithObject: [filename lastPathComponent]]
 		tag: 0];
 }
+
+StdStrBuf C4AbstractApp::GetGameDataPath()
+{
+	return StdCopyStrBuf([[[NSBundle mainBundle] resourcePath] fileSystemRepresentation]);
+}
