@@ -30,8 +30,10 @@
 #include <tlhelp32.h>
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
-#else
-
+#endif
+#include <assert.h>
+#if defined(__CRT_WIDE) || (defined(_MSC_VER) && _MSC_VER >= 1900)
+#define USE_WIDE_ASSERT
 #endif
 
 static bool FirstCrash = true;
@@ -162,13 +164,17 @@ namespace {
 				LOG_STATIC_TEXT("Additional information for the exception was not provided.\n");
 				break;
 			}
-#ifdef __CRT_WIDE
+#ifdef USE_WIDE_ASSERT
 #	define ASSERTION_INFO_FORMAT "%ls"
+#	define ASSERTION_INFO_TYPE wchar_t *
 #else
 #	define ASSERTION_INFO_FORMAT "%s"
+#	define ASSERTION_INFO_TYPE char *
 #endif
 			LOG_DYNAMIC_TEXT("Additional information for the exception:\n    Assertion that failed: " ASSERTION_INFO_FORMAT "\n    File: " ASSERTION_INFO_FORMAT "\n    Line: %d\n",
-				exc->ExceptionRecord->ExceptionInformation[0], exc->ExceptionRecord->ExceptionInformation[1], exc->ExceptionRecord->ExceptionInformation[2]);
+				reinterpret_cast<ASSERTION_INFO_TYPE>(exc->ExceptionRecord->ExceptionInformation[0]),
+				reinterpret_cast<ASSERTION_INFO_TYPE>(exc->ExceptionRecord->ExceptionInformation[1]),
+				(int) exc->ExceptionRecord->ExceptionInformation[2]);
 			break;
 		}
 
@@ -267,7 +273,7 @@ namespace {
 		if (SymInitialize(process, 0, true))
 		{
 			LOG_STATIC_TEXT("\nStack trace:\n");
-			STACKFRAME64 frame = {0};
+			auto frame = STACKFRAME64();
 			DWORD image_type;
 			CONTEXT context = *exc->ContextRecord;
 			// Setup frame info
@@ -313,11 +319,11 @@ namespace {
 				}
 				else if (image_base > 0)
 				{
-					LOG_DYNAMIC_TEXT("+%#lx", static_cast<size_t>(frame.AddrPC.Offset - image_base));
+					LOG_DYNAMIC_TEXT("+%#lx", static_cast<long>(frame.AddrPC.Offset - image_base));
 				}
 				else
 				{
-					LOG_DYNAMIC_TEXT("%#lx", static_cast<size_t>(frame.AddrPC.Offset));
+					LOG_DYNAMIC_TEXT("%#lx", static_cast<long>(frame.AddrPC.Offset));
 				}
 				DWORD disp;
 				line->SizeOfStruct = sizeof(*line);
@@ -434,8 +440,8 @@ LONG WINAPI GenerateDump(EXCEPTION_POINTERS* pExceptionPointers)
 
 	if (file != INVALID_HANDLE_VALUE)
 	{
-		MINIDUMP_USER_STREAM_INFORMATION user_stream_info = {0};
-		MINIDUMP_USER_STREAM user_stream = {0};
+		auto user_stream_info = MINIDUMP_USER_STREAM_INFORMATION();
+		auto user_stream = MINIDUMP_USER_STREAM();
 		char build_id[] = OC_BUILD_ID;
 		if (OC_BUILD_ID[0] != '\0')
 		{
@@ -466,7 +472,7 @@ namespace {
 	// replaces the trampoline with the original prologue, and calls the handler.
 	// If the standard handler returns control to assertion_handler(), it will then
 	// restore the hook.
-#ifdef __CRT_WIDE
+#ifdef USE_WIDE_ASSERT
 	typedef void (__cdecl *ASSERT_FUNC)(const wchar_t *, const wchar_t *, unsigned);
 	const ASSERT_FUNC assert_func = 
 		&_wassert;
@@ -521,7 +527,7 @@ namespace {
 
 	struct dump_thread_t {
 		HANDLE thread;
-#ifdef __CRT_WIDE
+#ifdef USE_WIDE_ASSERT
 		const wchar_t
 #else
 		const char
@@ -539,7 +545,7 @@ namespace {
 			return FALSE;
 
 		// Get thread info
-		CONTEXT ctx = {0};
+		auto ctx = CONTEXT();
 #ifndef CONTEXT_ALL
 #define CONTEXT_ALL (CONTEXT_CONTROL | CONTEXT_INTEGER | CONTEXT_SEGMENTS | \
 	CONTEXT_FLOATING_POINT | CONTEXT_DEBUG_REGISTERS | CONTEXT_EXTENDED_REGISTERS)
@@ -548,7 +554,7 @@ namespace {
 		BOOL result = GetThreadContext(data->thread, &ctx);
 
 		// Setup a fake exception to log
-		EXCEPTION_RECORD erec = {0};
+		auto erec = EXCEPTION_RECORD();
 		erec.ExceptionCode = STATUS_ASSERTION_FAILURE;
 		erec.ExceptionFlags = 0L;
 		erec.ExceptionInformation[0] = (ULONG_PTR)data->expression;
@@ -580,7 +586,7 @@ namespace {
 	}
 
 	// Replacement assertion handler
-#ifdef __CRT_WIDE
+#ifdef USE_WIDE_ASSERT
 	void __cdecl assertion_handler(const wchar_t *expression, const wchar_t *file, unsigned line)
 #else
 	void __cdecl assertion_handler(const char *expression, const char *file, int line)
