@@ -21,10 +21,6 @@
 #include <C4Application.h>
 #include <C4Log.h>
 
-#if AUDIO_TK == AUDIO_TK_FMOD
-#include <fmod_errors.h>
-#endif
-
 #if AUDIO_TK == AUDIO_TK_OPENAL
 #if defined(__APPLE__)
 #import <CoreFoundation/CoreFoundation.h>
@@ -37,11 +33,17 @@
 #include <alut.h>
 #undef _XBOX
 #endif
-#define alErrorCheck(X) X; { ALenum err = alGetError(); if (err) LogF("al error: %s (%x)", #X, err); }
+#define alErrorCheck(X) do { X; { ALenum err = alGetError(); if (err) LogF("al error: %s (%x)", #X, err); } } while (0)
 #endif
 
 /* helpers */
 
+void C4MusicFile::Announce()
+{
+	LogF(LoadResStr("IDS_PRC_PLAYMUSIC"), GetFilename(FileName));
+	announced = true;
+}
+
 bool C4MusicFile::ExtractFile()
 {
 	// safety
@@ -68,246 +70,7 @@ bool C4MusicFile::Init(const char *szFile)
 	return true;
 }
 
-#if AUDIO_TK == AUDIO_TK_FMOD
-bool C4MusicFileMID::Play(bool loop)
-{
-	// check existance
-	if (!FileExists(FileName))
-		// try extracting it
-		if (!ExtractFile())
-			// doesn't exist - or file is corrupt
-			return false;
-
-	// init fmusic
-	mod = FMUSIC_LoadSong(SongExtracted ? Config.AtTempPath(C4CFN_TempMusic2) : FileName);
-
-	if (!mod)
-	{
-		LogF("FMod: %s", FMOD_ErrorString(FSOUND_GetError()));
-		return false;
-	}
-
-	// Play Song
-	FMUSIC_PlaySong(mod);
-
-	return true;
-}
-
-void C4MusicFileMID::Stop(int fadeout_ms)
-{
-	if (mod)
-	{
-		FMUSIC_StopSong(mod);
-		FMUSIC_FreeSong(mod);
-		mod = NULL;
-	}
-	RemTempFile();
-}
-
-void C4MusicFileMID::CheckIfPlaying()
-{
-	if (FMUSIC_IsFinished(mod))
-		Application.MusicSystem.NotifySuccess();
-}
-
-void C4MusicFileMID::SetVolume(int iLevel)
-{
-	FMUSIC_SetMasterVolume(mod, Clamp((iLevel * 256) / 100, 0, 255));
-}
-
-/* MOD */
-
-C4MusicFileMOD::C4MusicFileMOD()
-		: mod(NULL), Data(NULL)
-{
-
-}
-
-C4MusicFileMOD::~C4MusicFileMOD()
-{
-	Stop();
-}
-
-bool C4MusicFileMOD::Play(bool loop)
-{
-	// Load Song
-	size_t iFileSize;
-	if (!C4Group_ReadFile(FileName, &Data, &iFileSize))
-		return false;
-
-	// init fmusic
-	mod = FMUSIC_LoadSongEx(Data, 0, iFileSize, FSOUND_LOADMEMORY, 0, 0);
-
-	if (!mod)
-	{
-		LogF("FMod: %s", FMOD_ErrorString(FSOUND_GetError()));
-		return false;
-	}
-
-	// Play Song
-	FMUSIC_PlaySong(mod);
-
-	return true;
-}
-
-void C4MusicFileMOD::Stop(int fadeout_ms)
-{
-	if (mod)
-	{
-		FMUSIC_StopSong(mod);
-		FMUSIC_FreeSong(mod);
-		mod = NULL;
-	}
-	if (Data) { delete[] Data; Data = NULL; }
-}
-
-void C4MusicFileMOD::CheckIfPlaying()
-{
-	if (FMUSIC_IsFinished(mod))
-		Application.MusicSystem.NotifySuccess();
-}
-
-void C4MusicFileMOD::SetVolume(int iLevel)
-{
-	FMUSIC_SetMasterVolume(mod, (int) ((iLevel * 255) / 100));
-}
-
-/* MP3 */
-
-C4MusicFileMP3::C4MusicFileMP3()
-		: stream(NULL), Data(NULL), Channel(-1)
-{
-
-}
-
-C4MusicFileMP3::~C4MusicFileMP3()
-{
-	Stop();
-}
-
-bool C4MusicFileMP3::Play(bool loop)
-{
-#ifndef USE_MP3
-	return false;
-#endif
-
-	// Load Song
-	size_t iFileSize;
-	if (!C4Group_ReadFile(FileName, &Data, &iFileSize))
-		return false;
-
-	// init fsound
-	int loop_flag = loop ? FSOUND_LOOP_NORMAL : 0;
-	stream = FSOUND_Stream_Open(Data, FSOUND_LOADMEMORY | FSOUND_NORMAL | FSOUND_2D | loop_flag, 0, iFileSize);
-
-	if (!stream) return false;
-
-	// Play Song
-	Channel = FSOUND_Stream_Play(FSOUND_FREE, stream);
-	if (Channel == -1) return false;
-
-	// Set highest priority
-	if (!FSOUND_SetPriority(Channel, 255))
-		return false;
-
-	return true;
-}
-
-void C4MusicFileMP3::Stop(int fadeout_ms)
-{
-	if (stream)
-	{
-		FSOUND_Stream_Close(stream);
-		stream = NULL;
-	}
-	if (Data) { delete[] Data; Data = NULL; }
-}
-
-void C4MusicFileMP3::CheckIfPlaying()
-{
-	if (FSOUND_Stream_GetPosition(stream) >= (unsigned) FSOUND_Stream_GetLength(stream))
-		Application.MusicSystem.NotifySuccess();
-}
-
-void C4MusicFileMP3::SetVolume(int iLevel)
-{
-	FSOUND_SetVolume(Channel, (int) ((iLevel * 255) / 100));
-}
-
-/* Ogg Vobis */
-
-C4MusicFileOgg::C4MusicFileOgg()
-		: stream(NULL), Data(NULL), Channel(-1), Playing(false)
-{
-
-}
-
-C4MusicFileOgg::~C4MusicFileOgg()
-{
-	Stop();
-}
-
-bool C4MusicFileOgg::Play(bool loop)
-{
-	// Load Song
-	size_t iFileSize;
-	if (!C4Group_ReadFile(FileName, &Data, &iFileSize))
-		return false;
-
-	// init fsound
-	int loop_flag = loop ? FSOUND_LOOP_NORMAL : 0;
-	stream = FSOUND_Stream_Open(Data, FSOUND_LOADMEMORY | FSOUND_NORMAL | FSOUND_2D | loop_flag, 0, iFileSize);
-
-	if (!stream) return false;
-
-	// Play Song
-	Channel = FSOUND_Stream_Play(FSOUND_FREE, stream);
-	if (Channel == -1) return false;
-
-	// Set highest priority
-	if (!FSOUND_SetPriority(Channel, 255))
-		return false;
-
-	Playing = true;
-
-	FSOUND_Stream_SetEndCallback(stream, &C4MusicFileOgg::OnEnd, this);
-
-	return true;
-}
-
-// End Callback
-signed char __stdcall C4MusicFileOgg::OnEnd(FSOUND_STREAM* stream, void* buff, int length, void *param)
-{
-	C4MusicFileOgg* pFile = static_cast<C4MusicFileOgg*>(param);
-	pFile->Playing = false;
-	return 0;
-}
-
-void C4MusicFileOgg::Stop(int fadeout_ms)
-{
-	if (stream)
-	{
-		FSOUND_Stream_Close(stream);
-		stream = NULL;
-	}
-	if (Data) { delete[] Data; Data = NULL; }
-	Playing = false;
-}
-
-void C4MusicFileOgg::CheckIfPlaying()
-{
-
-	if (!Playing)
-		//if(FSOUND_Stream_GetPosition(stream) >= (unsigned) FSOUND_Stream_GetLength(stream))
-		Application.MusicSystem.NotifySuccess();
-}
-
-void C4MusicFileOgg::SetVolume(int iLevel)
-{
-	FSOUND_SetVolume(Channel, (int) ((iLevel * 255) / 100));
-}
-
-#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
+#if AUDIO_TK == AUDIO_TK_SDL_MIXER
 C4MusicFileSDL::C4MusicFileSDL():
 		Data(NULL),
 		Music(NULL)
@@ -319,7 +82,7 @@ C4MusicFileSDL::~C4MusicFileSDL()
 	Stop();
 }
 
-bool C4MusicFileSDL::Play(bool loop)
+bool C4MusicFileSDL::Play(bool loop, double max_resume_time)
 {
 	const SDL_version * link_version = Mix_Linked_Version();
 	if (link_version->major < 1
@@ -411,7 +174,8 @@ void C4MusicFileSDL::SetVolume(int iLevel)
 /* Ogg Vobis */
 
 C4MusicFileOgg::C4MusicFileOgg() :
-	playing(false), streaming_done(false), loaded(false), channel(0), current_section(0), byte_pos_total(0), volume(1.0f)
+	playing(false), streaming_done(false), loaded(false), channel(0), current_section(0), byte_pos_total(0), volume(1.0f),
+	is_loading_from_file(false), last_source_file_pos(0), last_playback_pos_sec(0), last_interruption_time()
 {
 	for (size_t i=0; i<num_buffers; ++i)
 		buffers[i] = 0;
@@ -432,6 +196,11 @@ void C4MusicFileOgg::Clear()
 		loaded = false;
 	}
 	categories.clear();
+	is_loading_from_file = false;
+	source_file.Close();
+	last_source_file_pos = 0;
+	last_playback_pos_sec = 0;
+	last_interruption_time = C4TimeMilliseconds();
 }
 
 bool C4MusicFileOgg::Init(const char *strFile)
@@ -440,28 +209,46 @@ bool C4MusicFileOgg::Init(const char *strFile)
 	Clear();
 	// Base init file
 	if (!C4MusicFile::Init(strFile)) return false;
-	// Initial file loading
-	// Currently, the whole compressed file is kept in memory because reading/seeking inside C4Group is problematic. Uncompress while playing.
-	// This uses about 50MB of RAM (for ala's music pack) and increases startup time a bit.
-	// Later, this could be replaced with proper random access in c4group. Either replacing the file format or e.g. storing the current zlib state here
-	//  and then updating callbacks.read/seek/close/tell_func to read data from the group directly as needed
-	char *file_contents;
-	size_t file_size;
-	if (!C4Group_ReadFile(FileName, &file_contents, &file_size))
-		return false;
-	data.SetOwnedData((BYTE *)file_contents, file_size);
-
 	// Prepare ogg reader
 	vorbis_info* info;
 	memset(&ogg_file, 0, sizeof(ogg_file));
 	ov_callbacks callbacks;
-	callbacks.read_func = &::C4SoundLoaders::VorbisLoader::read_func;
-	callbacks.seek_func = &::C4SoundLoaders::VorbisLoader::seek_func;
-	callbacks.close_func = &::C4SoundLoaders::VorbisLoader::close_func;
-	callbacks.tell_func = &::C4SoundLoaders::VorbisLoader::tell_func;
+	// Initial file loading
+	// For packed groups, the whole compressed file is kept in memory because reading/seeking inside C4Group is problematic. Uncompress while playing.
+	// This increases startup time a bit.
+	// Later, this could be replaced with proper random access in c4group. Either replacing the file format or e.g. storing the current zlib state here
+	//  and then updating callbacks.read/seek/close/tell_func to read data from the group directly as needed
+	bool is_loading_from_file = FileExists(strFile);
+	void *data_source;
+	if (!is_loading_from_file)
+	{
+		char *file_contents;
+		size_t file_size;
+		if (!C4Group_ReadFile(FileName, &file_contents, &file_size))
+			return false;
+		data.SetOwnedData((BYTE *)file_contents, file_size);
+		// C4Group preloaded ogg reader
+		callbacks.read_func = &::C4SoundLoaders::VorbisLoader::mem_read_func;
+		callbacks.seek_func = &::C4SoundLoaders::VorbisLoader::mem_seek_func;
+		callbacks.close_func = &::C4SoundLoaders::VorbisLoader::mem_close_func;
+		callbacks.tell_func = &::C4SoundLoaders::VorbisLoader::mem_tell_func;
+		data_source = &data;
+	}
+	else
+	{
+		// Load directly from file
+		if (!source_file.Open(FileName))
+			return false;
+		// Uncompressed file ogg reader
+		callbacks.read_func = &::C4SoundLoaders::VorbisLoader::file_read_func;
+		callbacks.seek_func = &::C4SoundLoaders::VorbisLoader::file_seek_func;
+		callbacks.close_func = &::C4SoundLoaders::VorbisLoader::file_close_func;
+		callbacks.tell_func = &::C4SoundLoaders::VorbisLoader::file_tell_func;
+		data_source = this;
+	}
 
-	// open using callbacks
-	if (ov_open_callbacks(&data, &ogg_file, NULL, 0, callbacks) != 0)
+	// open using callbacks either to memory or to file loader
+	if (ov_open_callbacks(data_source, &ogg_file, NULL, 0, callbacks) != 0)
 	{
 		ov_clear(&ogg_file);
 		return false;
@@ -501,16 +288,68 @@ bool C4MusicFileOgg::Init(const char *strFile)
 		}
 	}
 
+	// File not needed for now
+	UnprepareSourceFileReading();
+
 	// mark successfully loaded
 	return loaded = true;
 }
 
-bool C4MusicFileOgg::Play(bool loop)
+StdStrBuf C4MusicFileOgg::GetDebugInfo() const
+{
+	StdStrBuf result;
+	result.Append(FileName);
+	result.AppendFormat("[%.0lf]", last_playback_pos_sec);
+	result.AppendChar('[');
+	bool sec = false;
+	for (auto i = categories.cbegin(); i != categories.cend(); ++i)
+	{
+		if (sec) result.AppendChar(',');
+		result.Append(i->getData());
+		sec = true;
+	}
+	result.AppendChar(']');
+	return result;
+}
+
+void C4MusicFileOgg::UnprepareSourceFileReading()
+{
+	// The file loader could just keep all files open. But if someone symlinks
+	// Music.ocg into their music folder with a million files in it, we would
+	// crash with too many open file handles. So close it for now and reopen
+	// when that piece is actually requested.
+	if (is_loading_from_file && source_file.IsOpen())
+	{
+		last_source_file_pos = source_file.Tell();
+		source_file.Close();
+	}
+}
+
+bool C4MusicFileOgg::PrepareSourceFileReading()
+{
+	// mem loading always OK
+	if (!is_loading_from_file) return true;
+	// ensure file is open
+	if (!source_file.IsOpen())
+	{
+		if (!source_file.Open(FileName)) return false;
+		if (last_source_file_pos) if (source_file.Seek(last_source_file_pos, SEEK_SET) < 0) return false;
+	}
+	return true;
+}
+
+bool C4MusicFileOgg::Play(bool loop, double max_resume_time)
 {
 	// Valid file?
 	if (!loaded) return false;
 	// stop previous
-	Stop();
+	if (playing)
+	{
+		if (max_resume_time > 0.0) return true; // no-op
+		Stop();
+	}
+	// Ensure data reading is ready
+	PrepareSourceFileReading();
 	// Get channel to use
 	alGenSources(1, (ALuint*)&channel);
 	if (!channel) return false;
@@ -520,13 +359,25 @@ bool C4MusicFileOgg::Play(bool loop)
 	this->loop = loop;
 	byte_pos_total = 0;
 
+	// Resume setting
+	if (max_resume_time > 0)
+	{
+		// Only resume if significant amount of data is left to be played
+		double time_remaining_sec = GetRemainingTime();
+		if (time_remaining_sec < max_resume_time) last_playback_pos_sec = 0.0;
+	}
+	else
+	{
+		last_playback_pos_sec = 0;
+	}
+
 	// initial volume setting
 	SetVolume(float(::Config.Sound.MusicVolume) / 100.0f);
 
 	// prepare read
 	ogg_info.sound_data.resize(num_buffers * buffer_size);
 	alGenBuffers(num_buffers, buffers);
-	ov_pcm_seek(&ogg_file, 0);
+	ov_time_seek(&ogg_file, last_playback_pos_sec);
 
 	// Fill initial buffers
 	for (size_t i=0; i<num_buffers; ++i)
@@ -538,11 +389,22 @@ bool C4MusicFileOgg::Play(bool loop)
 	return true;
 }
 
+double C4MusicFileOgg::GetRemainingTime()
+{
+	// Note: Only valid after piece has been stopped
+	return ov_time_total(&ogg_file, -1) - last_playback_pos_sec;
+}
 
 void C4MusicFileOgg::Stop(int fadeout_ms)
 {
 	if (playing)
 	{
+		// remember position for eventual later resume
+		ALfloat playback_pos_in_buffer = 0;
+		alErrorCheck(alGetSourcef(channel, AL_SEC_OFFSET, &playback_pos_in_buffer));
+		last_playback_pos_sec += playback_pos_in_buffer;
+		last_interruption_time = C4TimeMilliseconds::Now();
+		// stop!
 		alSourceStop(channel);
 		// clear queue
 		ALint num_queued=0;
@@ -560,6 +422,8 @@ void C4MusicFileOgg::Stop(int fadeout_ms)
 	}
 	playing = false;
 	channel = 0;
+	// close file
+	UnprepareSourceFileReading();
 }
 
 void C4MusicFileOgg::CheckIfPlaying()
@@ -641,9 +505,17 @@ void C4MusicFileOgg::Execute()
 		bool done = false;
 		while (num_processed--)
 		{
-			// refill processed buffers
-			ALuint buffer;
+			// release processed buffer
+			ALuint buffer; 
 			alErrorCheck(alSourceUnqueueBuffers(channel, 1, &buffer));
+			// add playback time of processed buffer to total playback time
+			ALint buf_bits = 16, buf_chans = 2, buf_freq = 44100;
+			alErrorCheck(alGetBufferi(buffer, AL_BITS, &buf_bits));
+			alErrorCheck(alGetBufferi(buffer, AL_CHANNELS, &buf_chans));
+			alErrorCheck(alGetBufferi(buffer, AL_FREQUENCY, &buf_freq));
+			double buffer_secs = double(buffer_size) / buf_bits / buf_chans / buf_freq * 8;
+			last_playback_pos_sec += buffer_secs;
+			// refill processed buffer
 			size_t buffer_idx;
 			for (buffer_idx=0; buffer_idx<num_buffers; ++buffer_idx)
 				if (buffers[buffer_idx] == buffer) break;
@@ -656,6 +528,8 @@ void C4MusicFileOgg::Execute()
 		if (state != AL_PLAYING && streaming_done)
 		{
 			Stop();
+			// reset playback to beginning for next time this piece is playing
+			last_playback_pos_sec = 0.0;
 		}
 		else if (state == AL_STOPPED)
 		{
