@@ -28,7 +28,7 @@ class C4MusicFile
 public:
 
 	C4MusicFile() :
-		pNext(NULL), LastPlayed(-1), NoPlay(false), loop(false), SongExtracted(false)
+		pNext(NULL), LastPlayed(-1), NoPlay(false), loop(false), SongExtracted(false), announced(false)
 	{ }
 	virtual ~C4MusicFile() { }
 
@@ -38,16 +38,26 @@ public:
 	int LastPlayed;
 	bool NoPlay;
 	bool loop;
+	bool announced;
 
 	virtual bool Init(const char *strFile);
-	virtual bool Play(bool loop = false) = 0;
+	virtual bool Play(bool loop = false, double max_resume_time = 0.0) = 0;
 	virtual void Stop(int fadeout_ms = 0) = 0;
 	virtual void CheckIfPlaying() = 0;
 	virtual void SetVolume(int) = 0;
 	virtual bool HasCategory(const char *szcat) const { return false; }
+	virtual double GetRemainingTime() { return 0.0; }
+	virtual bool HasResumePos() const { return false; }
+	virtual void ClearResumePos() { }
+	virtual C4TimeMilliseconds GetLastInterruptionTime() const { return C4TimeMilliseconds(); }
+
+	virtual StdStrBuf GetDebugInfo() const { return StdStrBuf(FileName); }
 
 	bool IsLooping() const { return loop; }
 
+	bool HasBeenAnnounced() const { return announced; }
+	void Announce();
+
 protected:
 
 	// helper: copy data to a (temp) file
@@ -57,82 +67,15 @@ protected:
 	bool SongExtracted;
 
 };
-#if AUDIO_TK == AUDIO_TK_FMOD
-class C4MusicFileMID : public C4MusicFile
-{
-public:
-	bool Play(bool loop = false);
-	bool Extract();
-	void Stop(int fadeout_ms = 0);
-	void CheckIfPlaying();
-	//C4MusicFileMID();
-	void SetVolume(int);
-protected:
-	FMUSIC_MODULE *mod;
-};
 
-/* MOD class */
-
-class C4MusicFileMOD : public C4MusicFile
-{
-public:
-	C4MusicFileMOD();
-	~C4MusicFileMOD();
-	bool Play(bool loop = false);
-	void Stop(int fadeout_ms = 0);
-	void CheckIfPlaying();
-	void SetVolume(int);
-protected:
-	FMUSIC_MODULE *mod;
-	char *Data;
-};
-
-/* MP3 class */
-
-class C4MusicFileMP3 : public C4MusicFile
-{
-public:
-	C4MusicFileMP3();
-	~C4MusicFileMP3();
-	bool Play(bool loop = false);
-	void Stop(int fadeout_ms = 0);
-	void CheckIfPlaying();
-	void SetVolume(int);
-protected:
-	FSOUND_STREAM *stream;
-	char *Data;
-	int Channel;
-};
-
-/* Ogg class */
-
-class C4MusicFileOgg : public C4MusicFile
-{
-public:
-	C4MusicFileOgg();
-	~C4MusicFileOgg();
-	bool Play(bool loop = false);
-	void Stop(int fadeout_ms = 0);
-	void CheckIfPlaying();
-	void SetVolume(int);
-
-	static signed char __stdcall OnEnd(FSOUND_STREAM* stream, void* buff, int length, void* param);
-protected:
-	FSOUND_STREAM *stream;
-	char *Data;
-	int Channel;
-
-	bool Playing;
-};
-
-#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
+#if AUDIO_TK == AUDIO_TK_SDL_MIXER
 typedef struct _Mix_Music Mix_Music;
 class C4MusicFileSDL : public C4MusicFile
 {
 public:
 	C4MusicFileSDL();
 	~C4MusicFileSDL();
-	bool Play(bool loop = false);
+	bool Play(bool loop = false, double max_resume_time = 0.0);
 	void Stop(int fadeout_ms = 0);
 	void CheckIfPlaying();
 	void SetVolume(int);
@@ -150,19 +93,29 @@ public:
 	~C4MusicFileOgg();
 	void Clear();
 	virtual bool Init(const char *strFile);
-	bool Play(bool loop = false);
+	bool Play(bool loop = false, double max_resume_time = 0.0);
 	void Stop(int fadeout_ms = 0);
 	void CheckIfPlaying();
 	void SetVolume(int);
 	virtual bool HasCategory(const char *szcat) const;
+	double GetRemainingTime();
+	bool HasResumePos() const { return (last_playback_pos_sec > 0);  }
+	void ClearResumePos() { last_playback_pos_sec = 0.0; last_interruption_time = C4TimeMilliseconds(); }
+	C4TimeMilliseconds GetLastInterruptionTime() const { return last_interruption_time; }
+	virtual StdStrBuf GetDebugInfo() const;
 private:
 	enum { num_buffers = 4, buffer_size = 160*1024 };
 	::C4SoundLoaders::VorbisLoader::CompressedData data;
 	::C4SoundLoaders::SoundInfo ogg_info;
 	OggVorbis_File ogg_file;
+	bool is_loading_from_file;
+	CStdFile source_file;
+	long last_source_file_pos; // remember position in source_file because it may be closed and re-opened
 	bool playing, streaming_done, loaded;
 	ALuint buffers[num_buffers];
 	ALuint channel;
+	double last_playback_pos_sec; // last playback position for resume when fading between pieces
+	C4TimeMilliseconds last_interruption_time; // set to nonzero when song is interrupted
 	int current_section;
 	size_t byte_pos_total;
 	float volume;
@@ -170,6 +123,10 @@ private:
 
 	bool FillBuffer(size_t idx);
 	void Execute(); // fill processed buffers
+	void UnprepareSourceFileReading(); // close file handle but remember buffer position for re-opening
+	bool PrepareSourceFileReading(); // close file handle but remember buffer position for re-opening
+
+	friend class C4SoundLoaders::VorbisLoader;
 };
 
 #endif
