@@ -16,6 +16,7 @@
 
 #include <C4Include.h>
 #include "C4SoundLoaders.h"
+#include <C4MusicFile.h>
 
 #include <C4Application.h>
 
@@ -103,7 +104,7 @@ AppleSoundLoader AppleSoundLoader::singleton;
 #endif
 
 #if AUDIO_TK == AUDIO_TK_OPENAL
-size_t VorbisLoader::read_func(void* ptr, size_t byte_size, size_t size_to_read, void* datasource)
+size_t VorbisLoader::mem_read_func(void* ptr, size_t byte_size, size_t size_to_read, void* datasource)
 {
 	size_t spaceToEOF;
 	size_t actualSizeToRead;
@@ -124,7 +125,7 @@ size_t VorbisLoader::read_func(void* ptr, size_t byte_size, size_t size_to_read,
 	return actualSizeToRead;
 }
 
-int VorbisLoader::seek_func(void* datasource, ogg_int64_t offset, int whence)
+int VorbisLoader::mem_seek_func(void* datasource, ogg_int64_t offset, int whence)
 {
 	CompressedData* data = (CompressedData*)datasource;
 	
@@ -143,16 +144,44 @@ int VorbisLoader::seek_func(void* datasource, ogg_int64_t offset, int whence)
 	return 0;
 }
 
-int VorbisLoader::close_func(void* datasource)
+int VorbisLoader::mem_close_func(void* datasource)
 {
 	return 1;
 }
 
-long VorbisLoader::tell_func(void* datasource)
+long VorbisLoader::mem_tell_func(void* datasource)
 {
 	return ((CompressedData*)datasource)->data_pos;
 }
 
+size_t VorbisLoader::file_read_func(void* ptr, size_t byte_size, size_t size_to_read, void* datasource)
+{
+	C4MusicFileOgg *ogg = static_cast<C4MusicFileOgg *>(datasource);
+	size_t bytes_to_read = size_to_read*byte_size;
+	size_t bytes_read = 0u;
+	ogg->source_file.Read(ptr, bytes_to_read, &bytes_read);
+	return bytes_read / byte_size;
+}
+
+int VorbisLoader::file_seek_func(void* datasource, ogg_int64_t offset, int whence)
+{
+	C4MusicFileOgg *ogg = static_cast<C4MusicFileOgg *>(datasource);
+	return ogg->source_file.Seek(static_cast<long int>(offset), whence);
+}
+
+int VorbisLoader::file_close_func(void* datasource)
+{
+	C4MusicFileOgg *ogg = static_cast<C4MusicFileOgg *>(datasource);
+	ogg->source_file.Close();
+	return 1;
+}
+
+long VorbisLoader::file_tell_func(void* datasource)
+{
+	C4MusicFileOgg *ogg = static_cast<C4MusicFileOgg *>(datasource);
+	return ogg->source_file.Tell();
+}
+
 bool VorbisLoader::ReadInfo(SoundInfo* result, BYTE* data, size_t data_length, uint32_t)
 {
 	CompressedData compressed(data, data_length);
@@ -163,10 +192,10 @@ bool VorbisLoader::ReadInfo(SoundInfo* result, BYTE* data, size_t data_length, u
 	OggVorbis_File ogg_file;
 	memset(&ogg_file, 0, sizeof(ogg_file));
 	ov_callbacks callbacks;
-	callbacks.read_func  = &read_func;
-	callbacks.seek_func  = &seek_func;
-	callbacks.close_func = &close_func;
-	callbacks.tell_func  = &tell_func;
+	callbacks.read_func  = &mem_read_func;
+	callbacks.seek_func = &mem_seek_func;
+	callbacks.close_func = &mem_close_func;
+	callbacks.tell_func = &mem_tell_func;
 	
 	// open using callbacks
 	if (ov_open_callbacks(&compressed, &ogg_file, NULL, 0, callbacks) != 0)
@@ -184,11 +213,16 @@ bool VorbisLoader::ReadInfo(SoundInfo* result, BYTE* data, size_t data_length, u
 	result->sample_rate = info->rate;
 	result->sample_length = ov_time_total(&ogg_file, -1)/1000.0;
 
-	// read
+	// Compute the total buffer size
+	const unsigned long total_size = static_cast<unsigned int>(result->sample_rate * result->sample_length * 1000.0 * info->channels * 2 + 0.5);
+	const unsigned long extra_size = 1024 * 8;
+
+	// read: try to read the whole track in one go, and if there is more data
+	// than we predicted in total_size, read the additional data in 8K blocks.
 	unsigned long buffer_size = 0;
 	long bytes_read;
 	do {
-		const int chunk_size = 1024*8;
+		const int chunk_size = total_size + extra_size - std::min(total_size, buffer_size);
 		int bitStream;
 		if (buffer_size+chunk_size > result->sound_data.size())
 			result->sound_data.resize(buffer_size+chunk_size);
@@ -212,8 +246,6 @@ bool WavLoader::ReadInfo(SoundInfo* result, BYTE* data, size_t data_length, uint
 	ALuint wav = alutCreateBufferFromFileImage((const ALvoid *)data, data_length);
 	if (wav == AL_NONE)
 	{
-		// wouldn't want an error on any .ogg file
-		//LogF("load wav error: %s", alutGetErrorString(alutGetError()));
 		return false;
 	}
 
@@ -268,29 +300,4 @@ bool SDLMixerSoundLoader::ReadInfo(SoundInfo* result, BYTE* data, size_t data_le
 
 SDLMixerSoundLoader SDLMixerSoundLoader::singleton;
 
-#elif AUDIO_TK == AUDIO_TK_FMOD
-bool FMODSoundLoader::ReadInfo(SoundInfo* result, BYTE* data, size_t data_length, uint32_t options)
-{
-	int32_t iOptions = FSOUND_NORMAL | FSOUND_2D | FSOUND_LOADMEMORY;
-	if (options & OPTION_Raw)
-		iOptions |= FSOUND_LOADRAW;
-	C4SoundHandle pSample;
-	if (!(pSample = FSOUND_Sample_Load(FSOUND_UNMANAGED, (const char *)data, iOptions, 0, data_length)))
-		{ return false; }
-	// get length
-	int32_t iSamples = FSOUND_Sample_GetLength(pSample);
-	int iSampleRate = 0;
-	if (!iSamples || !FSOUND_Sample_GetDefaults(pSample, &iSampleRate, 0, 0, 0))
-	{
-		FSOUND_Sample_Free(pSample);
-		return false;
-	}
-	result->sample_rate = iSampleRate;
-	result->sample_length = static_cast<double>(iSamples) / iSampleRate;
-	result->final_handle = pSample;
-	assert(result->sample_length > 0);
-	return true;
-}
-
-FMODSoundLoader FMODSoundLoader::singleton;
 #endif
