@@ -27,6 +27,7 @@
 #include <C4Config.h>
 
 #define DEBUG_BYTECODE_DUMP 0
+#include <iomanip>
 
 #define C4AUL_Include       "#include"
 #define C4AUL_Append        "#appendto"
@@ -182,7 +183,8 @@ private:
 	C4AulTokenType GetNextToken(OperatorPolicy Operator = OperatorsPlease); // get next token of SPos
 
 	void Shift(OperatorPolicy Operator = OperatorsPlease);
-	void Match(C4AulTokenType TokenType, const char * Message = NULL);
+	void Match(C4AulTokenType TokenType, const char * Expected = NULL);
+	void Check(C4AulTokenType TokenType, const char * Expected = NULL);
 	void UnexpectedToken(const char * Expected) NORETURN;
 	static const char * GetTokenName(C4AulTokenType TokenType);
 
@@ -259,7 +261,7 @@ void C4AulParse::Error(const char *pMsg, ...)
 	StdStrBuf Buf;
 	Buf.FormatV(pMsg, args);
 
-	throw new C4AulParseError(this, Buf.getData());
+	throw C4AulParseError(this, Buf.getData());
 }
 
 C4AulParseError::C4AulParseError(C4AulParse * state, const char *pMsg, const char *pIdtf, bool Warn)
@@ -357,7 +359,6 @@ struct C4ScriptOpDef
 	unsigned short Priority;
 	const char* Identifier;
 	C4AulBCCType Code;
-	C4AulBCCType ResultModifier; // code to apply to result after it was calculated
 	bool Postfix;
 	bool Changer; // changes first operand to result, rewrite to "a = a (op) b"
 	bool NoSecondStatement; // no second statement expected (++/-- postfix)
@@ -368,57 +369,55 @@ struct C4ScriptOpDef
 
 static C4ScriptOpDef C4ScriptOpMap[] =
 {
-	// priority                              postfix
-	// |  identifier                         |  changer
-	// |  |     Bytecode            Result   |  |  no second id
-	// |  |     |                   Modifier |  |  |  RetType   ParType1    ParType2
+	// priority                      postfix
+	// |  identifier                 |  changer
+	// |  |     Bytecode             |  |  no second id
+	// |  |     |                    |  |  |  RetType   ParType1    ParType2
 	// prefix
-	{ 15, "++", AB_Inc,             AB_ERR,  0, 1, 0, C4V_Int,  C4V_Int,    C4V_Any},
-	{ 15, "--", AB_Dec,             AB_ERR,  0, 1, 0, C4V_Int,  C4V_Int,    C4V_Any},
-	{ 15, "~",  AB_BitNot,          AB_ERR,  0, 0, 0, C4V_Int,  C4V_Int,    C4V_Any},
-	{ 15, "!",  AB_Not,             AB_ERR,  0, 0, 0, C4V_Bool, C4V_Bool,   C4V_Any},
-	{ 15, "+",  AB_ERR,             AB_ERR,  0, 0, 0, C4V_Int,  C4V_Int,    C4V_Any},
-	{ 15, "-",  AB_Neg,             AB_ERR,  0, 0, 0, C4V_Int,  C4V_Int,    C4V_Any},
+	{ 15, "++", AB_Inc,              0, 1, 0, C4V_Int,  C4V_Int,    C4V_Any},
+	{ 15, "--", AB_Dec,              0, 1, 0, C4V_Int,  C4V_Int,    C4V_Any},
+	{ 15, "~",  AB_BitNot,           0, 0, 0, C4V_Int,  C4V_Int,    C4V_Any},
+	{ 15, "!",  AB_Not,              0, 0, 0, C4V_Bool, C4V_Bool,   C4V_Any},
+	{ 15, "+",  AB_ERR,              0, 0, 0, C4V_Int,  C4V_Int,    C4V_Any},
+	{ 15, "-",  AB_Neg,              0, 0, 0, C4V_Int,  C4V_Int,    C4V_Any},
 	
 	// postfix (whithout second statement)
-	{ 16, "++", AB_Inc,             AB_Dec,  1, 1, 1, C4V_Int,  C4V_Int,    C4V_Any},
-	{ 16, "--", AB_Dec,             AB_Inc,  1, 1, 1, C4V_Int,  C4V_Int,    C4V_Any},
+	{ 16, "++", AB_Inc,              1, 1, 1, C4V_Int,  C4V_Int,    C4V_Any},
+	{ 16, "--", AB_Dec,              1, 1, 1, C4V_Int,  C4V_Int,    C4V_Any},
 	
 	// postfix
-	{ 14, "**", AB_Pow,             AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 13, "/",  AB_Div,             AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 13, "*",  AB_Mul,             AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 13, "%",  AB_Mod,             AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 12, "-",  AB_Sub,             AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 12, "+",  AB_Sum,             AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 11, "<<", AB_LeftShift,       AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 11, ">>", AB_RightShift,      AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 10, "<",  AB_LessThan,        AB_ERR,  1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
-	{ 10, "<=", AB_LessThanEqual,   AB_ERR,  1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
-	{ 10, ">",  AB_GreaterThan,     AB_ERR,  1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
-	{ 10, ">=", AB_GreaterThanEqual,AB_ERR,  1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
-	{ 9, "===", AB_Identical,       AB_ERR,  1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
-	{ 9, "!==", AB_NotIdentical,    AB_ERR,  1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
-	{ 9, "==",  AB_Equal,           AB_ERR,  1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
-	{ 9, "!=",  AB_NotEqual,        AB_ERR,  1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
-	{ 8, "&",   AB_BitAnd,          AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 6, "^",   AB_BitXOr,          AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 6, "|",   AB_BitOr,           AB_ERR,  1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 5, "&&",  AB_JUMPAND,         AB_ERR,  1, 0, 0, C4V_Bool, C4V_Bool,   C4V_Bool},
-	{ 4, "||",  AB_JUMPOR,          AB_ERR,  1, 0, 0, C4V_Bool, C4V_Bool,   C4V_Bool},
-	{ 3, "??",  AB_JUMPNNIL,        AB_ERR,  1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
+	{ 14, "**", AB_Pow,              1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 13, "/",  AB_Div,              1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 13, "*",  AB_Mul,              1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 13, "%",  AB_Mod,              1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 12, "-",  AB_Sub,              1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 12, "+",  AB_Sum,              1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 11, "<<", AB_LeftShift,        1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 11, ">>", AB_RightShift,       1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 10, "<",  AB_LessThan,         1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
+	{ 10, "<=", AB_LessThanEqual,    1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
+	{ 10, ">",  AB_GreaterThan,      1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
+	{ 10, ">=", AB_GreaterThanEqual, 1, 0, 0, C4V_Bool, C4V_Int,    C4V_Int},
+	{ 9, "==",  AB_Equal,            1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
+	{ 9, "!=",  AB_NotEqual,         1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
+	{ 8, "&",   AB_BitAnd,           1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 6, "^",   AB_BitXOr,           1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 6, "|",   AB_BitOr,            1, 0, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 5, "&&",  AB_JUMPAND,          1, 0, 0, C4V_Bool, C4V_Bool,   C4V_Bool},
+	{ 4, "||",  AB_JUMPOR,           1, 0, 0, C4V_Bool, C4V_Bool,   C4V_Bool},
+	{ 3, "??",  AB_JUMPNNIL,         1, 0, 0, C4V_Bool, C4V_Any,    C4V_Any},
 	
 	// changers
-	{ 2, "*=",  AB_Mul,             AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 2, "/=",  AB_Div,             AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 2, "%=",  AB_Mod,             AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 2, "+=",  AB_Sum,             AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 2, "-=",  AB_Sub,             AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 2, "&=",  AB_BitAnd,          AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 2, "|=",  AB_BitOr,           AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
-	{ 2, "^=",  AB_BitXOr,          AB_ERR,  1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "*=",  AB_Mul,              1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "/=",  AB_Div,              1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "%=",  AB_Mod,              1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "+=",  AB_Sum,              1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "-=",  AB_Sub,              1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "&=",  AB_BitAnd,           1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "|=",  AB_BitOr,            1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
+	{ 2, "^=",  AB_BitXOr,           1, 1, 0, C4V_Int,  C4V_Int,    C4V_Int},
  	
-	{ 0, NULL,  AB_ERR,             AB_ERR,  0, 0, 0, C4V_Nil,  C4V_Nil,    C4V_Nil}
+	{ 0, NULL,  AB_ERR,               0, 0, 0, C4V_Nil,  C4V_Nil,    C4V_Nil}
 };
 
 int C4AulParse::GetOperator(const char* pScript)
@@ -489,7 +488,7 @@ C4AulTokenType C4AulParse::GetNextToken(OperatorPolicy Operator)
 			C = *(++SPos);
 		}
 
-		Len = Min(Len, C4AUL_MAX_Identifier);
+		Len = std::min(Len, C4AUL_MAX_Identifier);
 		SCopy(TokenSPos, Idtf, Len);
 		return dir ? ATT_DIR : ATT_IDTF;
 	}
@@ -593,7 +592,7 @@ C4AulTokenType C4AulParse::GetNextToken(OperatorPolicy Operator)
 				}
 				}
 			else if (C == 0 || C == 10 || C == 13) // line break / feed
-				throw new C4AulParseError(this, "string not closed");
+				throw C4AulParseError(this, "string not closed");
 			else
 				// copy character
 				strbuf.push_back(C);
@@ -614,9 +613,9 @@ C4AulTokenType C4AulParse::GetNextToken(OperatorPolicy Operator)
 	{
 		// show appropriate error message
 		if (C >= '!' && C <= '~')
-			throw new C4AulParseError(this, FormatString("unexpected character '%c' found", C).getData());
+			throw C4AulParseError(this, FormatString("unexpected character '%c' found", C).getData());
 		else
-			throw new C4AulParseError(this, FormatString("unexpected character 0x%x found", (int)(unsigned char) C).getData());
+			throw C4AulParseError(this, FormatString("unexpected character 0x%x found", (int)(unsigned char) C).getData());
 	}
 }
 
@@ -662,8 +661,6 @@ static const char * GetTTName(C4AulBCCType e)
 	case AB_LessThanEqual: return "LessThanEqual";  // <=
 	case AB_GreaterThan: return "GreaterThan";  // >
 	case AB_GreaterThanEqual: return "GreaterThanEqual";  // >=
-	case AB_Identical: return "Identical";  // ===
-	case AB_NotIdentical: return "NotIdentical";  // !==
 	case AB_Equal: return "Equal";  // ==
 	case AB_NotEqual: return "NotEqual";  // !=
 	case AB_BitAnd: return "BitAnd";  // &
@@ -715,6 +712,9 @@ void C4AulScriptFunc::AddBCC(C4AulBCCType eType, intptr_t X, const char * SPos)
 	/* case AB_LOCALN_SET/AB_PROP_SET: -- expected to already have a reference upon creation, see MakeSetter */
 		bcc.Par.s->IncRef();
 		break;
+	case AB_CARRAY:
+		bcc.Par.a->IncRef();
+		break;
 	default: break;
 	}
 }
@@ -727,6 +727,9 @@ void C4AulScriptFunc::RemoveLastBCC()
 	case AB_STRING: case AB_CALL: case AB_CALLFS: case AB_LOCALN: case AB_LOCALN_SET: case AB_PROP: case AB_PROP_SET:
 		pBCC->Par.s->DecRef();
 		break;
+	case AB_CARRAY:
+		pBCC->Par.a->DecRef();
+		break;
 	default: break;
 	}
 	Code.pop_back();
@@ -812,8 +815,6 @@ int C4AulParse::GetStackValue(C4AulBCCType eType, intptr_t X)
 	case AB_LessThanEqual:
 	case AB_GreaterThan:
 	case AB_GreaterThanEqual:
-	case AB_Identical:
-	case AB_NotIdentical:
 	case AB_Equal:
 	case AB_NotEqual:
 	case AB_BitAnd:
@@ -925,7 +926,7 @@ int C4AulParse::AddBCC(C4AulBCCType eType, intptr_t X)
 		// Prune unneeded Incs / Decs
 		if(eType == AB_STACK && X < 0 && (pCPos1->bccType == AB_Inc || pCPos1->bccType == AB_Dec))
 		{
-			if(C4ScriptOpMap[pCPos1->Par.i].ResultModifier != pCPos1->bccType)
+			if(!pCPos1->Par.X)
 			{
 				pCPos1->bccType = eType;
 				pCPos1->Par.i = X;
@@ -1026,7 +1027,7 @@ C4V_Type C4AulParse::GetLastRetType(C4V_Type to)
 	case AB_LeftShift: case AB_RightShift: case AB_BitAnd: case AB_BitXOr: case AB_BitOr:
 		from = C4V_Int; break;
 	case AB_Not: case AB_LessThan: case AB_LessThanEqual: case AB_GreaterThan: case AB_GreaterThanEqual:
-	case AB_Identical: case AB_NotIdentical: case AB_Equal: case AB_NotEqual:
+	case AB_Equal: case AB_NotEqual:
 		from = C4V_Bool; break;
 	default:
 		from = C4V_Any; break;
@@ -1058,7 +1059,7 @@ C4AulBCC C4AulParse::MakeSetter(bool fLeaveValue)
 		break;
 	case AB_GLOBALN: Setter.bccType = AB_GLOBALN_SET; break;
 	default: 
-		throw new C4AulParseError(this, "assignment to a constant");
+		throw C4AulParseError(this, "assignment to a constant");
 	}
 	// Remove value BCC
 	RemoveLastBCC();
@@ -1187,17 +1188,19 @@ void C4AulParse::Shift(OperatorPolicy Operator)
 {
 	TokenType = GetNextToken(Operator);
 }
-void C4AulParse::Match(C4AulTokenType RefTokenType, const char * Message)
+void C4AulParse::Check(C4AulTokenType RefTokenType, const char * Expected)
 {
 	if (TokenType != RefTokenType)
-		// error
-		throw new C4AulParseError(this, Message ? Message :
-		                          FormatString("%s expected, but found %s", GetTokenName(RefTokenType), GetTokenName(TokenType)).getData());
+		UnexpectedToken(Expected ? Expected : GetTokenName(RefTokenType));
+}
+void C4AulParse::Match(C4AulTokenType RefTokenType, const char * Expected)
+{
+	Check(RefTokenType, Expected);
 	Shift();
 }
 void C4AulParse::UnexpectedToken(const char * Expected)
 {
-	throw new C4AulParseError(this, FormatString("%s expected, but found %s", Expected, GetTokenName(TokenType)).getData());
+	throw C4AulParseError(this, FormatString("%s expected, but found %s", Expected, GetTokenName(TokenType)).getData());
 }
 
 void C4AulScriptFunc::ParseFn(C4AulScriptContext* context)
@@ -1243,32 +1246,31 @@ void C4AulParse::Parse_Script(C4ScriptHost * scripthost)
 			{
 				Shift();
 				// get id of script to include
-				if (TokenType != ATT_IDTF)
-					UnexpectedToken("identifier");
+				Check(ATT_IDTF, "script name");
 				if (Type == PREPARSER)
 				{
 					// add to include list
-					Host->Includes.push_back(C4ID(StdStrBuf(Idtf)));
+					Host->Includes.push_back(StdCopyStrBuf(Idtf));
 				}
 				Shift();
 			}
 			else if (SEqual(Idtf, C4AUL_Append))
 			{
 				if (pOrgScript->GetPropList()->GetDef())
-					throw new C4AulParseError(this, "#appendto in a Definition");
+					throw C4AulParseError(this, "#appendto in a Definition");
 				// for #appendto * '*' needs to be ATT_STAR, not an operator.
 				Shift(StarsPlease);
 				if (Type == PREPARSER)
 				{
 					// get id of script to include/append
-					C4ID Id;
+					StdCopyStrBuf Id;
 					switch (TokenType)
 					{
 					case ATT_IDTF:
-						Id = C4ID(StdStrBuf(Idtf));
+						Id = StdCopyStrBuf(Idtf);
 						break;
 					case ATT_STAR: // "*"
-						Id = C4ID::None;
+						Id = StdCopyStrBuf("*");
 						break;
 					default:
 						// -> ID expected
@@ -1281,7 +1283,7 @@ void C4AulParse::Parse_Script(C4ScriptHost * scripthost)
 			}
 			else
 				// -> unknown directive
-				throw new C4AulParseError(this, "unknown directive: ", Idtf);
+				throw C4AulParseError(this, "unknown directive: ", Idtf);
 			break;
 		case ATT_IDTF:
 			// need a keyword here to avoid parsing random function contents
@@ -1311,18 +1313,17 @@ void C4AulParse::Parse_Script(C4ScriptHost * scripthost)
 		}
 		all_ok = true;
 	}
-	catch (C4AulError *err)
+	catch (C4AulError &err)
 	{
 		// damn! something went wrong, print it out
 		// but only one error per function
 		if (all_ok)
 		{
-			err->show();
+			err.show();
 			// and count (visible only ;) )
 			++::ScriptEngine.errCnt;
 		}
 		all_ok = false;
-		delete err;
 
 		if (Fn)
 		{
@@ -1342,41 +1343,40 @@ void C4AulParse::Parse_Script(C4ScriptHost * scripthost)
 
 void C4AulParse::Parse_Function()
 {
-	C4AulAccess Acc = AA_PUBLIC;
-	// Access?
-	if (SEqual(Idtf, C4AUL_Private)) { Acc = AA_PRIVATE; Shift(); }
-	else if (SEqual(Idtf, C4AUL_Protected)) { Acc = AA_PROTECTED; Shift(); }
-	else if (SEqual(Idtf, C4AUL_Public)) { Acc = AA_PUBLIC; Shift(); }
-	else if (SEqual(Idtf, C4AUL_Global)) { Acc = AA_GLOBAL; Shift(); }
+	bool is_global = SEqual(Idtf, C4AUL_Global);
+	// skip access modifier
+	if (SEqual(Idtf, C4AUL_Private) ||
+		SEqual(Idtf, C4AUL_Protected) ||
+		SEqual(Idtf, C4AUL_Public) ||
+		SEqual(Idtf, C4AUL_Global))
+	{
+		Shift();
+	}
+
 	// check for func declaration
 	if (!SEqual(Idtf, C4AUL_Func))
-		throw new C4AulParseError(this, "Declaration expected, but found identifier ", Idtf);
+		throw C4AulParseError(this, "Declaration expected, but found identifier ", Idtf);
 	Shift();
 	// get next token, must be func name
-	if (TokenType != ATT_IDTF)
-		UnexpectedToken("function name");
+	Check(ATT_IDTF, "function name");
 	// check: symbol already in use?
-	switch (Acc)
+	if (!is_global)
 	{
-	case AA_PRIVATE:
-	case AA_PROTECTED:
-	case AA_PUBLIC:
 		if (Host->LocalNamed.GetItemNr(Idtf) != -1)
-			throw new C4AulParseError(this, "function definition: name already in use (local variable)");
-		if (Host->GetPropList())
-			break;
-		// func in global context: fallthru
-	case AA_GLOBAL:
+			throw C4AulParseError(this, "function definition: name already in use (local variable)");
+	}
+	if (is_global || !Host->GetPropList())
+	{
 		if (Host != pOrgScript)
-			throw new C4AulParseError(this, "global func in appendto/included script: ", Idtf);
+			throw C4AulParseError(this, "global func in appendto/included script: ", Idtf);
 		if (Engine->GlobalNamedNames.GetItemNr(Idtf) != -1)
-			throw new C4AulParseError(this, "function definition: name already in use (global variable)");
+			throw C4AulParseError(this, "function definition: name already in use (global variable)");
 		if (Engine->GlobalConstNames.GetItemNr(Idtf) != -1)
 			Error("function definition: name already in use (global constant)");
 	}
 	// get script fn
 	C4AulScript * owner;
-	if (Acc == AA_GLOBAL)
+	if (is_global)
 		owner = Engine;
 	else
 		owner = Host;
@@ -1387,7 +1387,6 @@ void C4AulParse::Parse_Function()
 		if (f->SFunc() && f->SFunc()->pOrgScript == pOrgScript && f->Owner == owner)
 		{
 			if (Fn)
-				//throw new C4AulParseError(this, "Duplicate function ", Idtf);
 				Warn("Duplicate function %s", Idtf);
 			Fn = f->SFunc();
 		}
@@ -1412,25 +1411,26 @@ void C4AulParse::Parse_Function()
 	{
 		Fn->ClearCode();
 	}
+	Shift();
 
 	// Parse function body
-	Shift();
-	// expect an opening bracket now
-	if (TokenType != ATT_BOPEN)
-		UnexpectedToken("'('");
-	Shift();
+	Match(ATT_BOPEN);
 	// get pars
 	int cpar = 0;
 	while (TokenType != ATT_BCLOSE)
 	{
 		// too many parameters?
 		if (cpar >= C4AUL_MAX_Par)
-			throw new C4AulParseError(this, "'func' parameter list: too many parameters (max 10)");
-		// must be a name or type now
-		if (TokenType != ATT_IDTF)
+			throw C4AulParseError(this, "'func' parameter list: too many parameters (max 10)");
+		if (TokenType == ATT_LDOTS)
 		{
-			UnexpectedToken("parameter or closing bracket");
+			if (Type == PREPARSER) Fn->ParCount = C4AUL_MAX_Par;
+			Shift();
+			// don't allow any more parameters after ellipsis
+			break;
 		}
+		// must be a name or type now
+		Check(ATT_IDTF, "parameter, '...', or ')'");
 		// type identifier?
 		C4V_Type t = C4V_Any;
 		if (SEqual(Idtf, C4AUL_TypeInt)) { t = C4V_Int; Shift(); }
@@ -1465,9 +1465,7 @@ void C4AulParse::Parse_Function()
 			break;
 		}
 		// must be a comma now
-		if (TokenType != ATT_COMMA)
-			UnexpectedToken("comma or closing bracket");
-		Shift();
+		Match(ATT_COMMA, "',' or ')'");
 		cpar++;
 	}
 	Match(ATT_BCLOSE);
@@ -1520,7 +1518,40 @@ void C4AulParse::Parse_Function()
 			case AB_CALL: case AB_CALLFS: case AB_LOCALN: case AB_LOCALN_SET: case AB_PROP: case AB_PROP_SET:
 				fprintf(stderr, "\t%s\n", pBCC->Par.s->GetCStr()); break;
 			case AB_STRING:
-				fprintf(stderr, "\t\"%s\"\n", pBCC->Par.s->GetCStr()); break;
+			{
+				const StdStrBuf &s = pBCC->Par.s->GetData();
+				std::string es;
+				std::for_each(s.getData(), s.getData() + s.getLength(), [&es](char c) {
+					if (std::isgraph((unsigned char)c))
+					{
+						es += c;
+					}
+					else
+					{
+						switch (c)
+						{
+						case '\'': es.append("\\'"); break;
+						case '\"': es.append("\\\""); break;
+						case '\\': es.append("\\\\"); break;
+						case '\a': es.append("\\a"); break;
+						case '\b': es.append("\\b"); break;
+						case '\f': es.append("\\f"); break;
+						case '\n': es.append("\\n"); break;
+						case '\r': es.append("\\r"); break;
+						case '\t': es.append("\\t"); break;
+						case '\v': es.append("\\v"); break;
+						default:
+						{
+							std::stringstream hex;
+							hex << "\\x" << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>((unsigned char)c);
+							es.append(hex.str());
+							break;
+						}
+						}
+					}
+				});
+				fprintf(stderr, "\t\"%s\"\n", es.c_str()); break;
+			}
 			case AB_DEBUG: case AB_NIL: case AB_RETURN:
 			case AB_PAR: case AB_THIS:
 			case AB_ARRAYA: case AB_ARRAYA_SET: case AB_ARRAY_SLICE: case AB_ARRAY_SLICE_SET:
@@ -1596,7 +1627,7 @@ void C4AulParse::Parse_Statement()
 		         SEqual(Idtf, C4AUL_Public) ||
 		         SEqual(Idtf, C4AUL_Global))
 		{
-			throw new C4AulParseError(this, "unexpected end of function");
+			throw C4AulParseError(this, "unexpected end of function");
 		}
 		// get function by identifier: first check special functions
 		else if (SEqual(Idtf, C4AUL_If)) // if
@@ -1606,12 +1637,11 @@ void C4AulParse::Parse_Statement()
 		}
 		else if (SEqual(Idtf, C4AUL_Else)) // else
 		{
-			throw new C4AulParseError(this, "misplaced 'else'");
+			throw C4AulParseError(this, "misplaced 'else'");
 		}
 		else if (SEqual(Idtf, C4AUL_Do)) // while
 		{
 			Parse_DoWhile();
-			break;
 		}
 		else if (SEqual(Idtf, C4AUL_While)) // while
 		{
@@ -1629,7 +1659,7 @@ void C4AulParse::Parse_Statement()
 			if (TokenType == ATT_IDTF && SEqual(Idtf, C4AUL_VarNamed))
 				Shift();
 			// variable and "in"
-			if (TokenType == ATT_IDTF /*&& (iVarID = Fn->VarNamed.GetItemNr(Idtf)) != -1*/
+			if (TokenType == ATT_IDTF
 			    && GetNextToken() == ATT_IDTF
 			    && SEqual(Idtf, C4AUL_In))
 			{
@@ -1742,6 +1772,8 @@ int C4AulParse::Parse_Params(int iMaxCnt, const char * sWarn, C4AulFunc * pFunc)
 	case ATT_LDOTS:
 		// functions using ... always take as many parameters as possible
 		assert(Type == PREPARSER || Fn->ParCount == C4AUL_MAX_Par);
+		if (Type == PREPARSER && Fn->ParCount != C4AUL_MAX_Par && Config.Developer.ExtraWarnings)
+			Warn("'...' in function body forces function to take varargs");
 		Fn->ParCount = C4AUL_MAX_Par;
 		Shift();
 		// Push all unnamed parameters of the current function as parameters
@@ -1775,14 +1807,12 @@ int C4AulParse::Parse_Params(int iMaxCnt, const char * sWarn, C4AulFunc * pFunc)
 		}
 		++size;
 		// end of parameter list?
-		if (TokenType == ATT_COMMA)
-			Shift();
-		else if (TokenType == ATT_BCLOSE)
+		if (TokenType == ATT_BCLOSE)
 		{
 			Shift();
 			fDone = true;
 		}
-		else UnexpectedToken("',' or ')'");
+		else Match(ATT_COMMA, "',' or ')'");
 		break;
 	} while (!fDone);
 	// too many parameters?
@@ -1856,7 +1886,7 @@ void C4AulParse::Parse_PropList()
 		}
 		if (Fn->GetLastCode()->bccType == AB_CPROPLIST && Fn->GetLastCode()->Par.p->GetDef())
 		{
-			throw new C4AulParseError(this, "Can't use new on definitions yet.");
+			throw C4AulParseError(this, "Can't use new on definitions yet.");
 		}
 		++size;
 	}
@@ -1894,7 +1924,7 @@ C4Value C4AulParse::Parse_ConstPropList(const C4PropListStatic * parent, C4Strin
 {
 	C4Value v;
 	if (!Name)
-		throw new C4AulParseError(this, "a static proplist is not allowed to be anonymous");
+		throw C4AulParseError(this, "a static proplist is not allowed to be anonymous");
 	C4PropListStatic * p;
 	if (Type == PREPARSER)
 	{
@@ -1915,10 +1945,10 @@ C4Value C4AulParse::Parse_ConstPropList(const C4PropListStatic * parent, C4Strin
 		}
 		p = v.getPropList()->IsStatic();
 		if (!p)
-			throw new C4AulParseError(this, "internal error: constant proplist is not static");
+			throw C4AulParseError(this, "internal error: constant proplist is not static");
 		if (p->GetParent() != parent || p->GetParentKeyName() != Name)
 		{
-			throw new C4AulParseError(this, "internal error: constant proplist has the wrong parent");
+			throw C4AulParseError(this, "internal error: constant proplist has the wrong parent");
 		}
 		// In case of script reloads
 		p->Thaw();
@@ -2115,8 +2145,7 @@ void C4AulParse::Parse_ForEach()
 		Shift();
 	}
 	// get variable name
-	if (TokenType != ATT_IDTF)
-		UnexpectedToken("variable name");
+	Check(ATT_IDTF, "variable name");
 	if (Type == PREPARSER)
 	{
 		// insert variable
@@ -2125,7 +2154,7 @@ void C4AulParse::Parse_ForEach()
 	// search variable (fail if not found)
 	int iVarID = Fn->VarNamed.GetItemNr(Idtf);
 	if (iVarID < 0)
-		throw new C4AulParseError(this, "internal error: var definition: var not found in variable table");
+		throw C4AulParseError(this, "internal error: var definition: var not found in variable table");
 	Shift();
 	if (TokenType != ATT_IDTF || !SEqual(Idtf, C4AUL_In))
 		UnexpectedToken("'in'");
@@ -2198,7 +2227,7 @@ void C4AulParse::Parse_Expression(int iParentPrio)
 		{
 			// global func?
 			if (Fn->Owner == &::ScriptEngine)
-				throw new C4AulParseError(this, "using local variable in global function!");
+				throw C4AulParseError(this, "using local variable in global function!");
 			// insert variable by id
 			C4String * pKey = Strings.RegString(Idtf);
 			AddBCC(AB_LOCALN, (intptr_t) pKey);
@@ -2226,22 +2255,24 @@ void C4AulParse::Parse_Expression(int iParentPrio)
 		// function identifier: check special functions
 		else if (SEqual(Idtf, C4AUL_If))
 			// -> if is not a valid parameter
-			throw new C4AulParseError(this, "'if' may not be used as a parameter");
+			throw C4AulParseError(this, "'if' may not be used as a parameter");
 		else if (SEqual(Idtf, C4AUL_While))
 			// -> while is not a valid parameter
-			throw new C4AulParseError(this, "'while' may not be used as a parameter");
+			throw C4AulParseError(this, "'while' may not be used as a parameter");
 		else if (SEqual(Idtf, C4AUL_Else))
 			// -> else is not a valid parameter
-			throw new C4AulParseError(this, "misplaced 'else'");
+			throw C4AulParseError(this, "misplaced 'else'");
 		else if (SEqual(Idtf, C4AUL_For))
 			// -> for is not a valid parameter
-			throw new C4AulParseError(this, "'for' may not be used as a parameter");
+			throw C4AulParseError(this, "'for' may not be used as a parameter");
 		else if (SEqual(Idtf, C4AUL_Return))
 		{
 			Error("return may not be used as a parameter");
 		}
 		else if (SEqual(Idtf, C4AUL_Par))
 		{
+			if (Type == PREPARSER && Fn->ParCount != C4AUL_MAX_Par && Config.Developer.ExtraWarnings)
+				Warn("calling 'Par' in function body forces function to take varargs");
 			// functions using Par() always take as many parameters as possible
 			Fn->ParCount = C4AUL_MAX_Par;
 			// and for Par
@@ -2274,7 +2305,7 @@ void C4AulParse::Parse_Expression(int iParentPrio)
 			else
 				// not found? raise an error, if it's not a safe call
 				if (SEqual(Idtf, C4AUL_Inherited) && Type == PARSER)
-					throw new C4AulParseError(this, "inherited function not found (use _inherited to disable this message)");
+					throw C4AulParseError(this, "inherited function not found (use _inherited to disable this message)");
 				else
 				{
 					// otherwise, parse parameters, but discard them
@@ -2332,14 +2363,14 @@ void C4AulParse::Parse_Expression(int iParentPrio)
 				AddBCC(AB_CFUNCTION, reinterpret_cast<intptr_t>(val._getFunction()));
 				break;
 			default:
-				throw new C4AulParseError(this, FormatString("internal error: constant %s has unsupported type %d", Idtf, val.GetType()).getData());
+				throw C4AulParseError(this, FormatString("internal error: constant %s has unsupported type %d", Idtf, val.GetType()).getData());
 			}
 			Shift();
 		}
 		else
 		{
 			// identifier could not be resolved
-			throw new C4AulParseError(this, "unknown identifier: ", Idtf);
+			throw C4AulParseError(this, "unknown identifier: ", Idtf);
 		}
 		break;
 	case ATT_INT: // constant in cInt
@@ -2356,7 +2387,7 @@ void C4AulParse::Parse_Expression(int iParentPrio)
 		// postfix?
 		if (op->Postfix)
 			// oops. that's wrong
-			throw new C4AulParseError(this, "postfix operator without first expression");
+			throw C4AulParseError(this, "postfix operator without first expression");
 		Shift();
 		// generate code for the following expression
 		Parse_Expression(op->Priority);
@@ -2440,7 +2471,7 @@ void C4AulParse::Parse_Expression2(int iParentPrio)
 				// not found?
 				if (!postfixop->Identifier)
 				{
-					throw new C4AulParseError(this, "unexpected prefix operator: ", op->Identifier);
+					throw C4AulParseError(this, "unexpected prefix operator: ", op->Identifier);
 				}
 				// otherwise use the new-found correct postfix operator
 				op = postfixop;
@@ -2498,13 +2529,15 @@ void C4AulParse::Parse_Expression2(int iParentPrio)
 					Warn(FormatString("operator \"%s\" right side gets %s instead of %s", op->Identifier, GetC4VName(from), GetC4VName(to)).getData(), NULL);
 				}
 				// write byte code
-				AddBCC(op->Code, op - &C4ScriptOpMap[0]);
-				// write setter and mofidier
+				AddBCC(op->Code, 0);
+				// write setter and modifier
 				if (op->Changer)
 					{
 					AddBCC(Setter.bccType, Setter.Par.X);
-					if(op->ResultModifier != AB_ERR)
-						AddBCC(op->ResultModifier, op - &C4ScriptOpMap[0]);
+					// postfix ++ works by increasing, storing, then decreasing
+					// in case the result is thrown away, AddBCC will remove the decrease operation
+					if((op->Code == AB_Inc || op->Code == AB_Dec) && op->Postfix)
+						AddBCC(op->Code == AB_Inc ? AB_Dec : AB_Inc, 1);
 					}
 			}
 		}
@@ -2544,13 +2577,12 @@ void C4AulParse::Parse_Expression2(int iParentPrio)
 		break;
 	case ATT_DOT:
 		Shift();
-		if (TokenType != ATT_IDTF)
-			UnexpectedToken("Identifier");
+		Check(ATT_IDTF, "property name");
 		{
 			C4String * pKey = Strings.RegString(Idtf);
 			AddBCC(AB_PROP, (intptr_t) pKey);
-			Shift();
 		}
+		Shift();
 		break;
 	case ATT_CALL: case ATT_CALLFS:
 		{
@@ -2560,7 +2592,7 @@ void C4AulParse::Parse_Expression2(int iParentPrio)
 			Shift();
 			// expect identifier of called function now
 			if (TokenType != ATT_IDTF)
-				throw new C4AulParseError(this, "expecting func name after '->'");
+				throw C4AulParseError(this, "expecting func name after '->'");
 			if (Type == PARSER)
 			{
 				pName = ::Strings.RegString(Idtf);
@@ -2582,8 +2614,7 @@ void C4AulParse::Parse_Var()
 	while (1)
 	{
 		// get desired variable name
-		if (TokenType != ATT_IDTF)
-			UnexpectedToken("variable name");
+		Check(ATT_IDTF, "variable name");
 		if (Type == PREPARSER)
 		{
 			// insert variable
@@ -2592,7 +2623,7 @@ void C4AulParse::Parse_Var()
 		// search variable (fail if not found)
 		int iVarID = Fn->VarNamed.GetItemNr(Idtf);
 		if (iVarID < 0)
-			throw new C4AulParseError(this, "internal error: var definition: var not found in variable table");
+			throw C4AulParseError(this, "internal error: var definition: var not found in variable table");
 		Shift();
 		if(TokenType == ATT_SET)
 		{
@@ -2601,16 +2632,9 @@ void C4AulParse::Parse_Var()
 			Parse_Expression();
 			AddBCC(AB_POP_TO, 1 + iVarID - (iStack + Fn->VarNamed.iSize));
 		}
-		switch (TokenType)
-		{
-		case ATT_COMMA:
-			Shift();
-			break;
-		case ATT_SCOLON:
+		if (TokenType == ATT_SCOLON)
 			return;
-		default:
-			UnexpectedToken("',' or ';'");
-		}
+		Match(ATT_COMMA, "',' or ';'");
 	}
 }
 
@@ -2619,39 +2643,31 @@ void C4AulParse::Parse_Local()
 	Shift();
 	while (1)
 	{
+		Check(ATT_IDTF, "variable name");
 		if (Type == PREPARSER)
 		{
 			// get desired variable name
-			if (TokenType != ATT_IDTF)
-				UnexpectedToken("variable name");
 			// check: symbol already in use?
 			if (Host->GetPropList() && Host->GetPropList()->GetFunc(Idtf))
-				throw new C4AulParseError(this, "variable definition: name already in use");
+				throw C4AulParseError(this, "variable definition: name already in use");
 			// insert variable
 			Host->LocalNamed.AddName(Idtf);
 		}
 		char Name[C4AUL_MAX_Identifier] = ""; // current identifier
 		SCopy(Idtf, Name);
-		Match(ATT_IDTF);
+		Shift();
 		if (TokenType == ATT_SET)
 		{
 			if (!Host->GetPropList())
-				throw new C4AulParseError(this, "local variables can only be initialized on proplists");
+				throw C4AulParseError(this, "local variables can only be initialized on proplists");
 			Shift();
 			C4RefCntPointer<C4String> key = ::Strings.RegString(Name);
 			assert(Host->GetPropList()->IsStatic());
 			Parse_ConstExpression(Host->GetPropList()->IsStatic(), key);
 		}
-		switch (TokenType)
-		{
-		case ATT_COMMA:
-			Shift();
-			break;
-		case ATT_SCOLON:
+		if (TokenType == ATT_SCOLON)
 			return;
-		default:
-			UnexpectedToken("',' or ';'");
-		}
+		Match(ATT_COMMA, "',' or ';'");
 	}
 }
 
@@ -2666,11 +2682,10 @@ void C4AulParse::Parse_Static()
 	}
 	while (1)
 	{
+		Check(ATT_IDTF, "variable name");
 		if (Type == PREPARSER)
 		{
 			// get desired variable name
-			if (TokenType != ATT_IDTF)
-				UnexpectedToken("variable name");
 			// global variable definition
 			// check: symbol already in use?
 			if (Engine->GetPropList()->GetFunc(Idtf)) Error("function and variable with name %s", Idtf);
@@ -2681,17 +2696,10 @@ void C4AulParse::Parse_Static()
 				Engine->GlobalNamedNames.AddName(Idtf);
 			}
 		}
-		Match(ATT_IDTF);
-		switch (TokenType)
-		{
-		case ATT_COMMA:
-			Shift();
-			break;
-		case ATT_SCOLON:
+		Shift();
+		if (TokenType == ATT_SCOLON)
 			return;
-		default:
-			UnexpectedToken("',' or ';'");
-		}
+		Match(ATT_COMMA, "',' or ';'");
 	}
 }
 
@@ -2722,8 +2730,7 @@ C4Value C4AulParse::Parse_ConstExpression(C4PropListStatic * parent, C4String *
 		{
 			Shift();
 			// Create an array
-			if (Type == PARSER)
-				r.SetArray(new C4ValueArray());
+			r.SetArray(new C4ValueArray());
 			int size = 0;
 			bool fDone = false;
 			do
@@ -2734,8 +2741,7 @@ C4Value C4AulParse::Parse_ConstExpression(C4PropListStatic * parent, C4String *
 					// [] -> size 0, [*,] -> size 2, [*,*,] -> size 3
 					if (size > 0)
 					{
-						if (Type == PARSER)
-							r._getArray()->SetItem(size, C4VNull);
+						r._getArray()->SetItem(size, C4VNull);
 						++size;
 					}
 					fDone = true;
@@ -2744,18 +2750,14 @@ C4Value C4AulParse::Parse_ConstExpression(C4PropListStatic * parent, C4String *
 				case ATT_COMMA:
 				{
 					// got no parameter before a ","? then push nil
-					if (Type == PARSER)
-						r._getArray()->SetItem(size, C4VNull);
+					r._getArray()->SetItem(size, C4VNull);
 					Shift();
 					++size;
 					break;
 				}
 				default:
 				{
-					if (Type == PARSER)
-						r._getArray()->SetItem(size, Parse_ConstExpression(NULL, NULL));
-					else
-						Parse_ConstExpression(NULL, NULL);
+					r._getArray()->SetItem(size, Parse_ConstExpression(NULL, NULL));
 					++size;
 					if (TokenType == ATT_COMMA)
 						Shift();
@@ -2799,8 +2801,21 @@ C4Value C4AulParse::Parse_ConstExpression(C4PropListStatic * parent, C4String *
 	default:
 		UnexpectedToken("constant value");
 	}
-	// expect ',' (next global) or ';' (end of definition) now
 	Shift();
+	while (TokenType == ATT_DOT)
+	{
+		Shift();
+		Check(ATT_IDTF, "property name");
+		if (Type == PARSER)
+		{
+			C4String * k = ::Strings.FindString(Idtf);
+			if (!r.CheckConversion(C4V_PropList))
+				throw C4AulParseError(this, FormatString("proplist access: proplist expected, got %s", r.GetTypeName()).getData());
+			if (!k || !r._getPropList()->GetPropertyByS(k, &r))
+				r.Set0();
+		}
+		Shift();
+	}
 	if (TokenType == ATT_OPERATOR)
 	{
 		C4ScriptOpDef * op = &C4ScriptOpMap[cInt];
@@ -2834,40 +2849,29 @@ void C4AulParse::Parse_Const()
 	// get global constant definition(s)
 	while (1)
 	{
-		char Name[C4AUL_MAX_Identifier] = ""; // current identifier
+		Check(ATT_IDTF, "constant name");
 		// get desired variable name
-		if (TokenType != ATT_IDTF)
-			UnexpectedToken("constant name");
+		char Name[C4AUL_MAX_Identifier] = "";
 		SCopy(Idtf, Name);
 		// check func lists - functions of same name are not allowed
 		if (Engine->GetPropList()->GetFunc(Idtf))
 			Error("definition of constant hidden by function %s", Idtf);
 		if (Engine->GlobalNamedNames.GetItemNr(Idtf) != -1)
 			Error("constant and variable with name %s", Idtf);
-		Match(ATT_IDTF);
-		// expect '='
-		if (TokenType != ATT_SET)
-			UnexpectedToken("'='");
+		Shift();
+		Match(ATT_SET);
 		// expect value. Theoretically, something like C4AulScript::ExecOperator could be used here
 		// this would allow for definitions like "static const OCF_CrewMember = 1<<20"
 		// However, such stuff should better be generalized, so the preparser (and parser)
 		// can evaluate any constant expression, including functions with constant retval (e.g. Sqrt)
 		// So allow only simple constants for now.
-		Shift();
 
 		C4RefCntPointer<C4String> key = ::Strings.RegString(Name);
 		Parse_ConstExpression(NULL, key);
 
-		switch (TokenType)
-		{
-		case ATT_COMMA:
-			Shift();
-			break;
-		case ATT_SCOLON:
+		if (TokenType == ATT_SCOLON)
 			return;
-		default:
-			UnexpectedToken("',' or ';'");
-		}
+		Match(ATT_COMMA, "',' or ';'");
 	}
 }
 
@@ -2889,7 +2893,6 @@ void C4ScriptHost::CopyPropList(C4Set<C4Property> & from, C4PropListStatic * to)
 				C4AulScriptFunc * sf = prop->Value.getFunction()->SFunc();
 				if (sf)
 				{
-					//assert(sf->pOrgScript == *s);
 					C4AulScriptFunc *sfc;
 					if (sf->pOrgScript != this)
 						sfc = new C4AulScriptFunc(this, *sf);
